<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

use Joomla\Utilities\ArrayHelper;

/**
 * Entries controller.
 */
class RSDirectoryControllerEntries extends JControllerAdmin
{
	/**
	 * Constructor.
	 *
	 * @access public
	 *
	 * @param array $config An optional associative array of configuration settings.
	 */
	public function __construct( $config = array() )
	{
        parent::__construct($config);
            
        $this->registerTask('markAsUnpaid', 'markAsPaid');
    }
		
    /**
     * Method to get a model object, loading it if required.
     *
     * @access public
     * 
     * @param string $name The model name. Optional.
     * @param string $prefix The class prefix. Optional.
     * @param array $config Configuration array for model. Optional.
     */
    public function getModel( $name = 'Entry', $prefix = 'RSDirectoryModel', $config = array('ignore_request' => true) )
    {
        return parent::getModel($name, $prefix, $config);
    }
        
    /**
     * Method to process a batch process.
     *
     * @access public
     */
    public function batch()
    {
        // Check for request forgeries.
		$this->checkToken();
            
        $app = JFactory::getApplication();
            
        $data = $app->input->get( 'batch', array(), 'array' );
        $cid = $app->input->get( 'cid', array(), 'array' );
            
        // Get an instance of the Entries model.
        $entries_model = RSDirectoryModel::getInstance('Entries');
            
        // Check for validation errors.
        if ( !$entries_model->batchValidate($data, $cid) )
        {
            // Get the validation messages.
			$errors = $entries_model->getErrors();
                
			// Push up to three validation messages out to the user.
			for ( $i = 0, $n = count($errors); $i < $n && $i < 3; $i++ )
			{
				if ($errors[$i] instanceof Exception)
				{
					$app->enqueueMessage( $errors[$i]->getMessage(), 'warning');
				}
				else
				{
					$app->enqueueMessage($errors[$i], 'warning');
				}
			}
                
            $app->redirect( JUri::getInstance()->toString() );
        }
            
        // Attempt to process the batch.
        if ( !$entries_model->batchProcess($data, $cid) )
        {
            $message = JText::sprintf( 'JERROR_SAVE_FAILED', $model->getError() );
                
            $app->enqueueMessage($message, 'error');
            $app->redirect( JUri::getInstance()->toString() );
        }
            
        $app->enqueueMessage( JText::_('JLIB_APPLICATION_SUCCESS_BATCH') );
        $app->redirect( JUri::getInstance()->toString() );
    }
		
	/**
	 * Method to mark a list of items as paid/unpaid.
	 * 
	 * @access public
	 */
	public function markAsPaid()
	{
		// Check for request forgeries
		$this->checkToken();
			
		// Get items to publish from the request.
		$cid = JFactory::getApplication()->input->get( 'cid', array(), 'array' );
		$data = array('markAsPaid' => 1, 'markAsUnpaid' => 0);
		$task = $this->getTask();
		$value = ArrayHelper::getValue($data, $task, 0, 'int');
			
		if ( empty($cid) )
		{
			JLog::add( JText::_($this->text_prefix . '_NO_ITEM_SELECTED'), JLog::WARNING, 'jerror' );
		}
		else
		{
			// Get the model.
			$model = $this->getModel();
				
			// Make sure the item ids are integers.
			$cid = array_map('intval', $cid);
				
			// Mark the items as paid/unpaid.
			try
			{
				$model->markAsPaid($cid, $value);
					
				if ($value == 1)
				{
					$ntext = 'COM_RSDIRECTORY_N_ITEMS_MARKED_AS_PAID';
				}
				else if ($value == 0)
				{
					$ntext = 'COM_RSDIRECTORY_N_ITEMS_MARKED_AS_UNPAID';
				}
					
				$this->setMessage( JText::plural( $ntext, count($cid) ) );
			}
			catch (Exception $e)
			{
				$this->setMessage( JText::_('JLIB_DATABASE_ERROR_ANCESTOR_NODES_LOWER_STATE'), 'error' );
			}
		}
			
		$extension = $this->input->get('extension');
		$extensionURL = ($extension) ? '&extension=' . $extension : '';
		$this->setRedirect(JRoute::_('index.php?option=' . $this->option . '&view=' . $this->view_list . $extensionURL, false));
	}

	public function writeCsv()
	{
		$model = $this->getModel('Entries');
		$input = JFactory::getApplication()->input;

		try
		{
			$form_id = $input->getInt('form_id');
			if (empty($form_id))
			{
				throw new Exception(JText::_('COM_RSDIRECTORY_NO_FORM_SELECTED'));
			}
			$response = $model->writeCSV($form_id, $input->getInt('start'), $input->get('filehash'));

			$this->showResponse(true, $response);
		}
		catch (Exception $e)
		{
			$this->showResponse(false, $e->getMessage());
		}
	}

	public function exportCsv()
	{
		require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/export.php';

		$app      = JFactory::getApplication();
		$filename = JText::_('COM_RSDIRECTORY_ENTRIES_EXPORT_FILENAME');
		$fileHash = $app->input->get('filehash');

		RsdirectoryExport::buildCSVHeaders($filename);

		echo RsdirectoryExport::getCSV($fileHash);

		$app->close();
	}

	protected function showResponse($success, $data=null)
	{
		$app 		= JFactory::getApplication();
		$document 	= JFactory::getDocument();

		// set JSON encoding
		$document->setMimeEncoding('application/json');

		// compute the response
		$response = new stdClass();
		$response->success = $success;
		if ($data)
		{
			$response->response = $data;
		}

		// show the response
		echo json_encode($response);

		// close
		$app->close();
	}
}