<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * RSDirectory! Configuration helper.
 */
class RSDirectoryConfig
{
    /**
     * The class instance.
     *
     * @var RSDirectoryConfig
     *
     * @access private
     *
     * @static
     */
    private static $instance;

    /**
     * The configuration object.
     *
     * @var object
     *
     * @access private
     */
    private $config;

    /**
     * @var JDatabaseDriver
     */
    private $db;

    /**
     * The class constructor.
     *
     * @access private
     */
    private function __construct()
    {
        $this->db = JFactory::getDbo();
        $this->load();
    }

    /**
     * Get the class instance.
     *
     * @access public
     *
     * @static
     *
     * @return RSDirectoryConfig
     */
    public static function getInstance()
    {
        if ( !isset(self::$instance) )
        {
            self::$instance = new RSDirectoryConfig;
        }

        return self::$instance;
    }

    /**
     * Load the configuration from the database.
     *
     * @access private
     */
    private function load()
    {
        $db = &$this->db;

        $query = $db->getQuery(true)
            ->select('*')
            ->from( $db->qn('#__rsdirectory_config') );

        $db->setQuery($query);

        $config = new stdClass();

        if ( $results = $db->loadObjectList() )
        {
            foreach ($results as $result)
            {
                $config->{$result->name} = $result->value;
            }
        }

       if (!empty($config->use_joomla_email_configuration)) {
            // Get the Joomla! config object.
            $jconfig = JFactory::getConfig();

            // Set the default From Email
			$exists_email = isset($config->from_email) ? true : false;
            $config->from_email = $jconfig->get('mailfrom');
			
			if (is_null($config->from_email) && $exists_email) {
				$config->from_email = '';
			}

            // Set the default From Name
			$exists_from_name = isset($config->from_name) ? true : false;
            $config->from_name = $jconfig->get('fromname');
			
			if (is_null($config->from_name) && $exists_from_name) {
				$config->from_name = '';
			}
        }

        $this->config = $config;
    }

    /**
     * Reload the configuration from the database.
     *
     * @access public
     */
    public function reload()
    {
        $this->load();
    }

    /**
     * Get settings keys.
     *
     * @access public
     *
     * @return array
     */
    public function getKeys()
    {
        return array_keys( (array)$this->config);
    }

    /**
     * Get configuration object.
     *
     * @access public
     *
     * @return array
     */
    public function getData()
    {
        return $this->config;
    }

    /**
     * Get setting value.
     *
     * @access public
     *
     * @param string $key
     * @param mixed $default
     * @param bool $explode
     *
     * @return mixed
     */
    public function get($key, $default = false, $explode = false)
    {
        if ( isset($this->config->$key) )
        {
            if ($key == 'zoom_crop') {
                return explode(',', $this->config->$key);
            }

            return $explode ? self::explode($this->config->$key) : $this->config->$key;
        }

        return $default;
    }

    /**
     * Set setting value.
     *
     * @access public
     *
     * @param string $key
     * @param string $value
     *
     * @return bool
     */
    public function set($key, $value)
    {
        $db = &$this->db;

        if ($key == 'zoom_crop') {
            $value = implode(',', $value);
        }

        // Array are converted to strings here.
        if ( is_array($value) )
        {
            $value = implode("\n", $value);
        }

        // Check if the value already exists.
        if ( isset($this->config->$key) )
        {
			 $images_related = array(
                'mapicon_thumbnail_width',
                'mapicon_thumbnail_height',
                'small_thumbnail_width',
                'small_thumbnail_height',
                'big_thumbnail_width',
                'big_thumbnail_height',
                'normal_thumbnail_width',
                'normal_thumbnail_height',
                'zoom_crop',
            );

            if (in_array($key, $images_related)) {
                $this->removeImagesCache($key, $value, $this->config->$key);
            }
			
            // Update the existing value.
            $query = $db->getQuery(true)
                ->update( $db->qn('#__rsdirectory_config') )
                ->set( $db->qn('value') . '=' . $db->q($value) )
                ->where( $db->qn('name') . '=' . $db->q($key) );
        }
        else
        {
            // Insert new value.
            $query = $db->getQuery(true)
                ->insert( $db->qn('#__rsdirectory_config') )
                ->set( $db->qn('name') . '=' . $db->q($key) . ', ' . $db->qn('value') . '=' . $db->q($value) );
        }

        $db->setQuery($query);

        // Refresh our value.
        $this->config->$key = $value;

        // Execute the query.
        return $db->execute();
    }
	
	protected function removeImagesCache($setting, $new_value, $old_value) {
        // continue if only the new differs from the old
        if ($new_value == $old_value) {
            return;
        }

        switch ($setting) {
            case 'mapicon_thumbnail_width':
            case 'mapicon_thumbnail_height':
            case 'small_thumbnail_width':
            case 'small_thumbnail_height':
            case 'big_thumbnail_width':
            case 'big_thumbnail_height':
            case 'normal_thumbnail_width':
            case 'normal_thumbnail_height':
                // lets see if this is a height o width
                $thumb_type = explode('_', $setting);
                $axis = array_pop($thumb_type);

                // rebuild the setting without the axis
                $thumb_type = implode('_', $thumb_type);

                $w = 0;
                $h = 0;
                if ($axis == 'width') {
                    $w = (int)$new_value;
                    $h = (int)$this->config->{$thumb_type . '_height'};
                } else if ($axis == 'height') {
                    $h = (int)$new_value;
                    $w = (int)$this->config->{$thumb_type . '_width'};
                }
				
				// if the width and height are set delete the proper folder
                if ($w != 0 && $h != 0) {
                    $path = JPATH_SITE . "/components/com_rsdirectory/files/cache/{$w}x{$h}";
                    if (file_exists($path) && is_dir($path)) {
                        JFolder::delete($path);
                    }
                }
            break;

            case 'zoom_crop':
                $new_values = explode(',', $new_value);
                $old_values = explode(',', $old_value);

                $diff_new = array_diff($new_values, $old_values);
                $diff_old = array_diff($old_values, $new_values);
				
				// all changes
                $all_diff = array_merge($diff_new, $diff_old);

                if (!empty($all_diff)) {
                    foreach ($all_diff as $type) {
                        $w = (int)$this->config->{$type . '_thumbnail_width'};
                        $h = (int)$this->config->{$type . '_thumbnail_height'};
						
                        $path = JPATH_SITE . "/components/com_rsdirectory/files/cache/{$w}x{$h}";
                        if (file_exists($path) && is_dir($path)) {
                            JFolder::delete($path);
                        }
                    }
                }
            break;
        }
    }

    /**
     * Explode a new line delimited string into an array
     *
     * @param $string
     * @return array
     */
    protected static function explode($string)
    {
        $string = str_replace( array("\r\n", "\r"), "\n", $string );
        return explode("\n", $string);
    }
}