<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access'); 

/**
 * RSDirectory! Credits Helper class.
 */
abstract class RSDirectoryCredits
{
	/**
     * Check if an user has enough credits.
     *
     * @access public
     *
     * @static
     * 
     * @param int $credits
     * @param mixed $user_id
     * 
     * @return bool
     */
    public static function checkUserCredits($credits, $user_id = null)
    {
		if (!$credits)
			return true;
			
        if ($user_id === null)
        {
            $user = JFactory::getUser();
            $user_id = $user->id;
        }

        // call the getUserCredits function so that they are default credits they supposed to be added
        self::getUserCredits($user_id, true);
            
        // Get DBO.
        $db = JFactory::getDbo();
            
        $query = $db->getQuery(true)
               ->select( $db->qn('user_id') )
               ->from( $db->qn('#__rsdirectory_users') )
               ->where( $db->qn('user_id') . ' = ' . $db->q($user_id) )
               ->where( '(' . $db->qn('credits') . ' >= ' . $db->q($credits) . ' OR ' . $db->qn('unlimited_credits') . ' = ' . $db->q(1) . ')' );
               
        $db->setQuery($query);
            
        return (bool)$db->loadResult();
    }
        
    /**
     * Get an user's credits amount.
     *
     * @access public
     *
     * @static
     *
     * @param mixed $user_id
     * @param bool $add_default_value
     *
     * @return mixed
     */
    public static function getUserCredits($user_id = null, $add_default_value = false)
    {

        if ($user_id === null)
        {
            $user = JFactory::getUser();
            $user_id = $user->id;
        }

        // keep the default value (initial value)
        $default_value = (int) RSDirectoryHelper::checkUserPermission('initial_credits', $user_id);
            
        // Get DBO.
        $db = JFactory::getDbo();
            
        $query = $db->getQuery(true)
               ->select('*')
               ->from( $db->qn('#__rsdirectory_users') )
               ->where( $db->qn('user_id') . ' =' . $db->q($user_id) );
               
        $db->setQuery($query);
            
        $item = $db->loadObject();
            
        if ($item)
        {
            return $item->unlimited_credits ? 'unlimited' : $item->credits;
        }
        else
        {
            if ($add_default_value && $default_value > 0)
            {
                self::addUserCredits($user_id, 0, true);
            }
        }

            
        return $default_value;
    }
        
    /**
     * Get the total amount an user spent.
     *
     * @access public
     *
     * @static
     *
     * @param mixed $user_id
     *
     * @return int
     */
    public static function getUserSpentCredits($user_id = null)
    {
        if ($user_id === null)
        {
            $user = JFactory::getUser();
            $user_id = $user->id;
        }
            
        // Get DBO.
        $db = JFactory::getDbo();
            
        $query = $db->getQuery(true)
               ->select( 'SUM(' . $db->qn('credits') . ')' )
               ->from( $db->qn('#__rsdirectory_entries_credits') )
               ->group( $db->qn('user_id') )
               ->where( $db->qn('user_id') . ' = ' . $db->q($user_id) )
               ->where( $db->qn('free') . ' = ' . $db->q(0) )
               ->where( $db->qn('paid') . ' = ' . $db->q(1) );
                
        $db->setQuery($query);
            
        return $db->loadResult();
    }
        
    /**
     * Calculate the sum of the entry credits objects.
     *
     * @access public
     *
     * @static
     *
     * @param array $entry_credits_objects
     * 
     * @return mixed
     */
    public static function calculateCredits($entry_credits_objects)
    {
        if ( !is_array($entry_credits_objects) || empty($entry_credits_objects) )
            return false;
            
        $sum = 0 ;
            
        foreach ($entry_credits_objects as $entry_credits_object)
        {
            $sum += $entry_credits_object->credits;
        }

        if ($sum  < 0)
        {
            $sum = 0;
        }
            
        return $sum;
    }
        
    /**
     * Add entry credits objects.
     *
     * Be sure to check if the user has enough credits before trying to charge him.
     *
     * @access public
     *
     * @static
     * 
     * @param int $entry_id
     * @param array $objects
     * @param int $user_id
     * @param int $free
     * @param int $paid
     * 
     * @return bool
     */
    public static function addEntryCreditsObjects($entry_id, $objects, $user_id = 0, $free = 0, $paid = 1)
    {
        if ( !$entry_id || !is_numeric($entry_id) || !$objects || !is_array($objects) )
            return false;
            
            
        if (!$user_id)
        {
            $user = JFactory::getUser();
            $user_id = $user->id;
        }
            
        // Exit the function if the user id is not numeric.
        if ( !$user_id || !is_numeric($user_id) )
            return false;
            
        // Get DBO.
        $db = JFactory::getDbo();
            
        $query = $db->getQuery(true)
               ->select( $db->qn('unlimited_credits') )
               ->from( $db->qn('#__rsdirectory_users') )
               ->where( $db->qn('user_id') . ' = ' . $db->q($user_id) );
               
        $db->setQuery($query);
        $unlimited = (int) $db->loadResult();
            
        // Initialize the total cost.    
        $total_credits = 0;
            
        $columns = array(
            'entry_id',
            'user_id',
            'object_type',
            'object_id',
            'credits',
            'is_diff',
            'free',
            'unlimited_credits',
            'paid',
            'created_time',
        );
            
        $query = $db->getQuery(true)
               ->insert( $db->qn('#__rsdirectory_entries_credits') )
               ->columns( $db->qn($columns) );

                
        foreach ($objects as $key => $object)
        {
            $values = array(
                $db->q($entry_id),
                $db->q($user_id),
                $db->q($object->object_type),
                $db->q($object->object_id),
                $db->q($object->credits),
                $db->q((!empty($object->is_diff) ? 1 : 0)),
                $db->q($free),
                $db->q($unlimited),
                $db->q($paid),
                $db->q( JFactory::getDate()->toSql() ),
            );
                
            $query->values( implode(',', $values) );
                
            $total_credits += $object->credits;
        }
            
        $db->setQuery($query);
        $db->execute();
            
        if (!$free && !$unlimited && $paid)
        {
            // Charge the user's credits.
			self::chargeUserCredits($total_credits, $user_id);
        }
            
        return true;
    }
	
	/**
     * Charge the user's credits.
    */
	public static function chargeUserCredits($total_credits, $user_id) {
		 // Get DBO.
        $db = JFactory::getDbo();
		
		$query = $db->getQuery(true)
			   ->update( $db->qn('#__rsdirectory_users') )
			   ->set( $db->qn('credits') . ' = ' . $db->qn('credits') . ' - ' . $db->q($total_credits) )
			   ->where( $db->qn('user_id') . ' = ' . $db->q($user_id) );
			
		$db->setQuery($query);
		$db->execute();
	}
        
    /**
     * Return the credits spent on an entry to the user.
     * 
     * Note that the credits will be returned only if the entry was new (never published).
     * 
     * @access public
     *
     * @static
     *
     * @param int $entry_id
     */
    public static function returnEntryCredits($entry_id)
    {
        $entry = RSDirectoryHelper::getEntry($entry_id);
            
        if (!$entry || !$entry->new)
            return;
            
        $items = self::getEntryCreditsObjectList($entry_id);
            
        if (!$items)
            return;
            
        $total = 0;
            
        foreach ($items as $item)
        {
            if (!$item->free && $item->paid && !$item->unlimited_credits)
            {
                $total += $item->credits;
            }
        }
            
        if ($total)
        {
            self::addUserCredits($entry->user_id, $total);
        }
            
        $db = JFactory::getDbo();
            
        $query = $db->getQuery(true)
               ->delete( $db->qn('#__rsdirectory_entries_credits') )
               ->where( $db->qn('entry_id') . ' = ' . $db->q($entry_id) );
               
        $db->setQuery($query);
        $db->execute();
    }
        
    /**
     * Add a number of credits to an user.
     * 
     * @access public
     *
     * @static
     * 
     * @param int $user_id
     * @param int $credits Number of credits.
     * @param bool $ignore_zero_credits use unlimited with zero.
     * 
     * @return bool
     */
    public static function addUserCredits($user_id, $credits, $ignore_zero_credits = false)
    {
        // keep the default value (initial value)
        $default_value = (int) RSDirectoryHelper::checkUserPermission('initial_credits', $user_id);

        // Get DBO.
        $db = JFactory::getDbo();
            
        $query = $db->getQuery(true)
               ->select('*')
               ->from( $db->qn('#__rsdirectory_users') )
               ->where( $db->qn('user_id') . ' = ' . $db->q($user_id) );
                
        $db->setQuery($query);
            
        $result = $db->loadObject();
            
        $data = (object)array(
            'user_id' => $user_id,
            'credits' => empty($result) ? $default_value + $credits : $result->credits + $credits,
        );
            
        if (!$credits && !$ignore_zero_credits)
        {
            $data->unlimited_credits = 1;
        }
            
        // Insert credits.
        if ( empty($result) )
        {
            return $db->insertObject('#__rsdirectory_users', $data, 'user_id');
        }

        // Update credits.
        return $db->updateObject('#__rsdirectory_users', $data, 'user_id');
    }
	
	/**
     * Get entry credits object list.
     *
     * @access public
     *
     * @static
     * 
     * @param int $entry_id
     * 
     * @return mixed
     */
    public static function getEntryCreditsObjectList($entry_id)
    {
        // Get DBO.
        $db = JFactory::getDbo();
            
        $query = $db->getQuery(true)
               ->select('*')
               ->from( $db->qn('#__rsdirectory_entries_credits') )
               ->where( $db->qn('entry_id') . ' = ' . $db->q($entry_id) );
               
        $db->setQuery($query);
		
		$entry_credits = $db->loadObjectList();
            
        return $entry_credits;
    }
		
	/**
	 * Check if an entry has unpaid credits objects.
	 *
	 * @access public
	 *
	 * @static
	 *
	 * @param int $entry_id
	 *
	 * @return bool
	 */
	public static function hasUnpaidEntryCredits($entry_id)
	{
		// Get DBO.
        $db = JFactory::getDbo();
			
		$query = $db->getQuery(true)
		       ->select( $db->qn('id') )
			   ->from( $db->qn('#__rsdirectory_entries_credits') )
			   ->where( $db->qn('entry_id') . ' = ' . $db->q($entry_id) )
			   ->where( $db->qn('paid') . ' = ' . $db->q(0) );
			   
		$db->setQuery($query, 0, 1);
			
		return $db->loadResult();
	}
		
	/**
	 * Get the sum of unpaid entry credits.
	 *
	 * @access public
	 *
	 * @static
	 *
	 * @param int $entry_id
	 *
	 * @return int
	 */
	public static function getUnpaidEntryCreditsSum($entry_id)
	{
		// Get DBO.
        $db = JFactory::getDbo();
			
		$query = $db->getQuery(true)
		       ->select( 'SUM(' . $db->qn('credits') . ')' )
			   ->from( $db->qn('#__rsdirectory_entries_credits') )
			   ->where( $db->qn('entry_id') . ' = ' . $db->q($entry_id) )
			   ->where( $db->qn('free') . ' = ' . $db->q(0) )
			   ->where( $db->qn('paid') . ' = ' . $db->q(0) );
			   
		$db->setQuery($query);
			
		return (int)$db->loadResult();
	}
		
    /**
     * Return the entry credits objects that were not already paid.
     *
     * @access public
     *
     * @static
     *
     * @param array $entry_credits_objects_new
     * @param int $entry_id 
     *
     * @return array
     */
    public static function getEntryCreditsDiff($entry_credits_objects_new, $entry_id)
    {
        if ( !$entry_credits_objects_new || !is_array($entry_credits_objects_new) )
            return array();
            
        // Get the old entry credits objects array.
        $entry_credits_objects_old = self::getEntryCreditsObjectList($entry_id);
		
		// Get the renew type option value
		$renew_types = (int) RSDirectoryConfig::getInstance()->get('renew_types', 0);
		
		// check if entry is expired
		$is_expired = RSDirectoryHelper::isEntryExpired($entry_id);

        // check if the user can moderate the entry
        $can_moderate = !JFactory::getApplication()->isClient('site') || RSDirectoryHelper::checkUserPermission('can_edit_all_entries');
		
		
        if (!$entry_credits_objects_old)
            return $entry_credits_objects_new;
		
		if (!$is_expired || ($is_expired && $renew_types == 0)) {
			foreach ($entry_credits_objects_new as $i => $entry_credits_object_new)
			{
				foreach ($entry_credits_objects_old as $j => $entry_credits_object_old)
				{
					// Remove the object if it exists in the database.
					if (
						$entry_credits_object_new->object_type == 'form_field' &&
						$entry_credits_object_old->object_type == 'form_field' &&
						$entry_credits_object_new->object_id == $entry_credits_object_old->object_id &&
                        empty($entry_credits_object_new->is_diff)
					)
					{
                        $field =  RSDirectoryHelper::getField($entry_credits_object_new->object_id);
                        if ($field->name == 'promoted' && !$is_expired && $renew_types > 0 && !$can_moderate) {
                           continue;
                        }
						unset($entry_credits_objects_new[$i]);
					}
				}
			}
		}
		
        return $entry_credits_objects_new;
    }
        
    /**
     * Do a little cleaning to the entry credits objects array.
     *
     * @access public
     *
     * @static
     *
     * @param array $entry_credits_objects
     *
     * @return array
     */
    public static function cleanEntryCredits($entry_credits_objects)
    {
        // Initialize the results array.
        $results = array();
            
        if ( $entry_credits_objects && is_array($entry_credits_objects) )
        {
            foreach ($entry_credits_objects as $entry_credits_object)
            {
                // Remove duplicates.
                if ($entry_credits_object->object_type != 'uploaded_file')
                {
                    foreach ($results as $result)
                    {
                        if ($entry_credits_object->object_type == $result->object_type && $entry_credits_object->object_id == $result->object_id)
                            continue 2;
                    }
                }
                    
                if ($entry_credits_object->credits)
                {
                    $results[] = $entry_credits_object;
                }
            }
        }
            
        return $results;
    }
		
	/**
	 * Get the minimum required credit package for the specified entry.
	 *
	 * @access public
	 *
	 * @static
	 *
	 * @param int $entry_id
	 *
	 * @return mixed
	 */
	public static function getEntryMinimumRequiredCreditPackage($entry_id)
	{
		$user_credits = self::getUserCredits();
		$unpaid_entry_credits = self::getUnpaidEntryCreditsSum($entry_id);
			
		$required_credits = $user_credits === 'unlimited' || $user_credits >= $unpaid_entry_credits ? 0 : $unpaid_entry_credits - $user_credits;
			
		if (!$required_credits)
			return false;
			
		$price = $required_credits * (float)RSDirectoryConfig::getInstance()->get('credit_cost');
			
		return (object)array(
			'id' => 'minimum',
			'title' => JText::_('COM_RSDIRECTORY_MINIMUM_REQUIRED_CREDITS_TITLE'),
			'credits' => $required_credits,
			'price' => $price,
			'description' => JText::_('COM_RSDIRECTORY_MINIMUM_REQUIRED_CREDITS_DESC'),
			'class' => 'minimum-required-credits',
		);
	}
		
	/**
	 * Method to get entry summary
	 *
	 * @access public
	 *
	 * @static
	 *
	 * @param int $entry_id
	 * @param bool $substract_user_credits
	 *
	 * @return mixed
	 */
	public static function getEntrySummary($entry_id, $substract_user_credits = true)
	{
		if (!$entry_id)
			return false;
			
        // Get DBO.
		$db = JFactory::getDbo();
			
		$select = array(
			$db->qn('ec') . '.*',
			$db->qn('f.name', 'field_name'),
		);
			
		$query = $db->getQuery(true)
			   ->select($select)
			   ->from( $db->qn('#__rsdirectory_entries_credits', 'ec') )
			   ->leftJoin( $db->qn('#__rsdirectory_fields', 'f') . ' ON ' . $db->qn('ec.object_id') . ' = ' . $db->qn('f.id') . ' AND (' . $db->qn('ec.object_type') . ' = ' . $db->q('form_field').' OR ' . $db->qn('ec.object_type') . ' = ' . $db->q('uploaded_file').')' )
			   ->where( $db->qn('ec.entry_id') . ' = ' . $db->q($entry_id) )
			   ->where( $db->qn('ec.free') . ' = ' . $db->q(0) )
			   ->where( $db->qn('ec.paid') . ' = ' . $db->q(0) );
			   
		$db->setQuery($query);
			
		$items = $db->loadObjectList();
			
		if (!$items)
			return false;
			
		$results = array();
		$total = 0;
			
		 // hold the fields names, if there are uploaded_file thy may came from the same field
        static $field_names = array();

		foreach ($items as $item)
		{
            if (!isset($field_names[$item->object_id])) {
                $field_name = null;

                // check to see if the field has properties
                if( $field_data = RSDirectoryHelper::getField($item->object_id)) {
                    $field_name = $field_data->properties->get('form_caption', null);
                }

                // if no caption has been found use the name that is provided
                if (empty($field_name) && $item->field_name) {
                    $field_name = $item->field_name;
                }

                $field_names[$item->object_id] = $field_name;
            }

            $results[] = (object)array(
				'text' => JText::_( 'COM_RSDIRECTORY_CREDIT_OBJECT_TYPE_' . strtoupper($item->object_type) ) . ($field_names[$item->object_id] ? " (".$field_names[$item->object_id].")" : ''),
				'credits' => $item->credits,
			);
				
			$total += $item->credits;
		}
			
		if ($substract_user_credits)
		{
			$user_credits = RSDirectoryCredits::getUserCredits();
				
			$results[] = (object)array(
				'text' => JText::_('COM_RSDIRECTORY_CURRENT_CREDITS'),
				'credits' => $user_credits ? "-$user_credits" : 0,
			);
				
			$total -= $user_credits;
		}
			
		$results[] = (object)array(
			'is_total' => 1,
			'text' => JText::_('COM_RSDIRECTORY_TOTAL'),
			'credits' => $total,
		);
			
		return $results;
	}
		
	/**
     * Method to check if there's anything that requires credits.
     *
     * @access public
     *
     * @static
     *
     * @return bool
     */   
    public static function requiresCredits($form_id = null)
    {
        $db = JFactory::getDbo();
			
		// Check if there are fields that require credits.
		$query = $db->getQuery(true)
               ->select('COUNT(*)')
               ->from( $db->qn('#__rsdirectory_fields_properties', 'fp') )
               ->innerJoin( $db->qn('#__rsdirectory_fields', 'f') . ' ON ' . $db->qn('fp.field_id') . ' = ' . $db->qn('f.id') )
               ->innerJoin( $db->qn('#__rsdirectory_forms_fields', 'ff') . ' ON ' . $db->qn('f.id') . ' = ' . $db->qn('ff.field_id') )
                 ->where(
                    '('.
					'(' . $db->qn('fp.property_name') . ' = ' . $db->q('credits') . ' AND '. $db->qn('value') . ' > ' . $db->q(0) . ')' . ' OR ' .
					'(' . $db->qn('fp.property_name') . ' = ' . $db->q('credits_per_file') . ' AND '. $db->qn('value') . ' > ' . $db->q(0) . ')' . 'OR' .
                    '( '. $db->qn('fp.property_name') . ' = ' . $db->q('items') .' AND '. $db->qn('f.field_type_id') .' = '.$db->q(7).' AND '. $db->qn('fp.value').' REGEXP '.$db->q('.*"credits";i:[1-9].*').' )'. 'OR' .
					'( '. $db->qn('fp.property_name') . ' = ' . $db->q('items') .' AND '. $db->qn('f.field_type_id') .' = '.$db->q(8).' AND '. $db->qn('fp.value').' REGEXP '.$db->q('.*"credits";i:[1-9].*').' )'. 'OR' .
					'( '. $db->qn('fp.property_name') . ' = ' . $db->q('daily_cost') .' AND '. $db->qn('f.field_type_id') .' = '.$db->q(8).' AND '. $db->qn('fp.value') . ' > ' . $db->q(0) . ' )'. 'OR' .
                    '( '. $db->qn('fp.property_name') . ' = ' . $db->q('items') .' AND '. $db->qn('fp.value').' REGEXP '.$db->q('.*\\[c-[0-9].*\\].*').' )'.
                    ')'
				)
               ->where( $db->qn('f.published') . ' = ' . $db->q(1) );
			   
		if (!is_null($form_id)) {
			$query->where( $db->qn('ff.form_id') . ' = ' . $db->q($form_id) );
		}
               
        $db->setQuery($query);
        $result = (bool)$db->loadResult();
        
		if (is_null($form_id)) {
			// Check if there are unpaid entries.
			$query = $db->getQuery(true)
				   ->select('COUNT(*)')
				   ->from( $db->qn('#__rsdirectory_entries') )
				   ->where( $db->qn('paid') . ' = ' . $db->q(0) );
				   
			$db->setQuery($query);
			
			return $result || (bool)$db->loadResult();
		} else {
			return $result;
		}
        
    }

    public static function getLastPaidCreditField($field_type = '',$entry_id = null, $form_id = null, $form_fields = null) {
        if (empty($field_type) || is_null($form_id) || is_null($entry_id)) {
            return false;
        }

        if (is_null($form_fields)) {
            $form_fields =  RSDirectoryHelper::getFormFields($form_id);
        }

        $last_field_credit = false;

        $db = JFactory::getDbo();
        foreach($form_fields as $form_field) {
            if ($form_field->field_type == $field_type) {

                $query = $db->getQuery(true);
                $query->select('*')
                      ->from($db->qn('#__rsdirectory_entries_credits', 'ec'));

                // if the field is a publishing_period or renewal, the object_id is 0 so we must search otherwise
                if (in_array($field_type, array('publishing_period', 'renewal'))) {
                    $query->where($db->qn('ec.object_id') . ' = ' . $db->q(0));
                    $query->where($db->qn('ec.object_type') . ' = ' . $db->q($field_type));
                } else {
                    $query->where($db->qn('ec.object_id') . ' = ' . $db->q($form_field->id));
                }

                $query->where($db->qn('ec.entry_id') . ' = ' . $db->q($entry_id));
                $query->where($db->qn('ec.paid') . ' = ' . $db->q(1));
                $query->order($db->qn('ec.created_time') . ' DESC');

                $db->setQuery($query, 0 , 1);

                $last_field_credit = $db->loadObject();

                if(is_null($last_field_credit)) {
                    $last_field_credit = false;
                    break;
                }

                // additionally add the field properties
                $properties = empty($form_field->properties) ? new RSDirectoryFormFieldProperties : $form_field->properties;
                $last_field_credit->properties = $properties;

                break;
            }
        }

        return $last_field_credit;
    }
}