<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * Generate field value.
 */
class RSDirectoryField
{
    /**
     * The form field.
     *
     * @var object
     * 
     * @access protected
     */
    protected $form_field;
        
    /**
     * Entry data.
     *
     * @var object
     * 
     * @access protected
     */
    protected $entry;
        
    /**
     * The files list of a fileupload field.
     * 
     * @var array
     *
     * @access protected
     */
    protected $files_list;
        
    /**
     * Set to true when we call the setFilesList function.
     *
     * @var bool
     *
     * @access protected
     */
    protected $set_files_list;
        
    /**
     * The class constructor.
     *
     * @access public
     * 
     * @param object $form_field
     * @param object $entry
     */
    public function __construct($form_field, $entry)
    {
        $this->form_field = $form_field;
        $this->entry = $entry;
    }
        
    /**
     * Get RSDirectoryFormField instance.
     *
     * @access public
     * 
     * @static
     * 
     * @param object $form_field
     * @param object $entry
     * @param object $form
     * 
     * @return RSDirectoryField
     */
    public static function getInstance($form_field, $entry)
    {
        $rsdirectoryfield = new RSDirectoryField($form_field, $entry);
            
        return $rsdirectoryfield;
    }
        
    /**
     * Set files list.
     *
     * @access public
     *
     * @param array $files_list
     *
     * @return RSDirectoryField
     */
    public function setFilesList($files_list)
    {
        $this->files_list = $files_list;
        $this->set_files_list = true;
            
        return $this;
    }
        
    /**
     * Generate the field value.
     *
     * @access public
     *
     * @return string
     */
    public function generate($placeholder = true)
    {
		// if the form field does not exist, skip
        if (empty($this->form_field))
        {
            return null;
        }
		
        $form_field = $this->form_field;
        $properties = $form_field->properties;
        $entry = $this->entry;
            
        // Get the stored value.
        $field_value = isset($entry->{$form_field->column_name}) ? $entry->{$form_field->column_name} : '';
            
        switch ($form_field->field_type)
        {
            case 'big_subtitle':
            case 'small_subtitle':
            case 'description':
            case 'textarea':
                    
                $value = RSDirectoryHelper::cleanText( $field_value, true, $properties->get('clean_links') );
					
                if ( !$properties->get('allow_html') )
                {
                    $value = strip_tags($value);
                }
                    
                break;
                    
            case 'dropdown':
            case 'checkboxgroup':
            case 'radiogroup':
                    
                $value = str_replace("\r\n", ', ', $field_value);
                break;
                    
            case 'calendar':
            case 'dropdown_date_picker':
                if ($field_value == '0000-00-00 00:00:00') {
                    $value = '';
                } else {

                    if ($properties->get('date_mask') == 'human_readable') {
                        $value = RSDirectoryHelper::formatDate($field_value, $properties->get('date_mask'));
                    } else {
                        if (strlen($field_value) > 0)
                        {
                            $date = array();

                            $dateMask = $properties->get('date_mask');
                            $timeMask = $properties->get('time_mask');

                            if (!is_null($dateMask))
                            {
                                // legacy workaround (for dates that are like: "1988-00-00 00:00:00"), the "00" month and day doesn't exist
                                $dateTimeValues = explode(' ', $field_value);
                                $dateValues = explode('-', $dateTimeValues[0]);
                                $month = (int)$dateValues[1];
                                $day = (int)$dateValues[2];

                                if (!$month || !$day)
                                {
                                    $month = !$month ? '01' : str_pad($month, 2, '0');
                                    $day = !$day ? '01' : str_pad($day, 2, '0');
                                    $field_value = $dateValues[0] . '-' . $month . '-' . $day . ' ' . $dateTimeValues[1];
                                }
                                // end workaround
                                $date[] = RSDirectoryHelper::formatDate($field_value, $dateMask);
                            }
                            if (!is_null($timeMask)) {
                                $date[] = RSDirectoryHelper::formatDate($field_value, $timeMask);
                            }

                            $value = trim(implode(' ', $date));
                        } else {
                            $value = '';
                        }
                    }
                }
                    
                break;
                    
            case 'fileupload':
            case 'image_upload':
                    
                if ($this->set_files_list)
                {
                    // Get the files list.
                    $files_list = RSDirectoryHelper::getFilesObjectList($form_field->id, $entry->id);
                }
                else
                {
                    $files_list = $this->files_list;
                }
                    
                if ( !empty($files_list) )
                {
                    if ($form_field->field_type == 'image_upload')
                    {
                        $value = RSDirectoryFormField::getImagesList($files_list, 0, 0);
                    }
                    else
                    {
                        $value = RSDirectoryFormField::getFilesList($files_list, 0, 0);
                    }
                    
                }
                    
                break;
                    
            case 'map':
                    
                $address_column_name = "{$form_field->column_name}_address";
                $value = isset($entry->$address_column_name) ? $entry->{$address_column_name} : '';
                break;    
                    
            case 'free_text':
                    
                $value = $properties->get('text');
                break;
                    
            case 'link':
				 // Check to see if there is a value set first
                if (!empty($field_value))
                {
                    $anchor  = $form_field->properties->get('listing_anchor', '');
                    // Determine if the anchor is usable
                    $use_anchor = (!empty($anchor) && !$placeholder) || (!empty($anchor) && $placeholder && (int)$form_field->properties->get('listing_anchor_in_placeholder', '0'));

                    $value =  '<a href="'.$field_value.'"'.($form_field->properties->get('listing_open_in_new_window') ? ' target="_blank"' : '').'>' . ($use_anchor ? $anchor : $field_value) . '</a>';
                }
                else
                {
                    $value = '';
                }
			break;
                    
            default:
                if ($form_field->properties->get('default_validation_rule', 'none') == 'validurl' && !$placeholder) {
					$value = '<a href="'.$field_value.'">' . $field_value . '</a>';
				} else {
					$value = $field_value;
				}
            break;
        }
            
        return !isset($value) ? null : $value;
    }
}