<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

require_once dirname(__FILE__) . '/formfieldproperties.php';

/**
 * Form field generator class.
 */
class RSDirectoryFormField
{
    /**
     * The form field object.
     *
     * @var object
     * 
     * @access protected
     */
    protected $form_field;
        
    /**
     * Entry data.
     *
     * @var mixed
     * 
     * @access protected
     */
    protected $entry;
        
    /**
     * Entry credits object list.
     *
     * @var array
     * 
     * @access protected
     */
    protected $entry_credits;
        
    /**
     * Attributes array.
     *
     * @var array
     *
     * @access protected
     */
    protected $attrs = array();
        
    /**
     * Have we initialized the form_fields javascript array?
     *
     * @var bool
     * 
     * @access protected
     */
    protected static $form_fields_init;
        
    /**
     * The class constructor.
     *
     * @access public
     * 
     * @param object $form_field
     * @param object $entry
     * @param array $entry_credits
     */
    public function __construct( $form_field, $entry = null, $entry_credits = array() )
    {
        $this->form_field = $form_field;
        $this->entry = $entry;
        $this->entry_credits = $entry_credits;
    }
        
    /**
     * Get RSDirectoryFormField instance.
     *
     * @access public
     * 
     * @static
     * 
     * @param object $form_field
     * @param object $entry
     * @param array $entry_credits
     * 
     * @return RSDirectoryFormField
     */
    public static function getInstance( $form_field, $entry = null, $entry_credits = array() )
    {
        $rsdirectoryformfield = new RSDirectoryFormField($form_field, $entry, $entry_credits);
            
        return $rsdirectoryformfield;
    }
        
    /**
     * Set an attribute, creating a new one if it doesn't exist already, or adding to the current attribute string if it does.
     *
     * @access protected
     *
     * @param string $key
     * @param string $value
     */
    protected function setAttr($key, $value)
    {
        if ( isset($this->attrs[$key]) )
        {
            if ( substr($this->attrs[$key], -1, 1) !== ' ' )
            {
                $this->attrs[$key] .= ' ';
            }
                
            $this->attrs[$key] .= $value;
        }
        else
        {
            $this->attrs[$key] = $value;
        }
    }
	
	protected function removeAttr($key) {
		if (isset($this->attrs[$key])) {
			unset($this->attrs[$key]);
		}
	}
        
    /**
     * Set multiple attributes at once.
     *
     * @access protected
     *
     * @param array $data
     */
    protected function setAttrs($data)
    {
        if ( is_array($data) && $data )
        {
            foreach ($data as $key => $value)
            {
                $this->setAttr($key, $value);
            }
        }
    }
        
    /**
     * Get attribute.
     *
     * @access protected
     *
     * @param string $key
     *
     * @return mixed
     */
    protected function getAttr($key)
    {
        return isset($this->attrs[$key]) ? $this->attrs[$key] : null;
    }
        
    /**
     * Get the attributes string.
     *
     * @access protected
     *
     * @return string
     */
    protected function getAttrsString()
    {
        $str = '';
            
        if ($this->attrs)
        {
            foreach ($this->attrs as $key => $value)
            {
                $str .= " $key=\"$value\"";
            }
        }
            
        return $str;
    }
        
    /**
     * Generate HTML code.
     *
     * @access public
     * 
     * @return string
     */
    public function generate($form_id = null)
    {
        $form_field = $this->form_field;
        $entry = $this->entry;
        $entry_credits = $this->entry_credits;
        $properties = empty($form_field->properties) ? new RSDirectoryFormFieldProperties : $form_field->properties;
          
        if ( in_array( $form_field->field_type, array('section_break', 'captcha') ) )
        {
            return;
        }

        if ( ($form_field->field_type == 'publishing_period' || $form_field->field_type == 'promoted') && !unserialize( $properties->get('items') ) )
        {
            return;
        }

        // Get the mainframe.
        $app = JFactory::getApplication();
            
        // Get the JDocument object.    
        $doc = $app->getDocument();
            
        // Get the JInput object.
        $jinput = $app->input;
            
        // Get the posted data.
        $data = $app->getUserState('com_rsdirectory.edit.entry.data');
            
        // Is root user?
        $isRoot = JFactory::getUser()->get('isRoot');
            
        $can_edit_all_entries = RSDirectoryHelper::checkUserPermission('can_edit_all_entries');
            
        $is_site = $app->isClient('site');
        $view = $jinput->get('view');
        $layout = $jinput->get('layout');
            
        // Check if there was an error with the form field.
        $error_field_ids = $app->getUserState('com_rsdirectory.edit.entry.error_field_ids');
        $has_error = is_array($error_field_ids) && in_array($form_field->id, $error_field_ids);
            
            
        // Get the field model.
        $field_model = RSDirectoryModel::getInstance('Field');
            
        // Get an array field types that can be used as dependency parents.
        $dependency_parent_compatible = $field_model->getDependencyParentCompatible();
            
        // Check if the field is a dependency parent.
        if ( in_array($form_field->field_type, $dependency_parent_compatible) )
        {
            $is_dependency_parent = RSDirectoryHelper::isDependencyParent($form_field->id);
        }
            
        // Initialize the field wrapper attributes array.
        $field_wrapper_attrs = array(
            'class' => "rsdir-field-wrapper rsdir-field-type-$form_field->field_type rsdir-field-wrapper-$form_field->id ". RsdirectoryAdapterGrid::controlGroup() . ($has_error ? ' error' : ''),
        );
            
        // Initialize the items array.
        $items = array();
		
		// the renew type
		$renew_types = (int) RSDirectoryConfig::getInstance()->get('renew_types', 0);
            
        // Get dependency items.
        if ( $properties->get('dependency') )
        {
            if ( $dependency_options = $this->getDependencyOptions( $properties->get('dependency') ) )
            {
                $items = $dependency_options;
            }
        }
        else if ( $properties->get('items') )
        {
            // Set the select/checkbox/radio items.
            $items = RSDirectoryHelper::getOptions( $properties->get('items') );
        }
            
            
        // Set attributes.
        if ( $properties->get('size') )
        {
            $this->setAttr( 'size', $properties->get('size') );
        }
            
        if ( $properties->get('max_size') )
        {
            $this->setAttr( 'max_size', $properties->get('max_size') );
        }
            
        if ( $properties->get('additional_html_attributes') )
        {
            try {
                @$element = new SimpleXMLElement("<element " . $properties->get('additional_html_attributes') . " />");
                    
                foreach ( $element->attributes() as $key => $value )
                {
                    $this->setAttr($key, $value);
                }
            }
            catch (Exception $e)
            {
            }
        }
            
        $this->setAttr('class', 'rsdir-field');
            
            
        // Initialize the name attribute.
        $name_attr = "fields[$form_field->form_field_name]";
         
            
        // Set the default value.
        if ( isset($data[$form_field->form_field_name]) )
        {
            $default = $data[$form_field->form_field_name];
        }
        else if ( isset($entry->{$form_field->column_name}) && $form_field->column_name != 'promoted' )
        {
            $default = $entry->{$form_field->column_name};
        }
        else
        {
            $default = $properties->get('default_value');
        }
            
        // Initialize the form_fields JS variable.
        if (!self::$form_fields_init)
        {
            self::$form_fields_init = true;
                
            $doc->addScriptDeclaration("var form_fields = {};");
        }

            
        // Initialize the result strings.
        $label_str = '';
        $field_str = '';
        $before_str = '';
        $after_str = '';
            
            
        $caption = self::escapeHTML( $properties->get('form_caption') );
            
        // Add a required mark.
        if ( !empty($form_field->required) )
        {
            $caption .= ' <span class="rsdir-required" title="' . JText::_('COM_RSDIRECTORY_FIELD_REQUIRED') . '">*</span>';
        }
            
        // Add a label to the form field, if the form caption is set.
        if ($caption)
        {
            $label_str = '<label class="rsdir-label '.(RSDirectoryHelper::isJ4() ? 'form-label' : '').'">' . $caption . "</label>\n";
        }
            
        if ( !$isRoot && ($is_site || !$can_edit_all_entries) && $form_field->field_type != 'calendar' && $properties->get('readonly') )
        {
			// Radio, check boxes and dropdowns don't have a readonly attribute.
			if (in_array($form_field->field_type, array('radiogroup', 'checkboxgroup', 'dropdown'))) {
				$this->setAttr('disabled', 'disabled');
			} else {
				$this->setAttr('readonly', 'readonly');
			}
        }
		
		// need to see if the entry is expired (if the entry exists)
		$is_expired = false;
		if (!is_null($entry)) {
			$is_expired = RSDirectoryHelper::isEntryExpired($entry->id);
		}

        // need to see if we can charge the fields after expiration and the publishing period
        $charge_fields = true;
        $charge_period = true;
        $charge_promoted_period = true;
        if (!is_null($entry)) {
            if ($is_expired && $renew_types == 0) {
                $charge_fields = false;
            }

            // handle the publishing period charging
            if ($form_field->field_type == 'publishing_period') {
                if ($is_expired && $renew_types == 2) {
                    $charge_period = false;
                } else if (!$is_expired) {
                    // if the entry is not expired check to see if the publishing period has been paid
                    $unpaid = RSDirectoryHelper::findElements(array('object_type' => 'publishing_period', 'paid' => 0), $entry_credits);
                    if (!$unpaid) {
                        $charge_period = false;
                    }
                }
            }
        }


        if (!is_null($entry) && $form_field->field_type == 'promoted' && !$can_edit_all_entries) {
            // when promoted is unlimited
            $form_id          = !empty($form_id) ? $form_id : RSDirectoryHelper::getCategoryInheritedFormId($entry->category_id);
            $promoted_credit  = RSDirectoryCredits::getLastPaidCreditField('promoted', $entry->id, $form_id);
            $promoted_comparison_date = false;
            if ($promoted_credit) {
                // if there are both promoted_expiry and normal entry expiry must determine which will be our comparison
                if ($entry->promoted_expiry != '0000-00-00 00:00:00' && $entry->expiry_time != '0000-00-00 00:00:00') {
                    $p_exp = JFactory::getDate($entry->promoted_expiry)->toUnix();
                    $e_exp = JFactory::getDate($entry->expiry_time)->toUnix();
                    $promoted_comparison_date =  $e_exp < $p_exp ? $e_exp : $p_exp;
                } else if ($entry->promoted_expiry != '0000-00-00 00:00:00' && $entry->expiry_time == '0000-00-00 00:00:00') {
                    $promoted_comparison_date =  JFactory::getDate($entry->promoted_expiry)->toUnix();
                }
            }


            // when promoted is limited
            if ($promoted_comparison_date && $promoted_comparison_date > JFactory::getDate()->toUnix())
            {
                $charge_promoted_period = false;
                $this->setAttr('disabled', 'disabled');
            }
        }
		
		// need to track the number of file loaded
		$files_loaded = 0;
            
        // Proceed based on the form field type.
        switch ($form_field->field_type)
        {
            case 'title':
            case 'big_subtitle':
            case 'small_subtitle':
            case 'youtube':
            case 'vimeo':
            case 'link':

                $this->setAttr('class', 'rsdir-textbox');

                if (RSDirectoryHelper::isJ4())
                {
                    $this->setAttr('class', 'form-control');
                }

                $field_str .= "<input type=\"text\" name=\"$name_attr\"" . ( strlen($default) ? ' value="' . self::escapeHTML($default) . '"' : '' ) . $this->getAttrsString() . " />\n";
                    
                break;
                    
            case 'description':
            case 'textarea':
                    
                if ( $properties->get('allow_html') )
                {
                    if (RSDirectoryHelper::isJ4())
                    {
                        $jconfig = $app->getConfig();
                        $default_editor = $jconfig->get('editor', 'none');

                        $editor = JEditor::getInstance($default_editor);
                    }
                    else
                    {
                        $editor = JFactory::getEditor();
                    }

					$editor_id = preg_replace('/(\s|[^A-Za-z0-9_])+/', '_', $form_field->form_field_name).'_'.$form_field->id;

                    $field_str .= $editor->display( $name_attr, self::escapeHTML($default), '100%' , '20%' , '100' , '10', true, $editor_id, null, null, array('disabled'=>'disabled') );

                    if (RSDirectoryHelper::isJ4())
                    {
                        $field_str .= '<div class="clearfix"></div>';
                    }

                    if ($is_site || !$can_edit_all_entries)
                    {
                        $script = 'function rsdir_description_timeout'.$form_field->id.'()
                        {
                            window.setTimeout(function()
                            {
                                var content = Joomla.editors.instances["' . $editor_id . '"].getValue();
                                updateTextCreditsCost( document.getElementById("' . $editor_id . '"), content );
                                rsdir_description_timeout'.$form_field->id.'();
                            }, 1000);
                        }
                            
                        jQuery(function()
                        {
                            rsdir_description_timeout'.$form_field->id.'();
                        });';
                            
                        $doc->addScriptDeclaration($script);
                    }
                }
                else
                {
                    $this->setAttr('class', 'rsdir-textarea');

                    if (RSDirectoryHelper::isJ4()) {
                        $this->setAttr('class', 'form-control');
                    }
                        
                    $field_str .= '<textarea name="' . $name_attr . '" ' . $this->getAttrsString() . '>' . self::escapeHTML($default) . "</textarea>\n";
                }
                    
                break;
                    
            case 'images':
            case 'image_upload':
            case 'fileupload':
                    
                $max_files_number = $properties->get('max_files_number');
                    
                $script = "rsdir.files_limit[$form_field->id] = $max_files_number;";
                    
                // Add the script declaration.
                $doc->addScriptDeclaration($script);
                    
                if ($entry)
                {
                    // Get the files list for this field.
                    $files_list = RSDirectoryHelper::getFilesObjectList($form_field->id, $entry->id);
                }
                    
                $diff = $max_files_number - ( empty($files_list) ? 0 : count($files_list) );
                    
                $this->setAttr( 'class', in_array( $form_field->field_type, array('images', 'image_upload') ) ? 'rsdir-images' : 'rsdir-fileupload' );

                if (RSDirectoryHelper::isJ4()) {
                    $this->setAttr('class', 'form-control');
                }
					
                $field_str .= '<div class="rsdir-file-upload' . ($diff < 1 && $max_files_number ? ' hide' : '') . '">';

                $file_input  = '<input type="file" name="' . $name_attr . '[]"' . $this->getAttrsString() . ' />';
                $remove_icon = '<i class="rsdir-clear-upload rsdir-icon-x"></i>';

                $field_str .= RsdirectoryAdapterGrid::inputAppend($file_input, $remove_icon);
                $field_str .= '</div>';
                    
                if ($max_files_number > 1 || $max_files_number == 0)
                {
                   $field_str .= '<div class="rsdir-add-file-upload btn btn-mini btn-light' . ( $diff < 2 && $max_files_number ? ' hidden' : '' ) . (RSDirectoryHelper::isJ4() ? ' d-block':'').'"><i class="fa fa-plus"></i> ' . JText::_('COM_RSDIRECTORY_ADD_MORE') . '</div>';
                }
                    
                // Show the list of uploaded files for an entry.
                if ( !empty($files_list) )
                {
                    if ( in_array( $form_field->field_type, array('images', 'image_upload') ) )
                    {
                        $after_str .= self::getImagesList($files_list);
                    }
                    else
                    {
                        $after_str .= self::getFilesList($files_list);
                    }
					
					$files_loaded = count($files_list);
					
                }
				
				if (RSDirectoryHelper::isJ4()) {
                    // load the jQuery UI
                    JHtml::_('script','com_rsdirectory/jquery-ui-1.10.4.custom.min.js', array('relative' => true, 'version' => 'auto'));
                }
                    
                break;
                    
            case 'price':
                    
                $this->setAttr('class', 'rsdir-textbox input-small');
				
				 // set the correct output
                if (isset($default) && $default != '') {
                    $config                  = RSDirectoryConfig::getInstance();
                    $decimals 				 = $config->get('decimals');
                    $thousands_separator 	 = $config->get('thousands_separator');
                    $decimal_point 			 = $config->get('decimal_point');
					
					// if the thousands_separator is a "." then we must leave at least one because its the decimal point
                    $replace_all_but_one = $thousands_separator == '.' && $decimal_point != '.';
					
					// Make sure that the default value can be handled by the number_format function
					if (is_string($default))
                    {
                        // remove the thousands_separator, if any is defined
                        if ($thousands_separator && !$replace_all_but_one)
                        {
                            $default = str_replace($thousands_separator, '', $default);
                        }
                        else if ($thousands_separator && $replace_all_but_one)
                        {
                            if (substr_count($default, '.') > 1)
                            {
                                $default_parts = explode('.', $default);
                                $last_index = count($default_parts) - 1;

                                $default_parts[$last_index] = '.'. $default_parts[$last_index];
                                $default = implode('', $default_parts);
                            }
                        }

                        // if decimal point is set, we must make sure is a dot
                        if ($decimal_point) {
                            $default = str_replace($decimal_point, '.', $default);
                        }

                        $default = (float) $default;
                    }

                    if ($default > 0) {
                        $default = number_format($default, $decimals, $decimal_point, $thousands_separator);
                    } else {
                        $default = '';
                    }
                }

                if (RSDirectoryHelper::isJ4()) {
                    $this->setAttr('class', 'form-control');
                }
                $file_input     = '<input type="text" name="' . $name_attr . '"' . ( isset($default) ? ' value="' . self::escapeHTML($default) . '"' : '' ) . $this->getAttrsString() . " />\n ";
                $currency_sign  = self::escapeHTML( RSDirectoryConfig::getInstance()->get('currency') );

                $field_str .= RsdirectoryAdapterGrid::inputAppend($file_input, $currency_sign);

                break;
                    
            case 'publishing_period':
                    
                // Initialize the options array.
                $options = array();
                    
                // Get the items.
                $items = unserialize( $properties->get('items') );
                    
                // Initialize the credits array.
                $credits = array();

                // Initialize the credits days array.
                $days = array();
                    
                // Initialize the current value.
                $current = null;
                    
                if ( !empty($entry) && $items )
                {
                    foreach ($items as $i => $item)
                    {
                        if ($entry->period == $item->period)
                        {
                            if ( empty($default) )
                            {
                                $default = $i + 1;
                            }
                                
                            $current = $i + 1;
                                
                            break;
                        }
                    }
                }


                $promoted_field_id = null;

                if (!empty($form_id))
                {
                    // Get the promoted id
                    $db = JFactory::getDbo();
                    $query = $db->getQuery(true)
                        ->select($db->qn('id'))
                        ->from($db->qn('#__rsdirectory_fields', 'f'))
                        ->leftJoin($db->qn('#__rsdirectory_forms_fields', 'ff') . ' ON ' . $db->qn('ff.field_id') . ' = ' . $db->qn('f.id'))
                        ->where($db->qn('f.name') . ' = ' . $db->q('promoted'))
                        ->where($db->qn('ff.form_id') . ' = ' . $db->q($form_id));

                    $db->setQuery($query, 0, 1);

                    $promoted_field_id = $db->loadResult();
                }

                if ($items)
                {
                    foreach ($items as $i => $item)
                    {
                        if ($item->period)
                        {
                            $label = strtolower( JText::plural('COM_RSDIRECTORY_NUMBER_OF_DAYS', $item->period) );    
                        }
                        else
                        {
                            $label = JText::_('COM_RSDIRECTORY_NO_EXPIRY_LABEL');
                        }
                            
                        if ($item->credits)
                        {
                            $label .= ' (' . JText::plural('COM_RSDIRECTORY_NUMBER_OF_CREDITS', $item->credits) . ')';
                        }
                            
                        $options[] = JHtml::_('select.option', $i + 1, $label);
                        
						// do not charge the period if the renew type option does not allow this
						if ($is_expired && $renew_types == 2 && isset($default) && ($i + 1) == $default) {
							$item->credits = 0;
						}

                        $credits[] = ($i + 1) . ":$item->credits";
                        $days[] = ($i + 1) . ":$item->period";
                    }
                }
                    
                $this->setAttr('class', 'rsdir-publishing-period');

                $field_str .= '<div class="'.RsdirectoryAdapterGrid::controlGroup().'">';
                    
                if ( $properties->get('display_method') == 'dropdown' )
                {
                    // add form control for the select
                    if (RSDirectoryHelper::isJ4())
                    {
                        $this->setAttr('class', 'form-select');
                    }

                    array_unshift( $options, JHtml::_( 'select.option', '', JText::_('COM_RSDIRECTORY_SELECT_PUBLISHING_PERIOD') ) );
                        
                    $field_str .= JHtml::_( 'select.genericlist', $options, $name_attr, $this->getAttrsString(), 'value', 'text', isset($default) ? $default : null );
                }
                else if ( $properties->get('display_method') == 'radiogroup' )
                {
                    foreach ($options as $i => $option)
                    { 
                        $checked = isset($default) && $option->value == $default;

                        $id = '';

                        if (RSDirectoryHelper::isJ4())
                        {
                            $current_id = $this->getAttr('id');
                            $this->removeAttr('id');
                            if (!empty($current_id))
                            {
                                $id = $current_id.$i;
                            } else {
                                $id = 'rsdir-publishing-period'.$i;
                            }

                            $this->setAttr('id', $id);
                            $this->setAttr('class', 'form-check-input');
                        }

                        $attrs = $this->getAttrsString();
                            
                        $p_radio_label = '<label'.(RSDirectoryHelper::isJ4() ? ' for="'.$id.'"' : '').' class="rsdir-radio-label'.(RSDirectoryHelper::isJ4() ? ' form-check-label' : '').' radio' . ( $properties->get('flow') == 'horizontal' ? ' inline' : '' ) . '">{rsdir_radio}'. self::escapeHTML($option->text) . "</label>\n";

                        $p_radio_input = '<input type="radio" name="' . $name_attr . '" value="' . self::escapeHTML($option->value) . "\"$attrs" . ($checked ? ' checked="checked"' : '') . ' /> ';


                        $flow = $properties->get('flow') == 'horizontal' ? ' form-check-inline' : '';
                        $field_str .= RsdirectoryAdapterGrid::checkbox($p_radio_input, $p_radio_label, $flow);
                    }
                }
                    
                $field_str .= '</div>';
                    
                if ( $properties->get('show_republish') )
                {
                    $checked = !empty($data['renew']) || !empty($entry->renew);

                    $id = '';

                    if (RSDirectoryHelper::isJ4())
                    {
                        $id = 'rsdir-renew-labe-after';
                    }

                    $after_radio_label = '<label'.(RSDirectoryHelper::isJ4() ? ' for="'.$id.'"' : '').' class="rsdir-renew-label checkbox'.(RSDirectoryHelper::isJ4() ? ' form-check-label' : '').'">{rsdir_radio}'.JText::_('COM_RSDIRECTORY_AUTOMATICALLY_REPUBLISH_ENTRY_LABEL') . '</label>';
                    $after_radio_input = '<input type="checkbox"'.(RSDirectoryHelper::isJ4() ? ' class="form-check-input"' : '').' name="fields[renew]"' . ($checked ? ' checked="checked"' : '') . ' '.(!empty($id) ? 'id="'.$id.'"' : '').'/> ';

                    $after_str .= RsdirectoryAdapterGrid::checkbox($after_radio_input, $after_radio_label);
                }
				
                $script = "form_fields[$form_field->id] = {" .
                            "credits: null," .
                            "multiple_credits: {" . implode(',', $credits) . "}," .
                            "days: {" . implode(',', $days) . "}," .
                            "promoted_field_id: " . (!empty($promoted_field_id) ? $promoted_field_id : '0') . "," .
                            "current_values: [". ($current && !$charge_period ? $current : '') . "]," .
                            "selected_values: [" . ($default ? $default : '') . "]," .
                            "field_type : '".$form_field->field_type."'," .
                            "credits_per_file: null" .
                          "}";
                            
                $doc->addScriptDeclaration($script);
                    
                break;
                    
            case 'promoted':

                // Initialize the options array.
                $options = array();

                // Get the items.
                $items = unserialize( $properties->get('items') );

                // Initialize the credits array.
                $credits = array();

                // Initialize the credits days array.
                $days = array();

                // Initialize the current value.
                $current = null;

                if ( !empty($entry) && $items )
                {
                    foreach ($items as $i => $item)
                    {
                        if ($entry->promoted_selected == $item->period)
                        {
                            if ( empty($default) )
                            {
                                $default = $i + 1;
                            }

                            $current = $i + 1;

                            break;
                        }
                    }
                }

                // handle custom case
                if (!empty($entry)){
                    if($entry->promoted_selected == 'custom_high')
                    {
                        $current = 'custom_high';
                        $default = 'custom_high';
                    }

                    if($entry->promoted_selected == 'custom_low')
                    {
                        $current = 'custom_low';
                        $default = 'custom_low';
                    }
                }

                // Get the published id
                $db = JFactory::getDbo();
                $query = $db->getQuery(true)
                    ->select( $db->qn('id'))
                    ->from( $db->qn('#__rsdirectory_fields', 'f') )
                    ->where( $db->qn('f.name') . ' = ' . $db->q('publishing-period') );

                $db->setQuery($query, 0 , 1);

                $published_field_id = $db->loadResult();

                $unlimited_index = '';
                $additional_script = '';

                if ($items)
                {

                    foreach ($items as $i => $item)
                    {
                        if ($item->period)
                        {
                            $label = strtolower( JText::plural('COM_RSDIRECTORY_NUMBER_OF_DAYS', $item->period) );
                        }
                        else
                        {
                            // skip the unlimited one if is the user cannot modify all entries
                            if (!$can_edit_all_entries) {
                                continue;
                            }
                            $label = JText::_('COM_RSDIRECTORY_NO_EXPIRY_LABEL');
                        }

                        if ($item->credits && $item->period !== '0')
                        {
                            $label .= ' (' . JText::plural('COM_RSDIRECTORY_NUMBER_OF_CREDITS', $item->credits) . ')';
                        } 
						else if ($item->period === '0')
                        {
							// only users that can modify all entries can use unlimited and it should be free
                            $item->credits = 0;
                        }

                        $options[] = JHtml::_('select.option', $i + 1, $label);

                        $credits[] = ($i + 1) . ":$item->credits";
                        $days[] = ($i + 1) . ":$item->period";

                        // hold the unlimited index
                        if ($item->period === '0') {
                            $unlimited_index = ($i + 1);
                        }
                    }
                }

                // add the custom_high option if the default is "custom_high"
                if ($default == 'custom_high')
                {

                    if ($entry->promoted_period)
                    {
                        $label = strtolower( JText::plural('COM_RSDIRECTORY_NUMBER_OF_DAYS', $entry->promoted_period) );
                    }
                    else
                    {
                        $label = JText::_('COM_RSDIRECTORY_NO_EXPIRY_LABEL');
                    }

                    if ($entry->promoted_credits)
                    {
                        $label .= ' (' . JText::plural('COM_RSDIRECTORY_NUMBER_OF_CREDITS', $entry->promoted_credits) . ')';
                    }

                    $options[] = JHtml::_('select.option', 'custom_high', $label);
                    $credits[] = "'custom_high':$entry->promoted_credits";
                    $days[] = "'custom_high':$entry->promoted_period";
                }

                // add the custom_low option if the default is "custom_low"
                if ($default == 'custom_low')
                {

                    if ($entry->promoted_period)
                    {
                        $label = strtolower( JText::plural('COM_RSDIRECTORY_NUMBER_OF_DAYS', $entry->promoted_period) );
                    }
                    else
                    {
                        $label = JText::_('COM_RSDIRECTORY_NO_EXPIRY_LABEL');
                    }

                    if ($entry->promoted_credits)
                    {
                        $label .= ' (' . JText::plural('COM_RSDIRECTORY_NUMBER_OF_CREDITS', $entry->promoted_credits) . ')';
                    }

                    $options[] = JHtml::_('select.option', 'custom_low', $label);
                    $credits[] = "'custom_low':$entry->promoted_credits";
                    $days[] = "'custom_low':$entry->promoted_period";
                }

                $this->setAttr('class', 'rsdir-promoted-period');

                $field_str .= '<div class="'.RsdirectoryAdapterGrid::controlGroup().'">';

                if ( $properties->get('display_method') == 'dropdown' )
                {
                    // add form control for the select
                    if (RSDirectoryHelper::isJ4())
                    {
                        $this->setAttr('class', 'form-select');
                    }

                    array_unshift( $options, JHtml::_( 'select.option', '-1', JText::_('COM_RSDIRECTORY_SELECT_PROMOTED_PERIOD') ) );

                    $field_str .= JHtml::_( 'select.genericlist', $options, $name_attr, $this->getAttrsString(), 'value', 'text', isset($default) ? $default : null );

                    if (!empty($unlimited_index))
                    {
                        $additional_script = 'jQuery("document").ready(function(){
                            jQuery(".rsdir-promoted-period").on("change", function () {
                                if (jQuery(this).val() == "'.$unlimited_index.'") {
                                    jQuery(".rsdir-promoted-expiry").show();
                                } else {
                                    jQuery(".rsdir-promoted-expiry").hide();
                                }
                            });

                        });';
                    }
                }
                else if ( $properties->get('display_method') == 'radiogroup' )
                {
                    $id = '';

                    if (RSDirectoryHelper::isJ4())
                    {
                        $current_id = $this->getAttr('id');
                        $this->removeAttr('id');
                        if (!empty($current_id))
                        {
                            $id = $current_id.'0';
                        } else {
                            $id = 'rsdir-promoted-periodllabel'.'0';
                        }

                        $this->setAttr('id', $id);
                        $this->setAttr('class', 'form-check-input');
                    }

                    $attrs = $this->getAttrsString();

                    // add the null one
                    $pro_period_label = '<label'.(RSDirectoryHelper::isJ4() ? ' for="'.$id.'"' : '').' class="rsdir-radio-label'.(RSDirectoryHelper::isJ4() ? ' form-check-label' : '').' radio' . ( $properties->get('flow') == 'horizontal' ? ' inline' : '' ) . '">{rsdir_radio}'.JText::_('COM_RSDIRECTORY_NOT_PROMOTED_PERIOD') . "</label>\n";
                    $pro_period_input = '<input type="radio" name="' . $name_attr . '" value="-1" '.$attrs.'/> ';

                    $flow = $properties->get('flow') == 'horizontal' ? ' form-check-inline' : '';
                    $field_str .= RsdirectoryAdapterGrid::checkbox($pro_period_input, $pro_period_label, $flow);

                    foreach ($options as $i => $option)
                    {
                        // start at 1
                        $i++;

                        $id = '';

                        if (RSDirectoryHelper::isJ4())
                        {
                            $current_id = $this->getAttr('id');
                            $this->removeAttr('id');
                            if (!empty($current_id))
                            {
                                $id = $current_id.$i;
                            } else {
                                $id = 'rsdir-promoted-periodllabel'.$i;
                            }

                            $this->setAttr('id', $id);
                            $this->setAttr('class', 'form-check-input');
                        }

                        $attrs = $this->getAttrsString();

                        $checked = isset($default) && $option->value == $default;

                        $pro_period_label = '<label'.(RSDirectoryHelper::isJ4() ? ' for="'.$id.'"' : '').' class="rsdir-radio-label'.(RSDirectoryHelper::isJ4() ? ' form-check-label' : '').' radio' . ( $properties->get('flow') == 'horizontal' ? ' inline' : '' ) . '">{rsdir_radio}'.self::escapeHTML($option->text) . "</label>\n";
                        $pro_period_input = '<input type="radio" name="' . $name_attr . '" value="' . self::escapeHTML($option->value) . "\"$attrs" . ($checked ? ' checked="checked"' : '') . ' /> ';

                        $field_str .= RsdirectoryAdapterGrid::checkbox($pro_period_input, $pro_period_label, $flow);
                    }

                    if (!empty($unlimited_index))
                    {
                        $additional_script = 'jQuery("document").ready(function(){
                                jQuery(".rsdir-promoted-period").on("click", function () {
                                    if (jQuery(this).val() == "' . $unlimited_index . '") {
                                        jQuery(".rsdir-promoted-expiry").show();
                                    } else {
                                        jQuery(".rsdir-promoted-expiry").hide();
                                    }
                                });

                        });';
                    }
                }

                $field_str .= '</div>';

                $script = "form_fields[$form_field->id] = {" .
                    "credits: null," .
                    "multiple_credits: {" . implode(',', $credits) . "}," .
                    "days: {" . implode(',', $days) . "}," .
                    "daily_cost: ". ((int)$properties->get('daily_cost')) ."," .
                    "published_field_id: " . (!empty($published_field_id) ? $published_field_id : '0') . "," .
                    "current_values: [". ($current && !$charge_promoted_period ? (is_numeric($current) ? $current : "'".$current."'") : '') . "]," .
                    "selected_values: [" . ($default ? (is_numeric($default) ? $default : "'".$default."'") : '') . "]," .
                    "field_type : '".$form_field->field_type."'," .
                    "credits_per_file: null}";



                $doc->addScriptDeclaration($script.(!empty($additional_script) ? "\n".$additional_script : ''));


                break;
                    
            case 'textbox':
                    
                $this->setAttr('class', 'rsdir-textbox');
                if (RSDirectoryHelper::isJ4())
                {
                    $this->setAttr('class', 'form-control');
                }

                $field_str .= "<input type=\"text\" name=\"$name_attr\"" . ( strlen($default) ? ' value="' . self::escapeHTML($default) . '"' : '' ) . $this->getAttrsString() . " />\n";
                    
                break;
                    
            case 'dropdown':
            case 'country':
                    
                $is_multiple = $properties->get('multiple') && !$is_dependency_parent;

                $grouped_options = array();
                    
                // Set the default value.
                if ( isset($data[$form_field->form_field_name]) )
                {
                    $default = $data[$form_field->form_field_name];
                }
                else if ( isset($entry->{$form_field->column_name}) )
                {
                    if ($is_multiple)
                    {
                        $default = $entry->{$form_field->column_name} ? explode("\r\n", $entry->{$form_field->column_name}) : array();
                    }
                    else
                    {
                        $default = $entry->{$form_field->column_name};
                    }
                }
                else
                {
                    if ($is_multiple) {
                        $default = $properties->get('default_values') ? explode("\r\n", $properties->get('default_values')) : array();
                    } else if ($is_dependency_parent) {
                        $default = $properties->get('default_value');
                    } else {
                        $default = $properties->get('default_values');
                    }
                }

                // Initialize the options array.
                $options = array();
                    
                if (!$is_multiple)
                {
                    $options[] = array(
                        'items' => array(
                            JHtml::_( 'select.option', '', JText::_('COM_RSDIRECTORY_PLEASE_SELECT_OPTION') )
                        ),
                    );
                }
                    
                if ($items)
                {
                    $grouped_options = RSDirectoryHelper::getGroupedListOptions($items);

                    $options = array_merge(
                        $options,
                        $grouped_options['items']
                    );
                }
                    
                // Set the field name.
                $name = $name_attr . ($is_multiple ? '[]' : '');
                    
                $this->setAttr('class', 'rsdir-dropdown');

                if (RSDirectoryHelper::isJ4())
                {
                    $this->setAttr('class', 'form-select');
                }
                    
                // Is the select multiple?
                if ($is_multiple)
                {
                    $this->setAttr('multiple', 'multiple');    
                }
				
				if ($this->getAttr('disabled')) {
					$default_hidden_value = (array) $default;
					
					foreach ($default_hidden_value as $hidden_value) {
						$field_str .= '<input type="hidden" name="'.$name.'" value="'.self::escapeHTML($hidden_value).'" />';
					}
				}
                    
                $field_str .= JHtml::_(
                    'select.groupedlist',
                    $options,
                    $name,
                    array(
                        'id' => "fields$form_field->id",
                        'list.select' => isset($default) ? $default : null,
                        'list.attr' => $this->getAttrsString(),
                    )
                );

                // handle default credits
                $default_credits = $properties->get('credits');

                $skip_default = false;
                if ($entry_credits) {
                    // this check is done only if the entry is not expired
                    if (!$is_expired) {
                        foreach ($entry_credits as $object) {
                            if ($object->paid && $object->object_type == 'form_field' && $object->object_id == $form_field->id) {
                                $skip_default = true;
                                break;
                            }
                        }
                    }
                }

                if ($charge_fields) {
                    // check if there has been previously paid something
                    $unpaid = RSDirectoryHelper::findElements(array('object_type' => 'form_field', 'object_id' => $form_field->id, 'paid' => 0), $entry_credits);

                    if (isset($grouped_options['credits'])) {
                        // need to get the paid ones to
                        $paid = RSDirectoryHelper::findElements(array('object_type' => 'form_field', 'object_id' => $form_field->id, 'paid' => 1), $entry_credits);


                        // take the differences
                        $diff_total = 0;
                        $paid_diffs = RSDirectoryHelper::findElements(array('object_type' => 'form_field', 'object_id' => $form_field->id, 'paid' => 1, 'is_diff' => 1), $entry_credits, false);
                        if ($paid_diffs) {
                            foreach ($paid_diffs as $pd) {
                                $diff_total += (int) $pd->credits;
                            }
                        }

                        // the difference between the credits base on the selection and the already payed ones
                        $paid_credits = (!empty($paid->credits) ? (int)$paid->credits : 0) + $diff_total;

                        // init the cost based on the previous selection
                        $prev_credits = array();

                        // Initialize the current values.
                        $current_values = array();

                        if (!empty($entry) && $grouped_options['items']) {
                            $current = null;
                            if (isset($entry->{$form_field->column_name})) {
                                if ($is_multiple) {
                                    $current = $entry->{$form_field->column_name} ? explode("\r\n", $entry->{$form_field->column_name}) : array();
                                } else {
                                    $current = $entry->{$form_field->column_name};
                                }
                            }


                            if (!is_null($current)) {
                                foreach ($grouped_options['items'] as $group) {
                                    foreach ($group['items'] as $g_item) {
                                        if (!is_array($current)) {
                                            if ($g_item->value == $current) {
                                                $current_values[] = $g_item->value;

                                                if (isset($grouped_options['credits'][$g_item->value])) {
                                                    $prev_credits[] = (int)$grouped_options['credits'][$g_item->value];
                                                }
                                            }
                                        } else {
                                            foreach ($current as $val) {
                                                if ($g_item->value == $val) {
                                                    $current_values[] = $g_item->value;

                                                    if (isset($grouped_options['credits'][$g_item->value])) {
                                                        $prev_credits[] = (int)$grouped_options['credits'][$g_item->value];
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }

                        // add to the prev credits the default credits
                        if (!empty($default_credits)) {
                            $prev_credits[] = (int) $default_credits;
                        }

                        // total of the credits previous selected
                        $prev_credits = !empty($prev_credits) ? array_sum($prev_credits) : 0;

                        // Mark the field as paid.
                        if (strpos($field_wrapper_attrs['class'], 'rsdir-paid-field') === false) {
                            $field_wrapper_attrs['class'] .= ' rsdir-paid-field';
                        }

                        $script = "form_fields[$form_field->id] = {" .
                            "credits: ".$default_credits."," .
                            "multiple_credits: " . json_encode($grouped_options['credits']) . "," .
                            "current_values: " . ((!empty($current_values) && (!$unpaid || ($paid_credits != $prev_credits)) && !$is_expired) ? json_encode($current_values) : '[]') . "," .
                            "selected_values: ". ($default ? (is_array($default) ? json_encode($default) : '["' . $default . '"]') : '[]') . ",".
                            "field_type : '".$form_field->field_type."'," .
                            "credits_per_file: null" .
                            "}";

                        $doc->addScriptDeclaration($script);
                    } else {

                        $current = null;
                        if (!empty($entry) && isset($entry->{$form_field->column_name})) {
                            if ($is_multiple) {
                                $current = $entry->{$form_field->column_name} ? explode("\r\n", $entry->{$form_field->column_name}) : array();
                            } else {
                                $current = $entry->{$form_field->column_name};
                            }
                        }

                        if (!$skip_default) {
                            $script = "form_fields[$form_field->id] = {" .
                                "credits: ".$default_credits."," .
                                "multiple_credits: []," .
                                "current_values: [". ((!is_null($current) && !$unpaid && !$is_expired) ? 1 : '') . "]," .
                                "selected_values: [" . ($default ? 1 : '') . "]," .
                                "field_type : '".$form_field->field_type."'," .
                                "credits_per_file: null" .
                                "}";

                            $doc->addScriptDeclaration($script);
                        }
                    }
                }
                    
            break;
                    
            // Process a checkbox group field.
            case 'checkboxgroup':

                // Initiate the credits array
                $credits_all = array();
                    
                // Set the default value.
                if ( isset($data[$form_field->form_field_name]) )
                {
                    $default = $data[$form_field->form_field_name];
                }
                else if ( isset($entry->{$form_field->column_name}) )
                {
                    $default = $entry->{$form_field->column_name} ? explode("\r\n", $entry->{$form_field->column_name}) : array();
                }
                else
                {
                    $default = $properties->get('default_values') ? explode( "\r\n", $properties->get('default_values') ) : array();
                }
				
				if ($this->getAttr('disabled')) {
					$default_hidden_value = (array) $default;
					
					foreach ($default_hidden_value as $hidden_value) {
						$field_str .= '<input type="hidden" name="'.$name_attr.'[]" value="'.self::escapeHTML($hidden_value).'" />';
					}
				}
                    
                $this->setAttr('class', 'rsdir-checkbox');
                    
                $field_str .= '<div class="rsdir-items-group">';
                    
                // Get dependency items.
                if ( $properties->get('dependency') && empty($dependency_options) )
                {
                    $field_str .= JText::_('COM_RSDIRECTORY_NOTHING_TO_SELECT');
                }

                // hold the cleared items, without flags
                $default_items = array();
                    
                foreach ($items as $i=>$item)
                {
                    $value = $item->value;
					$text = $item->text;
                        
                    $disabled = false;
                        
                    if ( stripos($value, '[d]') !== false || stripos($text, '[d]') !== false )
                    {
                        $value = str_replace('[d]', '', $value);
                        $text = str_replace('[d]', '', $text);
                            
                        $disabled = true;
                    }

                    // check for credits in the value
                    if( preg_match('/\[c-\d+]/', $value, $credits) != false)
                    {
                        $value = str_replace($credits[0], '', $value);

                        // get the credits value
                        $credit_value = str_replace(array('[', ']', 'c-'), '', $credits[0]);
                        $credits_all[$value] = (int)$credit_value;
                    }

                    // check for credits in the text
                    if(preg_match('/\[c-\d+]/', $text, $credits))
                    {
                        $text = str_replace($credits[0], '', $text);

                        // get the credits value
                        $credit_value = str_replace(array('[', ']', 'c-'), '', $credits[0]);
                        // we need to save as reference the actual value
                        if (!isset($credits_all[$value])) {
                            $credits_all[$value] = (int)$credit_value;
                        }
                    }

                    $default_items[$value] = $text;
                    
					$allow_label_html = (int) $properties->get('allow_html', 0);
					
                    $checked = isset($default) && in_array($value, $default);

                    $id = '';

                    if (RSDirectoryHelper::isJ4())
                    {
                        $current_id = $this->getAttr('id');
                        $this->removeAttr('id');
                        if (!empty($current_id))
                        {
                            $id = $current_id.$i;
                        } else {
                            $id = 'rsdir-checkbox-label'.$i;
                        }

                        $this->setAttr('id', $id);
                        $this->setAttr('class', 'form-check-input');
                    }

                    $attrs = $this->getAttrsString();

                    $checkbox_label = '<label'.(RSDirectoryHelper::isJ4() ? ' for="'.$id.'"' : '').' class="rsdir-checkbox-label'.(RSDirectoryHelper::isJ4() ? ' form-check-label' : '').' checkbox' . ($properties->get('flow') == 'horizontal' ? ' inline' : '') . '">{rsdir_radio}'.($allow_label_html ? $text : self::escapeHTML($text)) . "</label>\n";
                    $checkbox_input = '<input type="checkbox" name="' . $name_attr . '[]" value="' . self::escapeHTML($value) . "\"$attrs" . ($checked ? ' checked="checked"' : '') . ($disabled ? ' disabled="disabled"' : '') . ' /> ';

                    $checkbox_flow = $properties->get('flow') == 'horizontal' ? ' form-check-inline' : '';
                    $field_str .= RsdirectoryAdapterGrid::checkbox($checkbox_input, $checkbox_label, $checkbox_flow);
                }
                    
                $field_str .= '</div><!-- .rsdir-items-group -->';


                // handle default credits
                $default_credits = $properties->get('credits');

                $skip_default = false;
                if ($entry_credits) {
                    // this check is done only if the entry is not expired
                    if (!$is_expired) {
                        foreach ($entry_credits as $object) {
                            if ($object->paid && $object->object_type == 'form_field' && $object->object_id == $form_field->id) {
                                $skip_default = true;
                                break;
                            }
                        }
                    }
                }

                if ($charge_fields) {
                    // check if there has been previously paid something
                    $unpaid = RSDirectoryHelper::findElements(array('object_type' => 'form_field', 'object_id' => $form_field->id, 'paid' => 0), $entry_credits);

                    if (!empty($credits_all)) {
                        // need to get the paid ones to
                        $paid = RSDirectoryHelper::findElements(array('object_type' => 'form_field', 'object_id' => $form_field->id, 'paid' => 1), $entry_credits);

                        // take the differences
                        $diff_total = 0;
                        $paid_diffs = RSDirectoryHelper::findElements(array('object_type' => 'form_field', 'object_id' => $form_field->id, 'paid' => 1, 'is_diff' => 1), $entry_credits, false);
                        if ($paid_diffs) {
                            foreach ($paid_diffs as $pd) {
                                $diff_total += (int) $pd->credits;
                            }
                        }

                        // the difference between the credits base on the selection and the already payed ones
                        $paid_credits = (!empty($paid->credits) ? (int)$paid->credits : 0) + $diff_total;

                        // init the cost based on the previous selection
                        $prev_credits = array();

                        // Initialize the current values.
                        $current_values = array();

                        if (!empty($entry) && !empty($default_items)) {
                            $current = null;
                            if (isset($entry->{$form_field->column_name})) {
                                $current = $entry->{$form_field->column_name} ? explode("\r\n", $entry->{$form_field->column_name}) : array();
                            }


                            if (!is_null($current)) {
                                foreach ($default_items as $v => $t) {
                                    foreach ($current as $val) {
                                        if ($v == $val) {
                                            $current_values[] = $v;

                                            if (isset($credits_all[$v])) {
                                                $prev_credits[] = (int)$credits_all[$v];
                                            }
                                        }
                                    }
                                }
                            }
                        }

                        // add to the prev credits the default credits
                        if (!empty($default_credits)) {
                            $prev_credits[] = (int) $default_credits;
                        }

                        // total of the credits previous selected
                        $prev_credits = !empty($prev_credits) ? array_sum($prev_credits) : 0;

                        // Mark the field as paid.
                        if (strpos($field_wrapper_attrs['class'], 'rsdir-paid-field') === false) {
                            $field_wrapper_attrs['class'] .= ' rsdir-paid-field';
                        }

                        $script = "form_fields[$form_field->id] = {" .
                            "credits: ".$default_credits."," .
                            "multiple_credits: " . json_encode($credits_all) . "," .
                            "current_values: " . ((!empty($current_values) && (!$unpaid || ($paid_credits != $prev_credits)) && !$is_expired) ? json_encode($current_values) : '[]') . "," .
                            "selected_values: ". ($default ? (is_array($default) ? json_encode($default) : '["' . $default . '"]') : '[]') . ",".
                            "field_type : '".$form_field->field_type."'," .
                            "credits_per_file: null" .
                            "}";

                        $doc->addScriptDeclaration($script);
                    } else {

                        $current = null;
                        if (!empty($entry) && isset($entry->{$form_field->column_name})) {
                            $current = $entry->{$form_field->column_name} ? explode("\r\n", $entry->{$form_field->column_name}) : array();
                        }

                        if (!$skip_default) {
                            $script = "form_fields[$form_field->id] = {" .
                                "credits: ".$default_credits."," .
                                "multiple_credits: []," .
                                "current_values: [". ((!is_null($current) && !$unpaid && !$is_expired) ? 1 : '') . "]," .
                                "selected_values: [" . ($default ? 1 : '') . "]," .
                                "field_type : '".$form_field->field_type."'," .
                                "credits_per_file: null" .
                                "}";

                            $doc->addScriptDeclaration($script);
                        }
                    }
                }
                    
                break;
                    
            case 'radiogroup':

                // Initiate the credits array
                $credits_all = array();
                    
                // Set the default value.
                if ( isset($data[$form_field->form_field_name]) )
                {
                    $default = $data[$form_field->form_field_name];
                }
                else if ( isset($entry->{$form_field->column_name}) )
                {
                    $default = $entry->{$form_field->column_name};
                }
                else
                {
                    $default = $properties->get('default_value');
                }
                    
                $this->setAttr('class', 'rsdir-radio');
                    
                if ($is_dependency_parent)
                {
                    $this->setAttr('class', 'rsdir-dependency');
                    $this->setAttr('data-id', $form_field->id);
                }
				
				if ($this->getAttr('disabled')) {
					$default_hidden_value = (array) $default;
					
					foreach ($default_hidden_value as $hidden_value) {
						$field_str .= '<input type="hidden" name="'.$name_attr.'" value="'.self::escapeHTML($hidden_value).'" />';
					}
				}
                    
                $field_str .= '<div class="rsdir-items-group">';
                    
                // Get dependency items.
                if ( $properties->get('dependency') && empty($dependency_options) )
                {
                    $field_str .= JText::_('COM_RSDIRECTORY_NOTHING_TO_SELECT');
                }


                // hold the cleared items, without flags
                $default_items = array();
                    
                foreach ($items as $i => $item)
                {
                    $value = $item->value;
					$text = $item->text;
                        
                    $disabled = false;
                        
                    if ( stripos($value, '[d]') !== false || stripos($text, '[d]') !== false )
                    {
                        $value = str_replace('[d]', '', $value);
                        $text = str_replace('[d]', '', $text);
                            
                        $disabled = true;
                    }

                    // check for credits in the value
                    if( preg_match('/\[c-\d+]/', $value, $credits) != false)
                    {
                        $value = str_replace($credits[0], '', $value);

                        // get the credits value
                        $credit_value = str_replace(array('[', ']', 'c-'), '', $credits[0]);
                        $credits_all[$value] = (int)$credit_value;
                    }

                    // check for credits in the text
                    if(preg_match('/\[c-\d+]/', $text, $credits))
                    {
                        $text = str_replace($credits[0], '', $text);

                        // get the credits value
                        $credit_value = str_replace(array('[', ']', 'c-'), '', $credits[0]);
                        // we need to save as reference the actual value
                        if (!isset($credits_all[$value])) {
                            $credits_all[$value] = (int)$credit_value;
                        }
                    }

                    $default_items[$value] = $text;
					
					$allow_label_html = (int) $properties->get('allow_html', 0);
                        
                    $checked = isset($default) && $value == $default;

                    $id = '';

                    if (RSDirectoryHelper::isJ4())
                    {
                        $current_id = $this->getAttr('id');
                        $this->removeAttr('id');
                        if (!empty($current_id))
                        {
                            $id = $current_id.$i;
                        } else {
                            $id = 'rsdir-radio-label'.$i;
                        }

                        $this->setAttr('id', $id);
                        $this->setAttr('class', 'form-check-input');
                    }

                    $attrs = $this->getAttrsString();

                    $radio_label = '<label'.(RSDirectoryHelper::isJ4() ? ' for="'.$id.'"' : '').' class="rsdir-radio-label'.(RSDirectoryHelper::isJ4() ? ' form-check-label' : '').' radio' . ( $properties->get('flow') == 'horizontal' ? ' inline' : '' ) . '">{rsdir_radio}'.($allow_label_html ? $text : self::escapeHTML($text)) . "</label>\n";
                    $radio_input = '<input type="radio" name="' . $name_attr . '" value="' . self::escapeHTML($value) . "\"$attrs" . ($checked ? ' checked="checked"' : '') . ($disabled ? ' disabled="disabled"' : '') . ' /> ';

                    $radio_flow = $properties->get('flow') == 'horizontal' ? ' form-check-inline' : '';
                    $field_str .= RsdirectoryAdapterGrid::checkbox($radio_input, $radio_label, $radio_flow);
                }
                    
                $field_str .= '</div><!-- .rsdir-items-group -->';

                // handle default credits
                $default_credits = $properties->get('credits');

                $skip_default = false;
                if ($entry_credits) {
                    // this check is done only if the entry is not expired
                    if (!$is_expired) {
                        foreach ($entry_credits as $object) {
                            if ($object->paid && $object->object_type == 'form_field' && $object->object_id == $form_field->id) {
                                $skip_default = true;
                                break;
                            }
                        }
                    }
                }

                if ($charge_fields) {
                    // check if there has been previously paid something
                    $unpaid = RSDirectoryHelper::findElements(array('object_type' => 'form_field', 'object_id' => $form_field->id, 'paid' => 0), $entry_credits);

                    if (!empty($credits_all)) {
                        // need to get the paid ones to
                        $paid = RSDirectoryHelper::findElements(array('object_type' => 'form_field', 'object_id' => $form_field->id, 'paid' => 1), $entry_credits);

                        // take the differences
                        $diff_total = 0;
                        $paid_diffs = RSDirectoryHelper::findElements(array('object_type' => 'form_field', 'object_id' => $form_field->id, 'paid' => 1, 'is_diff' => 1), $entry_credits, false);
                        if ($paid_diffs) {
                            foreach ($paid_diffs as $pd) {
                                $diff_total += (int) $pd->credits;
                            }
                        }

                        // the difference between the credits base on the selection and the already payed ones
                        $paid_credits = (!empty($paid->credits) ? (int)$paid->credits : 0) + $diff_total;

                        // init the cost based on the previous selection
                        $prev_credits = array();

                        // Initialize the current values.
                        $current_values = array();

                        if (!empty($entry) && !empty($default_items)) {
                            $current = null;
                            if (isset($entry->{$form_field->column_name})) {
                                $current = $entry->{$form_field->column_name};
                            }


                            if (!is_null($current)) {
                                foreach ($default_items as $v => $t) {
                                    if ($v == $current) {
                                        $current_values[] = $v;

                                        if (isset($credits_all[$v])) {
                                            $prev_credits[] = (int)$credits_all[$v];
                                        }
                                    }
                                }
                            }
                        }

                        // add to the prev credits the default credits
                        if (!empty($default_credits)) {
                            $prev_credits[] = (int) $default_credits;
                        }

                        // total of the credits previous selected
                        $prev_credits = !empty($prev_credits) ? array_sum($prev_credits) : 0;

                        // Mark the field as paid.
                        if (strpos($field_wrapper_attrs['class'], 'rsdir-paid-field') === false) {
                            $field_wrapper_attrs['class'] .= ' rsdir-paid-field';
                        }

                        $script = "form_fields[$form_field->id] = {" .
                            "credits: ".$default_credits."," .
                            "multiple_credits: " . json_encode($credits_all) . "," .
                            "current_values: " . ((!empty($current_values) && (!$unpaid || ($paid_credits != $prev_credits)) && !$is_expired) ? json_encode($current_values) : '[]') . "," .
                            "selected_values: ". ($default ? (is_array($default) ? json_encode($default) : '["' . $default . '"]') : '[]') . ",".
                            "field_type : '".$form_field->field_type."'," .
                            "credits_per_file: null" .
                            "}";

                        $doc->addScriptDeclaration($script);
                    } else {
                        $current = null;
                        if (!empty($entry) && isset($entry->{$form_field->column_name})) {
                            $current = $entry->{$form_field->column_name};
                        }

                        if (!$skip_default) {
                            $script = "form_fields[$form_field->id] = {" .
                                "credits: ".$default_credits."," .
                                "multiple_credits: []," .
                                "current_values: [". ((!is_null($current) && !$unpaid && !$is_expired) ? 1 : '') . "]," .
                                "selected_values: [" . ($default ? 1 : '') . "]," .
                                "field_type : '".$form_field->field_type."'," .
                                "credits_per_file: null" .
                                "}";

                            $doc->addScriptDeclaration($script);
                        }
                    }
                }
                    
                break;
                    
            case 'calendar':

                // Set the default value.
                if ( isset($data[$form_field->form_field_name]) )
                {
                    $default = $data[$form_field->form_field_name];
                }
                else if ( isset($entry->{$form_field->column_name}) )
                {
                    $datetime = explode(' ', $entry->{$form_field->column_name});
                    $date = explode('-', $datetime[0]); 
                        
                    if ( isset($date[2]) )
                    {
                        list($year, $month, $day) = $date;
                            
                        $default = "$month/$day/$year";
                    }
                    else
                    {
                        $default = '';
                    }
                }
                else
                {
                    $default = $properties->get('default_date');
                    $default = RSDirectoryHelper::escapeHTML($default);
                }
                
                if ($default == '00/00/0000')
                {
                    $default = '';
                }
                    
                $calendar_layout = $properties->get('calendar_layout');
                $calendar_id = 'rsdir-calendar-' . strtolower( str_replace('_', '-' , $form_field->name) );

                // Get the form mask option
                $form_date_mask = $properties->get('form_date_mask', 'mm/dd/yy');

                $this->setAttr('class', 'rsdir-calendar-input input-small');

                if (RSDirectoryHelper::isJ4())
                {
                    $this->setAttr('class', 'form-control');
                }

                if ( $properties->get('readonly') )
                {
                    $this->setAttr('readonly', 'readonly');
                }

                $jQuery_calendar_attrs = array(
                    'value'			=> (!isset($default) ? '' : $default),
                    'name' 			=> $name_attr,
                    'fake_id' 		=> $calendar_id. '-fake',
                    'id' 			=> $calendar_id,
                    'format_date' 	=> 'mm/dd/yy',
                    'date_mask'		=> $form_date_mask,
                    'layout'		=> $calendar_layout,
                    'fake_attributes' => $this->getAttrsString()
                );

                // Process the min date field.
                if ( $properties->get('min_date') )
                {
                    $jQuery_calendar_attrs['min_date'] = $properties->get('min_date');
                }

                // Process the max date field.
                if ( $properties->get('max_date') )
                {
                    $jQuery_calendar_attrs['max_date'] = $properties->get('max_date');
                }


                // Add the field to the DOM
                $field_str .= RSDirectoryHelper::render_jquery_calendar($jQuery_calendar_attrs);
                    
                break;
                    
            case 'dropdown_date_picker':
                    
                // Set the default value.
                if ( isset($entry->{$form_field->column_name}) )
                {
                    $date = explode( '-', substr( $entry->{$form_field->column_name}, 0, strpos($entry->{$form_field->column_name}, ' ') ) );
                        
                    if ( isset($date[2]) )
                    {
                        list($default_year, $default_month, $default_day) = $date;

                        // Remove leading 0.
                        $default_month = ltrim($default_month, 0);
                        $default_day = ltrim($default_day, 0);
                    }
                }
                    
                $start_year = $properties->get('start_year', 1900);
                $end_year = $properties->get('until_current_year') || !$properties->get('end_year') ? JFactory::getDate()->format('Y') : $properties->get('end_year');
                    
                $default = true;

                if (RSDirectoryHelper::isJ4())
                {
                    $this->setAttr('class', 'form-select');
                }
                else
                {
                    $this->setAttr('class', 'span3');
                }

                $field_str .= '<div class="row'.(!RSDirectoryHelper::isJ4() ? '-fluid' : '').'">';
                    
                // Build the year dropdown.
                if ( $properties->get('show_year_dropdown') )
                {
                    if ( isset($data[$form_field->form_field_name]['year']) )
                    {
                        $default_year = $data[$form_field->form_field_name]['year'];
                    }
                        
                    $default = empty($default_year) ? false : $default;
                        
                    // Initialize the years options array.
                    $options = array(
                        JHtml::_( 'select.option', 0, JText::_('COM_RSDIRECTORY_YEAR') ),
                    );
                        
                    for ($year = $end_year; $year >= $start_year; $year--)
                    {
                        $options[] = JHtml::_('select.option', $year, $year);
                    }

                    if (RSDirectoryHelper::isJ4())
                    {
                        $field_str .= '<div class="col">';
                    }
                        
                    $field_str .= JHtml::_(
                        'select.genericlist',
                        $options,
                        $name_attr . '[year]',
                        ' class="rsdir-year-dropdown ' . $this->getAttr('class') . '"',
                        'value',
                        'text',
                        empty($default_year) ? null : $default_year
                    );

                    if (RSDirectoryHelper::isJ4())
                    {
                        $field_str .= '</div>';
                    }
                }
                    
                // Build the month dropdown.
                if ( $properties->get('show_month_dropdown') )
                {
                    if ( isset($data[$form_field->form_field_name]['month']) )
                    {
                        $default_month = $data[$form_field->form_field_name]['month'];
                    }

                    $default = empty($default_month) ? false : $default;

                    // Initialize the months options array.
                    $options = array(
                        JHtml::_( 'select.option', 0, JText::_('COM_RSDIRECTORY_MONTH') ),
                    );

                    for ($month = 1; $month < 13; $month++)
                    {
                        $time = gmmktime(0, 0, 0, $month, 1, 1);

                        if ( $properties->get('month_format') == 2 )
                        {
                            $formated_month = JFactory::getDate($time)->format('F');
                        }
                        else if ( $properties->get('month_format') == 3 )
                        {
                            $formated_month = JFactory::getDate($time)->format('M');
                        }
                        else
                        {
                            $formated_month = $month < 10 ? "0$month" : $month;
                        }

                        $options[] = JHtml::_('select.option', $month, $formated_month);
                    }

                    if (RSDirectoryHelper::isJ4())
                    {
                        $field_str .= '<div class="col">';
                    }

                    $field_str .= JHtml::_(
                        'select.genericlist',
                        $options,
                        $name_attr. '[month]',
                        ' class="rsdir-month-dropdown ' . $this->getAttr('class') . '"',
                        'value',
                        'text',
                        empty($default_month) ? null : $default_month
                    );

                    if (RSDirectoryHelper::isJ4())
                    {
                        $field_str .= '</div>';
                    }
                }
                    
                // Build the day dropdown.
                if ( $properties->get('show_day_dropdown') )
                {
                    if ( isset($data[$form_field->form_field_name]['day']) )
                    {
                        $default_day = $data[$form_field->form_field_name]['day'];
                    }
                        
                    $default = empty($default_day) ? false : $default;
                        
                    // Initialize the days options array.
                    $options = array(
                        JHtml::_( 'select.option', 0, JText::_('COM_RSDIRECTORY_DAY') ),
                    );
                        
                    for ($day = 1; $day < 32; $day++)
                    {
                        $options[] = JHtml::_('select.option', $day, $day);
                    }

                    if (RSDirectoryHelper::isJ4())
                    {
                        $field_str .= '<div class="col">';
                    }
                        
                    $field_str .= JHtml::_(
                        'select.genericlist',
                        $options,
                        $name_attr . '[day]',
                        ' class="rsdir-day-dropdown ' . $this->getAttr('class') . '"',
                        'value',
                        'text',
                        empty($default_day) ? null : $default_day
                    );

                    if (RSDirectoryHelper::isJ4())
                    {
                        $field_str .= '</div>';
                    }
                }


                $field_str .= '</div>';

                    
                break;
                    
            case 'map':
                    
                // Reset the field prepend and append properties.
                $properties->set('field_prepend', null);
                $properties->set('field_append', null);
                    
                $address_column_name = "{$form_field->column_name}_address";
                $lat_column_name = "{$form_field->column_name}_lat";
                $lng_column_name = "{$form_field->column_name}_lng";
                    
                // Set the default value.
                if ( isset($data[$form_field->form_field_name]['address']) )
                {
                    $default_address = $data[$form_field->form_field_name]['address'];
                }
                else if ( isset($entry->$address_column_name) )
                {
                    $default_address = $entry->$address_column_name;
                }
                else
                {
                    $default_address = $properties->get('default_address');
                }
				
                if ( isset($data[$form_field->form_field_name]['lat'], $data[$form_field->form_field_name]['lng']) )
                {
                    $default_lat = (float) $data[$form_field->form_field_name]['lat'];
                    $default_lng = (float) $data[$form_field->form_field_name]['lng'];
                }
                else if ( isset($entry->$lat_column_name, $entry->$lng_column_name) )
                {
                    $default_lat = (float) $entry->$lat_column_name;
                    $default_lng = (float) $entry->$lng_column_name;
                }
                else
                {
                    $default_lat = (float) $properties->get('default_lat');
                    $default_lng = (float) $properties->get('default_lng');
                }
				
				if (!strlen($default_address) && (empty($default_lat) || empty($default_lng))) {
					$default = false;
				} else {
					$default = true;
				}
                    
                $this->setAttr('class', 'rsdir-textbox rsdir-map-address');

                if (RSDirectoryHelper::isJ4())
                {
                    $this->setAttr('class', 'form-control');
                }

                $id = "rsdir-$form_field->form_field_name";
                $this->setAttr('id', "$id-address");
                    
                if ( $properties->get('show_address_input', 1) )
                {
                    // ADDRESS
                    $field_str .= '<div class="rsdir-map-field">';

                    $input = '<input type="text" name="' . $name_attr . '[address]"' . ( $default_address ? ' value="' . self::escapeHTML($default_address) . '"' : '' ) . $this->getAttrsString() ." autocomplete=\"off\" />\n";

                    $field_str .= RsdirectoryAdapterGrid::inputPrepend($input, JText::_('COM_RSDIRECTORY_ADDRESS'));

                    $field_str .= '</div>';
                }
                else
                {
                    $field_str .= '<input type="hidden" name="' . $name_attr . '[address]"' . ( $default_address ? ' value="' . self::escapeHTML($default_address) . '"' : '' ) . $this->getAttrsString() ." />\n";
                }
                
				// No longer need this as each field will have its own ID now
				$this->removeAttr('id');
				
				// check if the default lat/lng are not 0
				if ((float)$default_lat == 0) {
					$default_lat = '';
				}
				if ((float)$default_lng == 0) {
					$default_lng = '';
				}
				
                if ( $properties->get('show_coordinates_inputs', 1) )
                {
					$this->setAttr('class', 'input-medium');
					
                    $field_str .= '<div class="rsdir-map-field">';
                    if (RSDirectoryHelper::isJ4())
                    {
                        $field_str .= '<div class="row gx-1"><div class="col gx-0">';
                    }

                    // LATITUDE
                    $input = '<input id="' . $id . '-lat" type="text" name="' . $name_attr . '[lat]"' . ( $default_lat ? ' value="' . self::escapeHTML($default_lat) . '"' : '' ) . $this->getAttrsString() . ' />';

                    $field_str .= RsdirectoryAdapterGrid::inputPrepend($input, JText::_('COM_RSDIRECTORY_LATITUDE'));

                    if (RSDirectoryHelper::isJ4())
                    {
                        $field_str .= '</div><div class="col">';
                    }

                    // LONGITUDE
                    $input = '<input id="' . $id . '-lng" type="text" name="' . $name_attr . '[lng]"' . ( $default_lng ? ' value="' . self::escapeHTML($default_lng) . '"' : '' ) . $this->getAttrsString() . ' />';

                    $field_str .= RsdirectoryAdapterGrid::inputPrepend($input, JText::_('COM_RSDIRECTORY_LONGITUDE'));

                    if (RSDirectoryHelper::isJ4())
                    {
                        $field_str .= '</div>';
                    }

                    $field_str .= '</div>';
                }
                else
                {					
                    $field_str .= '<input id="' . $id . '-lat" type="hidden" name="' . $name_attr . '[lat]"' . ( $default_lat ? ' value="' . self::escapeHTML($default_lat) . '"' : '' ) . $this->getAttrsString() . ' />';
                    $field_str .= '<input id="' . $id . '-lng" type="hidden" name="' . $name_attr . '[lng]"' . ( $default_lng ? ' value="' . self::escapeHTML($default_lng) . '"' : '' ) . $this->getAttrsString() . ' />';   
                }
                    
                $style = array();
                    
                $width = $properties->get('width', '100%');
                    
                if ( is_numeric($width) )
                {
                    $width .= 'px';
                }
                    
                $style[] = "width: $width;";
                    
                $style[] = 'height: ' . $properties->get('height') . 'px;';
                    
                $after_str .= '<div id="' . $id . '-canvas" class="rsdir-map" style="' . implode(' ', $style) . '"></div>';

                $config  = RSDirectoryConfig::getInstance();
                $map_type = $config->get('map_type', 'google');
                
				if ( isset($data[$form_field->form_field_name]['lat'], $data[$form_field->form_field_name]['lng']) || isset($entry->$lat_column_name, $entry->$lng_column_name) || !$properties->get('use_geolocation') )
				{
					$geocoder = '';
				}
				else
                {
                    $geocoder = 'try
					{
						if (navigator.geolocation) {
							var map = jQuery( document.getElementById("' . $id . '-canvas") ).data("'.($map_type == 'google' ? 'rsMap' : 'rsOMap').'");
							navigator.geolocation.getCurrentPosition(map.geoLocationProcess);
						}
					} catch (err) {}';
                }

                // If is the edit form
                if (!is_null($entry))
                {
                    $map_icon = $this->getMapicon($entry->category_id);
                } //if is the add entry form
                else
                {
                    $category_id = $jinput->getInt('category_id');
                    $map_icon = $this->getMapicon($category_id);
                }

                $map_params = array(
                    'map_id'            => $id . '-canvas',
                    'default_zoom'      => $properties->get('default_zoom'),
                    'inputAddress'	    => 'document.getElementById("' . $id . '-address")',
                    'inputLat'		    => 'document.getElementById("' . $id . '-lat")',
                    'inputLng'		    => 'document.getElementById("' . $id . '-lng")',
                    'markerDraggable'   => ( !$isRoot && ($is_site || !$can_edit_all_entries) && $properties->get('readonly') ? 'false' : 'true' )
                );

                if (!empty($map_icon)) {
                    $map_params['icon_params'] = $map_icon;
                }

                // Add the map
                RSDirectoryHelper::loadMap($map_params, $geocoder);
                break;
                
            case 'free_text':
                    
                $field_str .= '<div' . $this->getAttrsString() . '>' . $properties->get('text') . '</div>';
                    
                break;
        }
            
        // Add a loader gif after the field if it is a dependency parent.
        if ( !empty($is_dependency_parent) )
        {
            $after_str .= JHtml::_('image', 'com_rsdirectory/loader.gif', '', 'class="rsdir-loader hide" width="16" height="11"', true);
        }
          
        if ( ($is_site || !$can_edit_all_entries) && ( $properties->get('credits') || $properties->get('credits_per_file') ) )
        {
		
            $add_class_attr = true;
                
            // Initialize the credits value.
            $credits = $properties->get('credits');
            
			// need to know for the files and images fields
			$is_paid = false;
			$is_paid_files = false;
            if ($entry_credits)
            {
                foreach ($entry_credits as $object)
                {
                    if ( $object->paid && ($object->object_type == 'form_field' || $object->object_type == 'uploaded_file') && $object->object_id == $form_field->id)
                    {
                        if ( in_array( $form_field->field_type, array('images', 'image_upload', 'fileupload') ) )
                        {
                            if(!$is_expired) {
                                $credits = 0;
                                if ($object->object_type == 'form_field')
                                {
                                    $is_paid = true;
                                }
                                else
                                {
                                    $is_paid_files = true;
                                }
                            }

                            if ( $properties->get('credits_per_file') )
                                break;
                        }

                        $add_class_attr = false;

                        break;
                    }
                }
            }

            if (($add_class_attr && is_null($entry)) || ($add_class_attr && !is_null($entry) && !$is_expired)  || (!is_null($entry) && $is_expired && $renew_types > 0))
            {
				
                // Mark the field as paid.
                if (strpos($field_wrapper_attrs['class'], 'rsdir-paid-field') === false) {
                    $field_wrapper_attrs['class'] .= ' rsdir-paid-field';
                }

                // detect multiple array
                $multiple_array_fields = array('dropdown', 'country', 'checkboxgroup', 'radiogroup');

                if (!in_array($form_field->field_type, $multiple_array_fields)) {

                    $script = "form_fields[$form_field->id] = {" .
                        "credits: ".$credits."," .
                        "multiple_credits: null,";

                    if ( in_array( $form_field->field_type, array('images', 'image_upload', 'fileupload') ) )
                    {
                        if ((!$is_expired && !is_null($entry) && ($is_paid || !empty($files_loaded)))) {
                            // if there are files loaded to ignore them for the credit count they must be paid
                            $script .= 'current_values: [' . ($files_loaded ? ((!$is_paid && !$is_paid_files)  ? '"exist"' : $files_loaded) : '') . '],';
                        } else if($is_expired && !is_null($entry)){
							 $script .= 'current_values: [' . ($files_loaded ? '"exist"' : '') . '],';
                        } else {
                            $script .= 'current_values: [],';
                        }
                        $script .= "credits_per_file: " . $properties->get('credits_per_file') . ",".
                            "field_type : '".$form_field->field_type."',";
                        // need to send the id for the script to run ok
                        $script .= 'field_id: '.$form_field->id.'}';

                    } else {
                        $script .= "current_values: [" . ($default ? 1 : '') . "]," .
                            "credits_per_file: null,".
                            "field_type : '".$form_field->field_type."'}";
                    }
                    $doc->addScriptDeclaration($script);
                }
            }
        }
            
        $field_prepend = $properties->get('field_prepend');
        $field_append = $properties->get('field_append');
            
        if ( !empty($is_dependency_parent) )
        {
            $field_wrapper_attrs['class'] .= ' dependency-parent';
        }
            
        if ( $properties->get('dependency') )
        {
            $field_wrapper_attrs['class'] .= ' dependency-child';
            $field_wrapper_attrs['data-parent-id'] = $properties->get('dependency');
        }
            
        $field_wrapper_attrs['data-id'] = $form_field->id;
            
        $str = '<div';
            
        foreach ($field_wrapper_attrs as $attr => $value)
        {
            $str .= " $attr=\"$value\"";
        }
            
        $str .= '>';
        // Hold the help text class
        $help_text_cls = '';

        if ( $properties->get('show_help_text') && $properties->get('help_text') )
        {
            $help_text_cls = $properties->get('help_text_position') == 'inline' ? 'help-inline' : 'help-block';
        }

        // if Joomla 4 and the help text is inline need some changes to the layout
        if (!empty($help_text_cls) && $help_text_cls == 'help-inline' && RSDirectoryHelper::isJ4())
        {
            $str .= '<div class="row g-3 align-items-end"><div class="col-md-8">';
        }

        $str .= $label_str;
        if ( $properties->get('show_help_tip') && $properties->get('help_tip') )
        {
            $str .= ' <i class="rsdir-toggle-help fa fa-question-circle '.RSDirectoryHelper::getTooltipClass(true).'" title="' . self::escapeHTML( $properties->get('help_tip') ) . '"></i>';
        }
        $str .= $before_str;
            
        if ($field_prepend || $field_append)
        {
            $cls = array();

            if (($field_prepend || $field_append) && RSDirectoryHelper::isJ4())
            {
                $cls[] = 'input-group';
            }
                
            if ($field_prepend && !RSDirectoryHelper::isJ4())
            {
                $cls[] = 'input-prepend';
            }
                
            if ($field_append && !RSDirectoryHelper::isJ4())
            {
                $cls[] = 'input-append';
            }
                
            $str .= '<div class="' . implode(' ', $cls) . '">';
                
            if ($field_prepend)
            {
                $str .= '<div class="'.(RSDirectoryHelper::isJ4() ? 'input-group-text' : 'add-on').'">' . self::escapeHTML($field_prepend) . '</div>';
            }
                
            $str .= $field_str;
                
            if ($field_append)
            {
                $str .= '<div class="'.(RSDirectoryHelper::isJ4() ? 'input-group-text' : 'add-on').'">' . self::escapeHTML($field_append) . '</div>';
            }
                
            $str .= '</div>';
        }
        else
        {
            $str .= $field_str;
        }

        if ((!empty($help_text_cls) && !RSDirectoryHelper::isJ4()) || ($help_text_cls == 'help-block' && RSDirectoryHelper::isJ4()))
        {
            $str .= '<span class="' . $help_text_cls . '">' . self::escapeHTML( $properties->get('help_text') ) . '</span>';
        }

        if (!empty($help_text_cls)  && $help_text_cls == 'help-inline' && RSDirectoryHelper::isJ4())
        {
            $str .= '</div>';
            $str .= '<div class="col-md-4"><span class="' . $help_text_cls . '">' . self::escapeHTML( $properties->get('help_text') ) . '</span></div>';
            $str .= '</div>';
        }
            
        $str .= $after_str;
            
        $str .= "</div><!-- .rsdir-field-wrapper-$form_field->id -->\n";
            
        return $str;
    }

    /**
     * Method to get the category map marker.
     *
     * @access protected
     *
     * @param int $category_id
     *
     * @return array
     */
    protected function getMapicon($category_id) {
        static $map_icon;

        if (is_null($map_icon)) {
            $map_icon = array();
            $category = RSDirectoryHelper::getCategory($category_id);

            if (!empty($category->params)) {
                $cat_params = json_decode($category->params);
                if (!empty($cat_params->map_icon)) {
                    $icon = RSDirectoryHelper::getCategoryMapIconObject($cat_params->map_icon);
                    if ($icon) {
                        $map_icon['icon'] 		= RSDirectoryHelper::getImageURL($icon->hash, 'mapicon');
                        $map_icon['w'] 			= RSDirectoryConfig::getInstance()->get('mapicon_thumbnail_width');
                        $map_icon['h'] 			= RSDirectoryConfig::getInstance()->get('mapicon_thumbnail_height');
                        $map_icon['shadow'] 	= JUri::root() . "media/com_rsdirectory/images/leaflet/marker-shadow.png";
                        $map_icon['icon_anchor_x'] 	= ((int) $map_icon['w']) / 2;
                        $map_icon['icon_anchor_y'] 	= $map_icon['h'];
                        $map_icon['popup_anchor_y'] = -$map_icon['h'];
                    }
                }
            }
        }

        return $map_icon;
    }
        
    /**
     * Method to get dependency options.
     *
     * @access public
     *
     * @param mixed $dependency
     *
     * @return mixed
     */
    public function getDependencyOptions($dependency)
    {
        if (!$dependency)
            return;
            
        // Get the posted data.
        $data = JFactory::getApplication()->getUserState('com_rsdirectory.edit.entry.data');
            
        // Get dependency items.
        $parent = RSDirectoryHelper::getField($dependency);
		
		if (!$parent)
		{
			return;
		}
            
        if ( isset($data[$parent->form_field_name]) )
        {
            $parent_value = $data[$parent->form_field_name];
        }
        else if ( isset($this->entry->{$parent->column_name}) )
        {
            $parent_value = $this->entry->{$parent->column_name};
        }
        else
        {
            $parent_value = $parent->properties->get('default_value') ? $parent->properties->get('default_value') : '';
        }
            
        if ( isset($parent_value) )
        {
            $dependencies = RSDirectoryHelper::getDependencies(0, $dependency, $parent_value);

            if ( isset($dependencies[0]) )
            {
				foreach ($dependencies as $properties)
				{
					if ($this->form_field->id == $properties->field_id)
					{
						return RSDirectoryHelper::getOptions(RSDirectoryHelper::isCode($properties->items));
					}
				}
				
				// Fallback?
                return RSDirectoryHelper::getOptions(RSDirectoryHelper::isCode($dependencies[0]->items));
            }
        }
    }
        
    /**
     * Get the CAPTCHA field.
     *
     * @access
     *
     * @static
     *
     * @return mixed
     */
    public static function getCaptchaField()
    {
        // Get the RSDirectory! configuration.
        $config = RSDirectoryConfig::getInstance();
		
		$captcha = $config->get('captcha_type');
        
        // Get the captcha error.
        $error = JFactory::getApplication()->getUserState('com_rsdirectory.edit.entry.captcha_error');

        $str = RSDirectoryHelper::buildCaptcha('add_entry', '<div class="rsdir-field-wrapper rsdir-captcha-wrapper '.RsdirectoryAdapterGrid::controlGroup().' clearfix' . ($error ? ' error' : '') . '"><label class="rsdir-label">'.JText::_('COM_RSDIRECTORY_CAPTCHA_LABEL').'</label>%s</div>');

        return !empty($str) ? $str : null;
    }
        
    /**
     * Get the HTML code of a images list.
     *
     * @access public
     *
     * @static
     *
     * @param array $files_list
     * @param int $enable_ordering
     * @param int $enable_delete
	 * @param array $image_options
     *
     * @return mixed
     */
    public static function getImagesList($files_list, $enable_ordering = 1, $enable_delete = 1, $image_options = array())
    {
        // Do a few checks.
        if ( !$files_list || !is_array($files_list) )
            return;
            
        JText::script('COM_RSDIRECTORY_FILE_DELETION_CONFIRMATION');
            
        // Get the document object.
        $doc = JFactory::getDocument();
        
		// Add Magnific Popup script
		RSDirectoryHelper::loadMagnificPopup();
		
        // Get an instance of the RSDirectoryConfig object.
        $config = RSDirectoryConfig::getInstance();
            
        // Get the width of the small thumbnail.
        $width = $config->get('small_thumbnail_width');
            
        // Get the height of the small thumbnail.
        $height = $config->get('small_thumbnail_height');
            
        $gallery_class = 'rsdir-gallery-' . $files_list[0]->field_id . ' rsdir-images-list rsdir-files clearfix';
            
        if ($enable_ordering)
        {
            $gallery_class .= ' rsdir-order-files';
        }
            
        $gallery_id = 'rsdir-gallery-' . RSDirectoryHelper::randStr();
            
        // Build the files list.
        $str = '<ul id="' . $gallery_id . '" class="' . $gallery_class . '">';
            
        $only_path = '';
            
        foreach ($files_list as $i => $file)
        {
            // Set the src.
            $src = RSDirectoryHelper::getImageURL($file->hash, 'small');
                
            // Set the alt.
            $alt = JFile::stripExt($file->original_file_name);
                
            // Set the href.
            $href = RSDirectoryHelper::getImageURL($file->hash);

            if (!empty($image_options) && isset($image_options['nr']) && $image_options['nr'] == $i) {

                switch ($image_options['type'])
                {
                    case 'small':
                        $only_path = $src;
                    break;
                    case 'big':
                        $only_path = RSDirectoryHelper::getImageURL($file->hash, 'big');
                    break;
                    case 'normal':
                        $only_path = $href;
                    break;
                }

                if (!empty($only_path))
                {
                    continue;
                }
            }
                
            $str .= '<li class="rsdir-file">';
                
            $str .= '<a class="rsdir-img" href="' . $href . '">';    
            $str .= '<img src="' . $src . '" alt="' . self::escapeHTML($alt) . '" width="' . $width . '" height="' . $height . '" />';
            $str .= '</a>';
                
            if ($enable_delete)
            {
                $str .= '<span class="rsdir-delete-file btn btn-mini"><i class="rsdir-icon-x" title="' . JText::_('COM_RSDIRECTORY_DELETE_IMAGE') . '"></i></span>';
                $str .= JHtml::_('image', 'com_rsdirectory/loader.gif', '', 'class="rsdir-image-loader hide" width="16" height="11"', true);
            }
                
            if ($enable_ordering || $enable_delete)
            {
                $str .= '<input class="rsdir-file-pk" type="hidden" value="' . $file->id . '" />';
            }
                
            $str .= '</li>';
        }

        if (!empty($image_options))
        {
            return $only_path;
        }

        $str .= '</ul>';
            
        return $str;
    }
        
    /**
     * Output the user details fields.
     *
     * @access public
     *
     * @static
     */
    public static function outputRegFields()
    {
        // Get mainframe.
        $app = JFactory::getApplication();
            
        // Get the posted data.
        $data = $app->getUserState('com_rsdirectory.edit.entry.data');
            
        // Get the names of the registration fields that contain errors.
        $error_reg_fields = $app->getUserState('com_rsdirectory.edit.entry.error_reg_fields');

        $form = RSDirectoryHelper::getUserDetailsForm('fields');

        foreach ($form->getFieldsets() as $fieldset)
        {

            if ($fields = $form->getFieldset($fieldset->name))
            {
                foreach ($fields as $field)
                {
                    $options = array();
                    if (is_array($error_reg_fields) && in_array($field->fieldname, $error_reg_fields))
                    {
                        $options['class'] = 'error';
                    }
                    // set the value from the post data
                    $field->value = !empty($data['reg'][$field->fieldname]) ? $data['reg'][$field->fieldname] : '';

                    echo $field->renderField($options);
                }
            }
        }
    }
        
    /**
     * Get the HTML code of a files list.
     * 
     * @access public
     *
     * @static
     *
     * @param array $files_list
     * @param int $enable_ordering
     * @param int $enable_delete
     *
     * @return mixed
     */
    public static function getFilesList($files_list, $enable_ordering = 1, $enable_delete = 1)
    {
        // Do a few checks.
        if ( !$files_list || !is_array($files_list) )
            return;
            
        JText::script('COM_RSDIRECTORY_FILE_DELETION_CONFIRMATION');
            
        // Build the files list.
        $str = '<table class="rsdir-files-list rsdir-files table table-striped">';
            
        $str .= '<tbody' . ($enable_ordering ? ' class="rsdir-order-files"' : '') . '>';
            
        foreach ($files_list as $i => $file)
        {
            $href = RSDirectoryHelper::absJRoute("index.php?option=com_rsdirectory&task=file.download&hash=$file->hash");
                
            $str .= '<tr class="rsdir-file"><td>';
                
            if ($enable_delete)
            {
                $str .= '<i class="rsdir-icon-x pull-right rsdir-delete-file" title="' . JText::_('COM_RSDIRECTORY_DELETE_FILE') . '"></i>';
                $str .= JHtml::_('image', 'com_rsdirectory/loader.gif', '', 'class="rsdir-image-loader hide pull-right" width="16" height="11"', true);
            }
                
            $str .= '<a href="' . $href . '">' . self::escapeHTML($file->original_file_name) . '</a>';
                
            if ($enable_ordering || $enable_delete)
            {
                $str .= '<input class="rsdir-file-pk" type="hidden" value="' . $file->id . '" />';
            }
                
            $str .= '</td></tr>';
        }
            
        $str .= '</tbody>';
            
        $str .= '</table>';
            
        return $str;
    }
        
    /**
     * Escape HTML.
     *
     * @access public
     * 
     * @static
     * 
     * @param string $string
     * 
     * @return string
     */
    public static function escapeHTML($string)
    {
        return htmlentities($string, ENT_COMPAT, 'UTF-8');
    }
}