<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');


/**
 * Invoice generator class.
 */
class RSDirectoryInvoice
{
    protected $document;
    protected $user_id;

    protected $transaction_id;

    public function __construct($transaction_id = null)
    {
        if (empty($transaction_id)) {
            throw new Exception(JText::_('COM_RSDIRECTORY_INVOICE_NOT_AVAILABLE'));
        }

        $this->transaction_id = $transaction_id;
    }

    public static function getInstance($transaction_id = null) {
        static $invoice_instances = array();

        if (!isset($invoice_instances[$transaction_id])) {
            $invoice_instances[$transaction_id] = new RSDirectoryInvoice($transaction_id);
        }

        return $invoice_instances[$transaction_id];
    }

    /**
     * Method to get a table object, load it if necessary.
     *
     * @access public
     *
     * @param string $type
     * @param string $prefix
     * @param array $config
     *
     * @return object
     */
    public function getTable( $type = 'Transaction', $prefix = 'RSDirectoryTable', $config = array() )
    {
        JTable::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_rsdirectory/tables');

        return JTable::getInstance($type, $prefix, $config);
    }

    /**
     * Method to set an User ID. Used only in the administration part
     *
     * @access public
     *
     * @param int $user_id The id of the User ID.
     *
     */
    public function setUserId($user_id) {
        $this->user_id = $user_id;
    }

    /**
     * Method to get a single record.
     *
     * @access public
     *
     *
     * @return mixed Object on success, false on failure.
     */
    public function getItem()
    {
        // If the access is made from the front end we need to overwrite the user_id
        if (JFactory::getApplication()->isClient('site')) {
            $this->user_id = JFactory::getUser()->id;
        }

        $table = $this->getTable();
        $table->load( array( 'id' => $this->transaction_id, 'user_id' => $this->user_id ) );

        return $table->id ? $table : null;
    }

    /**
     * Method to get the invoice template.
     *
     * @access public
     *
     *
     * @return mixed Object
     */
    public function getTemplate()
    {
        $template 	 = RSDirectoryConfig::getInstance()->get('general_invoice_layout', '');

        if (!empty($template))
        {

            if ($transaction = $this->getItem())
            {
                require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/placeholders.php';

                $config = RSDirectoryConfig::getInstance();
                $title = $config->get('general_invoice_title', JText::_('COM_RSDIRECTORY_INVOICE_TITLE'));

                $entry = new stdClass();
                $entry->skip_entry = true;
                $entry->user_id = JFactory::getUser()->id;

                $custom_fields = array(
                    'invoice-transaction-table' => $this->buildTransactionTable($transaction),
                    'invoice-id'		 => str_pad($transaction->id, $config->get('general_invoice_padding', 0), 0, STR_PAD_LEFT),
                    'transaction-id'	 => $transaction->id,
                    'price-credits'		 => $this->priceFormat($transaction->price),
                    'tax'				 => $this->priceFormat($transaction->tax),
                    'tax-type' 			 => $transaction->tax_type,
                    'tax-value' 		 => $this->priceFormat($transaction->tax_value),
                    'total' 			 => $this->priceFormat($transaction->total),
                    'credit-title' 		 => $transaction->credit_title,
                    'currency' 			 => $transaction->currency,
                    'currency_sign' 	 => $config->get('currency_sign', ''),
                    'credits' 			 => $transaction->credits,
                    'payment-method' 	 => $transaction->gateway,
                    'date-purchased' 	 => $transaction->date_created
                );

                // handle placeholders for the title
                $this->document = new stdClass();
                $this->document->title = RSDirectoryPlaceholders::getInstance($title, array(), $entry, null, $custom_fields)->process();

                // handle placeholders for the invoice content
                $template = RSDirectoryPlaceholders::getInstance($template, array(), $entry, null, $custom_fields)->process();
            }
        }

        return $template;
    }

    /**
     * Method convert price using just the decimal and thousands separator.
     *
     * @access protected
     *
     *
     * @return string
     */
    protected function priceFormat($price) {
        $config = RSDirectoryConfig::getInstance();
        $decimals 				= $config->get('decimals');
        $decimal_point 			= $config->get('decimal_point');
        $thousands_separator 	= $config->get('thousands_separator');

       return number_format($price, $decimals, $decimal_point, $thousands_separator);
    }

    /**
     * Method to get the document related object.
     *
     * @access public
     *
     *
     * @return mixed Object
     */
    public function getDocument()
    {
        return $this->document;
    }


    /**
     * Method to build the transaction table for the proper placeholder.
     *
     * @access protected
     *
     *
     * @return string
     */
    protected function buildTransactionTable($transaction)
    {
        // build the price/total accordingly
        $price = RSDirectoryHelper::formatPrice($transaction->price);
        $total = RSDirectoryHelper::formatPrice($transaction->total);

        $table = '<table width="100%" border="1" cellpadding="3" cellspacing="1" class="invoice_table">
                <thead>
                    <tr>
                        <th class="package">'.JText::_('COM_RSDIRECTORY_INVOICE_CREDIT_PACKAGE').'</th>
                        <th class="price">'.JText::_('COM_RSDIRECTORY_INVOICE_PRICE').'</th>
                        <th class="quantity">'.JText::_('COM_RSDIRECTORY_INVOICE_QTY').'</th>
                        <th class="total">'.JText::_('COM_RSDIRECTORY_INVOICE_TOTAL').'</th>
                    </tr>
                </thead>
                <tbody>
                    <tr>
                        <td class="package">'.RSDirectoryHelper::escapeHTML($transaction->credit_title).'</td>
                        <td class="price">'.RSDirectoryHelper::escapeHTML($price).'</td>
                        <td class="quantity">1</td>
                        <td class="total">'.RSDirectoryHelper::escapeHTML($price).'</td>
                    </tr>
                    <tr>
                        <td colspan="3" class="subtotal">'.JText::_('COM_RSDIRECTORY_INVOICE_SUBTOTAL').'</td>
                        <td class="subtotal_value">'.RSDirectoryHelper::escapeHTML($price).'</td>
                    </tr>';

        if ($transaction->tax > 0)
        {
            $format_tax = RSDirectoryHelper::formatPrice($transaction->tax);
            $table .='<tr>
                        <td colspan="3" class="tax">'.JText::_('COM_RSDIRECTORY_INVOICE_TAX').' '.($transaction->tax_type != 'fixed' ? RSDirectoryHelper::escapeHTML($transaction->tax_value).' %' : '').'</td>
                        <td class="tax_value">'.RSDirectoryHelper::escapeHTML($format_tax).'</td>
                    </tr>';
        }

        $table .='<tr>
                        <td class="grand total" colspan="3" align="right">'.JText::_('COM_RSDIRECTORY_INVOICE_GRAND_TOTAL').'</td>
                        <td class="grand total">'.RSDirectoryHelper::escapeHTML($total).'</td>
                    </tr>
                </tbody>
            </table>';


        return $table;
    }
}