<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

class RSDirectoryRoute
{
	/**
	 * RSDirectoryRoute instance.
	 *
	 * @access private
	 *
	 * @static
	 *
	 * @var RSDirectoryRoute
	 */
	private static $instance;
		
	/**
	 * Base route.
	 */
	const BASE_ROUTE = 'index.php?option=com_rsdirectory';
		
	/**
	 * Constructor.
	 *
	 * @access private
	 */
	private function __construct()
	{
		$this->router = JApplicationCms::getInstance('site')->getRouter();
	}
		
	/**
	 * Method to get RSDirectoryRoute instance.
	 *
	 * @access public
	 *
	 * @static
	 *
	 * @return RSDirectoryRoute
	 */	
	public static function getInstance()
	{
		// Create a new instance of RSDirectoryRoute.
		if ( empty(self::$instance) )
		{
			self::$instance = new RSDirectoryRoute;
		}
			
		return self::$instance;
	}
		
	/**
	 * Method to get a humanly readible URL from an internal Joomla URL.
	 *
	 * @access public
	 *
	 * @param string $route
	 * @param bool $absolute
	 * @param bool $xhtml
	 *
	 * @return string
	 */
	public function getParsedURL($route, $absolute = false, $xhtml = true)
	{
		$router = $this->router->build($route);
			
		$url = $router->toString();
        $url = str_replace('/administrator', '', $url);
		 
		if ($absolute)
		{
			$base = str_replace( '/administrator', '', JUri::base(true) );
			$url = JUri::root() . substr( $url, strlen($base) + 1 );
		}
			
		if ($xhtml)
		{
			$url = htmlspecialchars($url);
		}
			
		return $url;
	}


	/**
	 * Method get the first match ItemId for the List entries menu type.
	 *
	 * @access public
	 *
	 * @static
	 *
	 * @return string
	 */
	public static function getListEntriesBestMatch() {
		static $items;
		if ($items === null) {
			$items = array();

			if ($menus = JFactory::getApplication()->getMenu('site')) {
				$component  = JComponentHelper::getComponent('com_rsdirectory');
				$attributes = array('component_id');
				$values     = array($component->id);
				$items 		= $menus->getItems($attributes, $values);
			}
		}


		if ($items) {
			foreach ($items as $item) {
				// Skip all other menu items that are not List Entries
				if ($item->query['view'] != 'entries') {
					continue;
				}

				// Found a match
				return $item->id;
			}
		}

		return '';
	}
		
	/**
	 * Method to generate an entry URL.
	 *
	 * @access public
	 *
	 * @static
	 *
	 * @param int $id
	 * @param string $title
	 * @param string $layout
	 * @param int $Itemid
	 * @param bool $absolute
	 * @param bool $xhtml
	 *
	 * @return string
	 */
	public static function getEntryURL($id, $title, $layout = 'default', $Itemid = 0, $absolute = false, $xhtml = true)
	{
		$layout = empty($layout) ? 'default' : $layout;
			
		if (RSDirectoryConfig::getInstance()->get('advanced_sef') && !$Itemid && $layout == 'default') {
			$Itemid = self::getEntryBestMatch($id);
		}

		$route = self::BASE_ROUTE . "&view=entry&layout=$layout&id=" . self::sef($id, $title) . ($Itemid ? "&Itemid=$Itemid" : '');
		
		return self::getInstance()->getParsedURL($route, $absolute, $xhtml);
	}
	
	protected static function getEntryBestMatch($id) {
		static $items;
		
		if ($items === null) {
			$items = array();
			
			if ($menus = JFactory::getApplication()->getMenu('site')) {
				$component  = JComponentHelper::getComponent('com_rsdirectory');
				$attributes = array('component_id');
				$values     = array($component->id);
				$items 		= $menus->getItems($attributes, $values);
			}
			
			JTable::addIncludePath(JPATH_ADMINISTRATOR.'/components/com_rsdirectory/tables');
		}
		
		if ($items) {
			$entry = JTable::getInstance('Entry', 'RSDirectoryTable');
			if (!$entry->load($id)) {
				return 0;
			}
			
			$matches = array(
				'entry' 					 => false,
				'category' 					 => false,
				'multiple_categories'		 => false,
				'parent_category'			 => array(),
				'multiple_parent_categories' => array(),
				'user_category'				 => false,
				'user_multiple_categories'	 => false,
				'user'						 => false,
				'all_categories'			 => false
			);
			
			foreach ($items as $item) {
				if (isset($item->query) && is_array($item->query) && isset($item->query['view']) && ($item->query['view'] == 'entries' || $item->query['view'] == 'entry')) {
					// Found a menu item with the exact entry setup
					if ($item->query['view'] == 'entry' && isset($item->query['id']) && $item->query['id'] == $entry->id) {
						$matches['entry'] = $item->id;
						continue;
					}
					
					if ($item->query['view'] == 'entries') {						
						$featured_categories = RSDirectoryHelper::arrayInt($item->getParams()->get('featured_categories'));
						
						if (!empty($item->query['user'])) {
							if ($item->query['user'] == $entry->user_id) {
								if (count($featured_categories) == 1 && in_array($entry->category_id, $featured_categories)) {
									$matches['user_category'] = $item->id;
								} elseif (in_array($entry->category_id, $featured_categories)) {
									$matches['user_multiple_categories'] = $item->id;
								} else {
									$matches['user'] = $item->id;
								}
							}
							
							continue;
						}
						
						// Found best match - a menu item with the exact category
						if (count($featured_categories) == 1 && in_array($entry->category_id, $featured_categories)) {
							$matches['category'] = $item->id;
							continue;
						}
						
						// Found second best match - a menu item with the exact category selected in a list of categories
						if (in_array($entry->category_id, $featured_categories)) {
							$matches['multiple_categories'] = $item->id;
							continue;
						}
						
						// Get all the categories
						$categories = RSDirectoryHelper::getCategories();
						
						// Get the category hierarchy.
						if ($hierarchy = RSDirectoryHelper::getCategoryHierarchy($entry->category_id, $categories)) {
							foreach ($hierarchy as $level => $category) {
								if (in_array($category->id, $featured_categories)) {
									// Found a menu item with the exact parent we're looking for
									if (count($featured_categories) == 1) {
										$matches['parent_category'][$level] = $item->id;
									} else {
										// Found a menu item with the exact parent selected in a list of categories
										$matches['multiple_parent_categories'][$level] = $item->id;
									}
								}
							}
						}
						
						// Found l(e)ast possible match - a menu item with entries from all categories
						if (empty($featured_categories) || in_array(0, $featured_categories)) {
							$matches['all_categories'] = $item->id;
							continue;
						}
					}
				}
			}
			
			krsort($matches['parent_category'], SORT_NUMERIC);				
			$matches['parent_category'] = reset($matches['parent_category']);
			
			krsort($matches['multiple_parent_categories'], SORT_NUMERIC);
			$matches['multiple_parent_categories'] = reset($matches['multiple_parent_categories']);
			
			$matches = array_filter($matches);
			if ($match = reset($matches)) {
				return $match;
			}
		}
		
		return 0;
	}
		
	/**
	 * Method to generate a category entries URL.
	 *
	 * @access public
	 *
	 * @static
	 *
	 * @param int $id
	 * @param string $title
	 * @param int $Itemid
	 * @param bool $absolute
	 * @param bool $xhtml
	 *
	 * @return string
	 */
	public static function getCategoryEntriesURL($id, $title, $Itemid = 0, $absolute = false, $xhtml = true)
	{
		if (RSDirectoryConfig::getInstance()->get('advanced_sef') && !$Itemid) {
			$Itemid = self::getCategoryEntriesBestMatch($id);
		}
		
		// load the category so that we can identify the alias
		$category = RSDirectoryHelper::getCategory($id);

		$route = self::BASE_ROUTE . "&view=entries&category=" . self::sef($id, $category->alias) . ($Itemid ? "&Itemid=$Itemid" : '');
		return self::getInstance()->getParsedURL($route, $absolute, $xhtml);
	}
	
	protected static function getCategoryEntriesBestMatch($id) {
		static $items;
		if ($items === null) {
			$items = array();
			
			if ($menus = JFactory::getApplication()->getMenu('site')) {
				$component  = JComponentHelper::getComponent('com_rsdirectory');
				$attributes = array('component_id');
				$values     = array($component->id);
				$items 		= $menus->getItems($attributes, $values);
			}
		}
		
		if ($items) {
			$matches = array(
				'category' 					 => false,
				'multiple_categories'		 => false,
				'parent_category'			 => array(),
				'multiple_parent_categories' => array(),
				'all_categories'			 => false
			);
			
			foreach ($items as $item) {
				if (isset($item->query) && is_array($item->query) && isset($item->query['view']) && $item->query['view'] == 'entries') {
					if (!empty($item->query['user'])) {
						continue;
					}
					
					$featured_categories = RSDirectoryHelper::arrayInt($item->getParams()->get('featured_categories'));
					
					// Found best match - a menu item with the exact category
					if (count($featured_categories) == 1 && in_array($id, $featured_categories)) {
						$matches['category'] = $item->id;
						continue;
					}
					
					// Found second best match - a menu item with the exact category selected in a list of categories
					if (in_array($id, $featured_categories)) {
						$matches['multiple_categories'] = $item->id;
						continue;
					}
					
					// Get all the categories
					$categories = RSDirectoryHelper::getCategories();
					
					// Get the category hierarchy.
					if ($hierarchy = RSDirectoryHelper::getCategoryHierarchy($id, $categories)) {
						foreach ($hierarchy as $level => $category) {
							if (in_array($category->id, $featured_categories)) {
								// Found a menu item with the exact parent we're looking for
								if (count($featured_categories) == 1) {
									$matches['parent_category'][$level] = $item->id;
								} else {
									// Found a menu item with the exact parent selected in a list of categories
									$matches['multiple_parent_categories'][$level] = $item->id;
								}
							}
						}
					}
					
					// Found l(e)ast possible match - a menu item with entries from all categories
					if (empty($featured_categories) || in_array(0, $featured_categories)) {
						$matches['all_categories'] = $item->id;
						continue;
					}
				}
			}
			
			krsort($matches['parent_category'], SORT_NUMERIC);				
			$matches['parent_category'] = reset($matches['parent_category']);
			
			krsort($matches['multiple_parent_categories'], SORT_NUMERIC);
			$matches['multiple_parent_categories'] = reset($matches['multiple_parent_categories']);
			
			$matches = array_filter($matches);
			if ($match = reset($matches)) {
				return $match;
			}
		}
		
		return 0;
	}
		
	/**
	 * Method to generate an user entries URL.
	 *
	 * @access public
	 *
	 * @static
	 *
	 * @param int $id
	 * @param string $name
	 * @param bool $absolute
	 * @param bool $xhtml
	 *
	 * @return string
	 */
	public static function getUserEntriesURL($id, $name, $absolute = false, $xhtml = true)
	{
		$route = self::BASE_ROUTE . "&view=entries&user=" . self::sef($id, $name);
		
		if (RSDirectoryConfig::getInstance()->get('advanced_sef') && $Itemid = self::getUserEntriesBestMatch($id)) {
			$route .= "&Itemid=$Itemid";
		}
		
		return self::getInstance()->getParsedURL($route, $absolute, $xhtml);
	}

	protected static function getUserEntriesBestMatch($id, $redo = false) {
		static $items;
		if ($items === null) {
			$items = array();

			if ($menus = JFactory::getApplication()->getMenu('site')) {
				$component  = JComponentHelper::getComponent('com_rsdirectory');
				$attributes = array('component_id');
				$values     = array($component->id);
				$items 		= $menus->getItems($attributes, $values);
			}
		}

		if ($items) {
			foreach ($items as $item) {
				if (isset($item->query) && is_array($item->query) && isset($item->query['view']) && $item->query['view'] == 'entries') {
					if ((!$redo && isset($item->query['user']) && $item->query['user'] == $id) || $redo) {
						$featured_categories = RSDirectoryHelper::arrayInt($item->getParams()->get('featured_categories'));

						if (empty($featured_categories) || in_array(0, $featured_categories)) {
							return $item->id;
						}
					}
				}
			}

			if (!$redo) {
				return self::getUserEntriesBestMatch($id, true);
			}
		}

		return 0;
	}
		
	/**
	 * Method to get a generic URL.
	 *
	 * @access public
	 *
	 * @static
	 *
	 * @param string $view
	 * @param string $layout
	 * @param mixed $params
	 * @param bool $absolute
	 * @param bool $xhtml
	 *
	 * @return string
	 */
	public static function getURL($view, $layout = 'default', $params = '', $absolute = false, $xhtml = true)
	{
		$layout = empty($layout) ? 'default' : $layout;
			
		$params_string = '';
			
		if ($params)
		{
			if ( is_array($params) )
			{
				foreach ($params as $key => $value)
				{
					$params_string .= "&$key=$value";
				}
			}
			else if ( is_string($params) )
			{
				$params_string = substr($params, 0, 1) != '&' ? "&$params" : $params;
			}
		}
			
		$route = self::BASE_ROUTE . "&view=$view&layout=$layout" . $params_string;
		return self::getInstance()->getParsedURL($route, $absolute, $xhtml);
	}
		
	/**
     * SEF.
     *
     * @access public
     *
     * @static
     *
     * @param int $id
     * @param string $name
     *
     * @return string
     */
    public static function sef($id, $name)
    {
		if (JFactory::getConfig()->get('unicodeslugs') == 1)
		{
			$title = JFilterOutput::stringURLUnicodeSlug($name);
		}
		else
		{
			$title = JFilterOutput::stringURLSafe($name, JComponentHelper::getParams('com_languages')->get('site'));
		}
		
		$parts = array(
			(int) $id,
			$title
		);
		
		if (!strlen($parts[1]) || empty($parts[1]) || is_numeric($parts[1])) {
			unset($parts[1]);
		}
		
		return implode(':', $parts);
    }
}