<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * Entry model.
 */
class RSDirectoryModelEntry extends JModelAdmin
{
    /**
     * Model context string.
     *
     * @var string
     */
    protected $_context = 'com_rsdirectory.entry';
        
    /**
     * An array holding the field names of the registration fields that contain errors.
     * 
     * @var array
     */
    protected $error_reg_fields = array();
    
    /**
     * An array holding the ids of the fields that contain errors.
     * 
     * @var array
     */
    protected $error_field_ids = array();
        
    /**
     * The CAPTCHA error.
     *
     * @var mixed
     */
    protected $captcha_error;

    /**
     * Holds the Validation status.
     *
     * @var bool
     */
    protected $validation_errors = false;
        
    /**
     * Set a field error.
     *
     * @access private
     *
     * @param int $field_id
     * @param string $error_message
     */
    private function setFieldError($field_id, $error_message)
    {
        $this->error_field_ids[] = $field_id;
        $this->validation_errors = true;

        JFactory::getApplication()->enqueueMessage($error_message, 'error');
    }
        
    /**
     * Set CAPTCHA error.
     *
     * @access private
     *
     * @param string $error_message
     */
    private function setCaptchaError($error_message)
    {
        $this->captcha_error = $error_message;
        $this->validation_errors = true;

        JFactory::getApplication()->enqueueMessage($error_message, 'error');
    }
        
    /**
     * Set registration field error.
     *
     * @access private
     *
     * @param string $field_name
     * @param string $error_message
     */
    private function setRegError($field_name, $error_message)
    {
        $this->error_reg_fields[] = $field_name;
        $this->validation_errors = true;

        JFactory::getApplication()->enqueueMessage($error_message, 'error');
    }

    public function getErrorsType($type = 'fields') {
        switch($type)
        {
            case 'registration':
                return $this->error_reg_fields;
            break;
            case 'captcha':
                return !is_null($this->captcha_error);
            break;

            default:
                return $this->error_field_ids;
            break;
        }
    }
        
    /**
     * Method to get a table object, load it if necessary.
     * 
     * @access public
     * 
     * @param string $type
     * @param string $prefix
     * @param array $config
     * 
     * @return object
     */
    public function getTable( $type = 'Entry', $prefix = 'RSDirectoryTable', $config = array() )
    {
        return JTable::getInstance($type, $prefix, $config);
    }
        
    /**
     * Method for getting the form from the model.
     *
     * @access public
     * 
     * @param array $data
     * @param bool $loadData
     * 
     * @return mixed
     */
    public function getForm( $data = array(), $loadData = true )
    {
        JForm::addFormPath(JPATH_ADMINISTRATOR . '/components/com_rsdirectory/models/forms');
		JForm::addFieldPath(JPATH_ADMINISTRATOR . '/components/com_rsdirectory/models/fields');
            
        // Get the form.
        $form = $this->loadForm( 'com_rsdirectory.entry', 'entry', array('control' => 'fields', 'load_data' => $loadData) );
            
        if ( empty($form) )
            return false;

        return $form;
    }
        
    /**
     * Method to get the data that should be injected in the form.
     *
     * @access protected
     * 
     * @return array
     */
    protected function loadFormData()
    {
        $app = JFactory::getApplication();
            
        $data = $app->getUserState( 'com_rsdirectory.edit.entry.data', array() );
            
        $category_id = empty($data['category_id']) ? $app->input->getInt('category_id') : $data['category_id'];
            
        // Check the session for previously entered form data.
        if ( empty($data) )
        {
            $data = (array)$this->getItem();
        }
			
		if ($category_id)
        {
            $data['category_id'] = $category_id;
        }
            
        return $data;
    }
        
    /**
     * Method to get a single record.
     *
     * @access public
     *
     * @param int $pk The id of the primary key.
     *
     * @return mixed Object on success, false on failure.
     */
    public function getItem($pk = null)
    {
        // Get the id of the primary key.
        $pk = $pk ? $pk : (int) $this->getState( $this->getName() . '.id' );
            
        static $cache = array();
            
        if ( !isset($cache[$pk]) )
        {
            $cache[$pk] = RSDirectoryHelper::getEntry($pk);
        }
            
        return $cache[$pk];
    }
        
    /**
     * Validate form data.
     *
     * @access public
     * 
     * @param object $form The form to validate against.
     * @param array $data The data to validate.
     * @param string $group The name of the field group to validate.
     * @param bool $ajax If the validation is triggered from an Ajax Call.
     *
     * @return mixed
     */
    public function validate($form, $data, $group = null, $ajax = false)
    {
        // Get the mainframe.
        $app = JFactory::getApplication();
            
        // Front-end?
        $is_site = $app->isClient('site');

        // Get the JUser object.
        $user = JFactory::getUser();

        // separate the files names that are send in post with ajax for validation
        $files_ajax = array();
        if ($ajax)
        {
            if (isset($data['files']))
            {
                $files_ajax = $data['files'];
                unset($data['files']);
            }
        }
            
        if (!$is_site)
        {
            $return = parent::validate($form, $data, $group);
                
            if (!$return)
                return false;
                
            foreach ($return as $key => $value)
            {
                $data[$key] = $value;
            }
        } else {
            if (isset($data['published_time']))
            {
                // Get the user timezone setting defaulting to the server timezone setting.
                $timeZone = JFactory::getUser()->getParam('timezone', JFactory::getConfig()->get('offset'));

                // Return a MySQL formatted datetime string in UTC.
                $data['published_time'] = JFactory::getDate($data['published_time'], $timeZone)->toSql();
            }
        }
            
        // Get the task.
        $task = $app->input->get('task');
            
        // On front-end enable only the save & saveAndBuyCredits tasks.
        if ( $is_site && !in_array( $task, array('save', 'saveAndBuyCredits', 'validateAjax') ) )
            return false;
            
        // Get DBO.
        $db = JFactory::getDbo();
            
        // Load entry.
        if ( !empty($data['id']) )
        {
            $entry = $this->getItem($data['id']);
                
            if (!$entry)
                return false;
                
            if ( empty($data['category_id']) )
            {
                $data['category_id'] = $entry->category_id;   
            }

            // If the entry exits but the user_id is not sent keep the old one
            if (!$is_site && empty($data['user_id'])) {
                $data['user_id'] = $entry->user_id;
            }
        } else if (!$is_site && !isset($data['user_id'])) {
            $data['user_id'] = $user->id;
        }
            
        // Can moderate?
        $can_moderate = !$is_site || RSDirectoryHelper::checkUserPermission('can_edit_all_entries');
            
        if ( empty($data['category_id']) )
        {
            $app->enqueueMessage(JText::_('COM_RSDIRECTORY_FIELD_ERROR_SELECT_CATEGORY'), 'error');
            return false;
        }
            
        // Check if the category exists.
        $query = $db->getQuery(true)
               ->select( $db->qn('id') )
               ->from( $db->qn('#__categories') )
               ->where( $db->qn('id') . ' = ' . $db->q($data['category_id']) . ' AND ' . $db->qn('extension') . ' = ' . $db->q('com_rsdirectory') );
              
        $db->setQuery($query);
            
        if ( !$db->loadResult() )
        {
            $app->enqueueMessage(JText::_('COM_RSDIRECTORY_FIELD_ERROR_INVALID_CATEGORY'), 'error');
            return false;
        }
            
        // Get the form associated to the category.
        $form = RSDirectoryHelper::getCategoryInheritedForm($data['category_id']);
            
        // Return false if no form was associated to the category.
        if (!$form)
            return false;
            
        // Get the published form fields assigned to the form.
        $form_fields = RSDirectoryHelper::getFormFields($form->id, 1, 1);
            
        // Return false if there are no form fields assigned to the form.
        if (!$form_fields)
            return false;
            
        // Get the RSDirectory! configuration.
        $config = RSDirectoryConfig::getInstance();
            
        // Get the uploaded files.
        $form_files = $app->input->files->get('fields');

        // emulate the files
        if ($ajax && !empty($files_ajax))
        {
            foreach($files_ajax as $field_name => $field_files)
            {
                foreach ($field_files as $index => $field_file)
                {
                    list($field_file_name, $field_file_size) = explode('|', $field_file);

                    $form_files[$field_name][$index] = array(
                        'name'     => $field_file_name,
                        'tmp_name' => $field_file_name,
                        'size'     => (int) $field_file_size
                    );
                }
            }
        }
            
        // Set the unallowed file extensions.
        $unallowed_files = array('php', 'php5', 'phtml', 'php4', 'php3', 'phps', 'js', 'vbs', 'asp', 'exe');
            
            
        // Validate the user id.
        if ( !$is_site && empty($data['user_id']) )
        {
            $app->enqueueMessage(JText::_('COM_RSDIRECTORY_FIELD_ERROR_SELECT_ENTRY_AUTHOR'), 'error');
        }
		
		  // Validate the consent if present
        if ($is_site) {
            if ($config->get('use_consent_checkbox_on_entry_form', 1)) {
                // Check the consent if added/edited in frontend
                if (!$app->input->getInt('i_consent_to_privacy_policy_and_submit_my_details', 0)) {
                    $this->setFieldError('i_consent_to_privacy_policy_and_submit_my_details', JText::_('COM_RSDIRECTORY_CONSENT_ENTRY_ERROR'));
                } else {
                    // Save the checked consent in the session.
                    $app->setUserState('com_rsdirectory.edit.entry.consent', 1);
                }
            }
        }
              
        // Validate the CAPTCHA field.
        if ( $is_site && RSDirectoryHelper::checkUserPermission('add_entry_captcha') && empty($entry->id) && !$ajax)
        {
            if ( $config->get('captcha_type') == 'built_in' )
            {
                require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/captcha/captcha.php';
                    
                // Initialize the CAPTCHA object.
                $captcha = new RsdirectoryCaptcha('add_entry');
                $captcha->case_sensitive = $config->get('captcha_case_sensitive');
                    
                // Validate the CAPTCHA value.
                if ( empty($data['captcha']) || !$captcha->check( trim($data['captcha']) ) )
                {
                    $this->setCaptchaError( JText::_('COM_RSDIRECTORY_CAPTCHA_ERROR') );
                }
            }
			else
			{
				try {
					$input	  = $app->input;
					$response = $input->get('g-recaptcha-response', '', 'raw');
					$ip		  = $input->server->get('REMOTE_ADDR');
					$secretKey= $config->get('recaptcha_new_secret_key');

					$http = JHttpFactory::getHttp();
					if ($request = $http->get('https://www.google.com/recaptcha/api/siteverify?secret='.urlencode($secretKey).'&response='.urlencode($response).'&remoteip='.urlencode($ip))) {
						$json = json_decode($request->body);
					}
				} catch (Exception $e) {
					$this->setCaptchaError($e->getMessage());
				}
				
				if (!$json->success) {
					$this->setCaptchaError( JText::_('COM_RSDIRECTORY_RECAPTCHA_ERROR') );
				}
			}
        }
        else if ($is_site && RSDirectoryHelper::checkUserPermission('add_entry_captcha') && empty($entry->id) && $ajax)
        {

            if ( $config->get('captcha_type') == 'built_in' )
            {
                if ( empty($data['captcha']))
                {
                    $this->setCaptchaError( JText::_('COM_RSDIRECTORY_CAPTCHA_ERROR') );
                }
            }
            else
            {
                $input	  = $app->input;
                $response = $input->get('g-recaptcha-response', '', 'raw');

                if (empty($response))
                {
                    $this->setCaptchaError( JText::_('COM_RSDIRECTORY_RECAPTCHA_ERROR') );
                }
            }
        }
            
        // Validate the registration fields.            
        if ($is_site && !$user->id)
        {
            $reg = empty($data['reg']) ? array() : $data['reg'];
                
            if ( empty($reg['name']) )
            {
                $this->setRegError( 'name', JText::_('COM_RSDIRECTORY_NAME_REQUIRED') );
            }
            else if ( strlen( trim($reg['name']) ) < 2 )
            {
                $this->setRegError( 'name', JText::_('COM_RSDIRECTORY_NAME_LENGTH_ERROR') );
            }
                
            if ( empty($reg['email']) )
            {
                $this->setRegError( 'email', JText::_('COM_RSDIRECTORY_EMAIL_REQUIRED') );
            }
            else if ( !RSDirectoryHelper::email($reg['email']) )
            {
                $this->setRegError( 'email', JText::_('COM_RSDIRECTORY_PROVIDE_VALID_EMAIL') );
            }
            else if ( RSDirectoryHelper::userExists( array('email' => $reg['email']) ) )
            {
                $this->setRegError( 'email', JText::_('COM_RSDIRECTORY_EMAIL_IN_USE') );
            }
        }
            

        // Validate each form field.
        foreach ($form_fields as $form_field)
        {
            if ($form->use_title_template && $form_field->field_type == 'title')
                continue;
                
            if ($form->use_big_subtitle_template && $form_field->field_type == 'big_subtitle')
                continue;
                
            if ($form->use_small_subtitle_template && $form_field->field_type == 'small_subtitle')
                continue;
                
            if ($form->use_description_template && $form_field->field_type == 'description')
                continue;
                
            // Get the field properties.
            $properties = empty($form_field->properties) ? new RSDirectoryFormFieldProperties : $form_field->properties;
                
            if ( ($form_field->field_type == 'publishing_period' || $form_field->field_type == 'promoted') && !unserialize( $properties->get('items') ) )
                continue;
                
            if ($form_field->field_type == 'map')
            {
                $field_value = isset($data[$form_field->form_field_name]['address']) && $data[$form_field->form_field_name]['address'] != '' ? $data[$form_field->form_field_name]['address'] : '';
            }
            else if ($form_field->field_type == 'promoted')
            {
                $field_value = isset($data[$form_field->form_field_name]) && $data[$form_field->form_field_name] != '-1' ? $data[$form_field->form_field_name] : '';
            }
            else
            {   
                $field_value = isset($data[$form_field->form_field_name]) && $data[$form_field->form_field_name] != '' ? $data[$form_field->form_field_name] : '';    
            }
                
            // Get the validation message.
            $validation_message = $properties->get('validation_message');
                
            // Unset any previously set items.
			unset($items);
                
            // Unset any previously set valid keys.
            unset($valid_keys);
                
            // Process DROPDOWN DATE PICKER field.
            if ($form_field->field_type == 'dropdown_date_picker')
            {
                $date = (object)$field_value;
                    
                $show_day_dropdown = $properties->get('show_day_dropdown');
                $show_month_dropdown = $properties->get('show_month_dropdown');
                $show_year_dropdown = $properties->get('show_year_dropdown');
                    
                // Field is required.
                if ($form_field->required)
                {
                    // Set an error if one of the values is empty.
                    if ( ( $show_day_dropdown && empty($date->day) ) || ( $show_month_dropdown && empty($date->month) ) || ( $show_year_dropdown && empty($date->year) ) )
                    {
                        $this->setFieldError($form_field->id, $validation_message);
                        continue;
                    }
                }
                    
                if ( $show_year_dropdown && !empty($date->year) )
                {
                    $start_year = $properties->get('start_year');
                    $end_year = $properties->get('until_current_year') || !$properties->get('end_year') ? JFactory::getDate()->format('Y') : $properties->get('end_year');
                        
                    if ($date->year < $start_year || $date->year > $end_year)
                    {
                        $this->setFieldError($form_field->id, $validation_message);
                        continue;
                    }
                }
                    
                if ( $show_month_dropdown && !empty($date->month) && ($date->month < 1 || $date->month > 12) )
                {
                    $this->setFieldError($form_field->id, $validation_message);
                    continue;
                }
                    
                if (
                    $show_year_dropdown && $show_month_dropdown && $show_day_dropdown &&
                    !empty($date->year) && !empty($date->month) && !empty($date->day) &&
                    !checkdate($date->month, $date->day, $date->year)
                )
                {
                    $this->setFieldError($form_field->id, $validation_message);
                    continue;
                }
                    
                $valid = true;
                    
                // If one field was selected, then all fields must be selected.
                if ($show_year_dropdown && $show_month_dropdown && $show_day_dropdown)
                {
                    if ( ( !empty($date->year) || !empty($date->month) || !empty($date->day) ) && ( empty($date->year) || empty($date->month) || empty($date->day) ) )
                    {
                        $valid = false;
                    }
                    else if ( !empty($date->year) && !empty($date->month) && !empty($date->day) )
                    {
                        $filled_in = true;
                    }
                }
                else if ($show_year_dropdown && $show_month_dropdown)
                {
                    if ( ( !empty($date->year) || !empty($date->month) ) && ( empty($date->year) || empty($date->month) ) )
                    {
                        $valid = false;
                    }
                    else if ( !empty($date->year) && !empty($date->month) )
                    {
                        $filled_in = true;
                    }
                }
                else if ($show_year_dropdown && $show_day_dropdown)
                {
                    if ( ( !empty($date->year) || !empty($date->day) ) && ( empty($date->year) || empty($date->day) ) )
                    {
                        $valid = false;
                    }
                    else if ( !empty($date->year) && !empty($date->day) )
                    {
                        $filled_in = true;
                    }
                }
                else if ($show_month_dropdown && $show_day_dropdown)
                {
                    if ( ( !empty($date->month) || !empty($date->day) ) && ( empty($date->month) || empty($date->day) ) )
                    {
                        $valid = false;
                    }
                    else if ( !empty($date->month) && !empty($date->day) )
                    {
                        $filled_in = true;
                    }
                }
                    
                if ( empty($filled_in) && !$valid )
                {
                    $this->setFieldError($form_field->id, $validation_message);
                }
                    
                continue;
            }
                
            // Process FILEUPLOAD and IMAGES fields.
            if ( in_array( $form_field->field_type, array('images', 'image_upload', 'fileupload') ) )
            {
                // Get the max filecount.
                $max_file_count = $properties->get('max_files_number', 0);
                    
                if ( !empty($entry) )
                {
                    $existing_files = RSDirectoryHelper::getFilesObjectList($form_field->id, $entry->id);
                        
                    // Set the target directory.
                    $entry_dir = JPATH_SITE . "/components/com_rsdirectory/files/entries/$entry->id";
                }
				
                    
                // Initialize the files count.
                $files_count = empty($existing_files) ? 0 : count($existing_files);
                    
                // Get the accepted file extensions.
                $accepted_files = $properties->get('accepted_files') ? explode( "\r\n", $properties->get('accepted_files') ) : array();
                
				if (empty($this->to_delete)) {
					$this->to_delete = array();
				}
				if (isset($data['deleted'][$form_field->id])) {
					foreach ($data['deleted'][$form_field->id] as $file_pk) {
						// Get the file.
						$file = RSDirectoryHelper::getFileObject($file_pk);
						if (!$file)
						{
							$this->setFieldError($form_field->id, JText::_('COM_RSDIRECTORY_FILE_DELETION_INVALID_ERROR'));
							continue 2;
						}
						
						// Can this user delete the file?
						$user = JFactory::getUser();
						if ($file->user_id != $user->id && !$can_moderate)
						{
							$this->setFieldError($form_field->id, JText::_('COM_RSDIRECTORY_FILE_DELETION_PERMISSION_ERROR') );
							continue 2;
						}
						
						$this->to_delete[] = $file;
						$files_count--;
					}
				}
				
                // Validate each file.
                if ( !empty($form_files[$form_field->form_field_name]) )
                {
                    foreach ($form_files[$form_field->form_field_name] as $key => $file)
                    {
                        if (!$file['name'])
                            continue;
                            
                        // Validate the FILE TYPE of the IMAGES field.
                        if ( !$ajax && in_array( $form_field->field_type, array('images', 'image_upload')) && !RSDirectoryHelper::isImage($file['tmp_name']) )
                        {
                            $this->setFieldError($form_field->id, $validation_message);
                            continue 2;
                        }
                            
                        // Increment the files count.
                        $files_count++;
                            
                        // The maximum file count has been reached.
                        if ($max_file_count && $files_count >  $max_file_count)
                        {
                            $this->setFieldError($form_field->id, $validation_message);
                            continue 2;
                        }
                            
                        // Get the extension.
                        $ext = strtolower( JFile::getExt($file['name']) );
                            
                        // Skip this file if it's unallowed.
                        if ( in_array($ext, $unallowed_files) )
                        {
                            $this->setFieldError($form_field->id, $validation_message);
                            continue 2;
                        }
                            
                        if ($accepted_files)
                        {
                            // Convert all the accepted files extensions to lowercase.
                            foreach ($accepted_files as &$value)
                            {
                                $value = strtolower($value);
                            }
                                
                            // Skip this file if it's unallowed.
                            if ( !in_array($ext, $accepted_files) )
                            {
                                $this->setFieldError($form_field->id, $validation_message);
                                continue 2;
                            }
                        }
                            
                        // Skip this file if it's too big.
                        if ( $properties->get('maximum_file_size') && $properties->get('maximum_file_size') * 1024 < $file['size'] )
                        {
                            $this->setFieldError($form_field->id, $validation_message);
                            continue 2;
                        }
                    }
                }
                    
                // Set an error if the field is required and nothing was uploaded.
                if ( $form_field->required && empty($files_count) )
                {
                    $this->setFieldError($form_field->id, $validation_message);
                    continue;
                }
                   
                continue;
            }

                
            // Set an error if the field is required and the value is empty.
            if ($form_field->required && RSDirectoryHelper::trim($field_value) === '')
            {
                $this->setFieldError($form_field->id, $validation_message);
                continue;
            }
			
            // Proceed with validating the field if the posted value is not empty.
            if ( (is_string($field_value) && strlen($field_value)) || (is_array($field_value) && !empty($field_value)) )
            {
                if ($form_field->field_type == 'price')
                {
                    $field_value = trim($field_value);

                    $decimal_point          = $config->get('decimal_point');
                    $thousands_separator 	= $config->get('thousands_separator');

                    // remove the thousands separator for the price, because is formatted at display - use only the dot(.) for the decimals
                    $field_value = str_replace(array($thousands_separator, $decimal_point), array('', '.'), $field_value);
					
                    $data[$form_field->form_field_name] = $field_value;
                        
                    if ( !is_numeric($field_value) || $field_value < 0 )
                    {
                        $this->setFieldError($form_field->id, $validation_message);
                        continue;
                    }
					
					// if the price field is required then the value must not be 0
					if ($form_field->required && $field_value == 0) {
                        $this->setFieldError($form_field->id, $validation_message);
                        continue;
                    }
                }
                    
                // Check if the field allows an array of values.
                if ( $form_field->field_type != 'checkboxgroup' && !$properties->get('multiple') && is_array($field_value) )
                {
                    $this->setFieldError($form_field->id, $validation_message);
                    continue;
                }
                    
                if ($form_field->field_type == 'youtube')
                {
                    $uri = new JUri($field_value);
                        
                    if ( !( $uri->getHost() == 'youtu.be' && trim( substr( $uri->getPath(), 1 ) ) ) && !$uri->getVar('v') )
                    {
                        $this->setFieldError($form_field->id, $validation_message);
                        continue;
                    }
                }

                if ($form_field->field_type == 'vimeo')
                {
                    $uri = new JUri($field_value);

                    if ( !( $uri->getHost() == 'vimeo.com' && trim( substr( $uri->getPath(), 1 ) ) ) )
                    {
                        $this->setFieldError($form_field->id, $validation_message);
                        continue;
                    }


                }
				
				if ($form_field->field_type == 'link') {
					
					if (!RSDirectoryHelper::validurl($field_value)) {
						$this->setFieldError($form_field->id, $validation_message);
                        continue;
					}
				}
                    
                // Get the items array if the field has one.
                switch ($form_field->field_type)
                {
                    case 'publishing_period':
                    case 'promoted':

                        @$items = $properties->get('items') ? unserialize( $properties->get('items') ) : array();
                        $items = $items ? $items : array();
                            
                        $valid_keys = array();
                            
                        for ( $i = 1; $i <= count($items); $i++ )
                        {
                            $valid_keys[] = $i;
                        }

                        // accept the custom value
                        if ($form_field->field_type == 'promoted' && (int)$properties->get('daily_cost') > 0)
                        {
                            $valid_keys[] = 'custom_high';
                            $valid_keys[] = 'custom_low';
                        }
                        
                        break;
                            
                    case 'dropdown':
                    case 'checkboxgroup':
                    case 'radiogroup':
                    case 'country':
                            
                        $items = $properties->get('items') ? RSDirectoryHelper::getOptions( $properties->get('items') ) : array();
                            
                        if ($items)
                        {
							$valid_keys = array();
								
                            foreach ($items as $item)
                            {
                                if (
                                    stripos($item->value, '[g]') === false && stripos($item->text, '[g]') === false &&
                                    stripos($item->value, '[d]') === false && stripos($item->text, '[d]') === false
                                )
                                {
                                    // remove the credit part
                                    if( preg_match('/\[c-\d+]/', $item->value, $credits) != false)
                                    {
                                        $item->value = str_replace($credits[0], '', $item->value);
                                    }

                                    $valid_keys[] = $item->value;
                                }


                            }
                        }
                            
                        break;
                }
                    
                $valid = true;
                    
                // If the field has an items array, then validate if the posted values are valid.
                if ( isset($valid_keys) )
                {
                    if (!$valid_keys)
                    {
                        $valid = false;
                    }
                        
                    if ( is_array($field_value) )
                    {
                        foreach ($field_value as $value)
                        {
                            if ( !in_array($value, $valid_keys) )
                            {
                                $valid = false;
                                break;
                            }
                        }
                    }
                    else
                    {
                        if ( !in_array($field_value, $valid_keys) )
                        {
                            $valid = false;
                        }
                    }
                }
                    
                // Validate the value if there is a validation rule set.
                if ( $valid && $properties->get('default_validation_rule') != 'none' )
                {
                    // Validated the value of the form field.
                    switch ( $properties->get('default_validation_rule') )
                    {
                        case 'alpha':
                            $valid = RSDirectoryHelper::alpha( $field_value, $properties->get('extra_accepted_chars') );
                            break;
                            
                        case 'numeric':
                            $valid = RSDirectoryHelper::numeric( $field_value, $properties->get('extra_accepted_chars') );
                            break;
                            
                        case 'alphanumeric':
                            $valid = RSDirectoryHelper::alphanumeric( $field_value, $properties->get('extra_accepted_chars') );
                            break;
                            
                        case 'email':
                            $valid = RSDirectoryHelper::email($field_value);
                            break;
                            
                        case 'emaildns':
                            $valid = RSDirectoryHelper::emaildns($field_value);
                            break;
                            
                        case 'uniquefield':
                            $valid = RSDirectoryHelper::uniquefield($field_value, $form_field->column_name, !empty($data['id']) ? $data['id'] : null);
                            break;
                            
                        case 'uniquefielduser':
                            $valid = RSDirectoryHelper::uniquefielduser($field_value, $form_field->column_name, !empty($data['id']) ? $data['id'] : null);
                            break;
                            
                        case 'uszipcode':
                            $valid = RSDirectoryHelper::uszipcode($field_value);
                            break;
                            
                        case 'phonenumber':
                            $valid = RSDirectoryHelper::phonenumber($field_value);
                            break;
                            
                        case 'creditcard':
                            $valid = RSDirectoryHelper::creditcard($field_value);
                            break;
                            
                        case 'custom_characters':
                            $valid = RSDirectoryHelper::custom( $field_value, $properties->get('custom_validation_rule') );
                            break;
                            
                        case 'custom_php':
                            @$valid = eval( $properties->get('custom_validation_rule') );
                            break;
                            
                        case 'ipaddress':
                            $valid = RSDirectoryHelper::ipaddress($field_value);
                            break;
                            
                        case 'validurl':
                            $valid = RSDirectoryHelper::validurl($field_value);
                            break;
                            
                        case 'regex':
                            $valid = RSDirectoryHelper::regex( $field_value, $properties->get('regex_syntax') );
                            break;
                            
                        default:
                            $valid = true;
                            break;
                    }
                }
                    
                // Validate characters limit.
                $characters_limit = $properties->get('characters_limit');
                    
                if ( $valid && $characters_limit && strlen( strip_tags($field_value) ) > $characters_limit )
                {
                    $valid = false;    
                }

                // Validate Min/Max Selections
                if ($valid && !empty($properties->get('use_min_max_selections', 0))) {

                    $verify = false;

                    switch($form_field->field_type)
                    {
                        case 'checkboxgroup':
                            $verify = true;
                        break;

                        case 'dropdown':
                        case 'country':
                            if (!empty($properties->get('multiple')))
                            {
                                $verify = true;
                            }
                        break;
                    }

                    if ($verify)
                    {
                        $min = (int) $properties->get('min_selections', 1);
                        $max = (int) $properties->get('max_selections', 2);

                        $selected_values = is_array($field_value) ? count($field_value) : 1;

                        if ($selected_values < $min || $selected_values > $max)
                        {
                            $valid = false;
                        }
                    }
                }
                    
                // Set an error if the value is invalid.
                if (!$valid)
                {
                    $this->setFieldError($form_field->id, $validation_message);
                    continue;
                }
            }
        }


        // if the request is made by the Ajax validation do not store the data in the user state
        if (!$ajax)
        {
            // Store the ids of the fields that contain errors.
            $app->setUserState('com_rsdirectory.edit.entry.error_field_ids', $this->error_field_ids);

            // Store the CAPTCHA error.
            $app->setUserState('com_rsdirectory.edit.entry.captcha_error', $this->captcha_error);

            // Store the names of the registration fields that contain errors.
            $app->setUserState('com_rsdirectory.edit.entry.error_reg_fields', $this->error_reg_fields);
        }
            
        if ( $this->validation_errors )
        {
            return false;
        }

        return $ajax ? true : $data;
    }
        
    /**
     * Save the field.
     *
     * @access public
     * 
     * @param array $data
     * 
     * @return bool
     */
    public function save($data)
    {
        // Get mainframe.
        $app = JFactory::getApplication();
            
        // Get DBO.
        $db = JFactory::getDbo();
            
        // Front-end?
        $is_site = $app->isClient('site');
            
        // Get the task.
        $task = $app->input->get('task');
            
        // Get the JUser object.
        $user = JFactory::getUser();
		
		$config = RSDirectoryConfig::getInstance();
		
		// Get the renew type option value
		$renew_types = (int) $config->get('renew_types', 0);
            
        // Load entry.
        if ( !empty($data['id']) )
        {
            $current_entry = $this->getItem($data['id']);
              
            if ( empty($data['category_id']) )
            {
                $data['category_id'] = $current_entry->category_id;
            }
                
            if ($task == 'save2copy')
            {
                // Copy the entry object.
                $entry_copy = clone $current_entry;
                    
                // Unset entry.
                unset($current_entry);
            }
        }
            
        $can_moderate = !$is_site || RSDirectoryHelper::checkUserPermission('can_edit_all_entries');

        $charge_credits = true;
        if ($can_moderate && isset($data['charge']) && !$data['charge']){
            $charge_credits = false;
        }
		
		$import = false;
		if (isset($data['import']) && $data['import']) {
			$import = true;
			unset($data['import']);
		}
            
        // Get the uploaded files.
        $form_files = $app->input->files->get('fields');
            
        // Initialize the entries credits objects array.
        $entry_credits_objects = array();
            
        // Initialize the emails array. E.g.: submit_entry, publish_entry..
        $emails = array();
            
        // Get the form associated to the category.
        $form = RSDirectoryHelper::getCategoryInheritedForm($data['category_id']);
            
        // Get all the form fields.
        $form_fields = RSDirectoryHelper::getFormFields($form->id);

        $expired = false;
        $promoted_purchased = false;
        $promoted_period = -1;
        $promoted_credits = 0;
        $promoted_comparison_date = false;
        $promoted_period_custom = '';

        if (!empty($current_entry)) {
            $expired = $current_entry->expiry_time != '0000-00-00 00:00:00' && JFactory::getDate($current_entry->expiry_time)->toUnix() < JFactory::getDate()->toUnix();

            $promoted_credit            = RSDirectoryCredits::getLastPaidCreditField('promoted', $current_entry->id, $form->id, $form_fields);
            if ($promoted_credit) {
                $check_dates = array(
                    'start' => JFactory::getDate($current_entry->created_time)->toUnix()
                );

                if ($current_entry->expiry_time != '0000-00-00 00:00:00') {

                    $check_dates['end'] = JFactory::getDate($current_entry->expiry_time)->toUnix();

                    if ($current_entry->modified_time != '0000-00-00 00:00:00') {
                        $check_dates['start'] = JFactory::getDate($current_entry->modified_time)->toUnix();
                    }
                }

                $purchased_date = JFactory::getDate($promoted_credit->created_time)->toUnix();
                if (($purchased_date >= $check_dates['start'] && !isset($check_dates['end']))
                    || ($purchased_date >= $check_dates['start'] && isset($check_dates['end']) && $purchased_date < $check_dates['end'])) {
                    $promoted_purchased = true;
                }

            }

            // Establish the comparison date for the promoted_period

            // if there are both promoted_expiry and normal entry expiry must determine which will be our comparison
            if ($current_entry->promoted_expiry != '0000-00-00 00:00:00' && $current_entry->expiry_time != '0000-00-00 00:00:00') {
                $p_exp = JFactory::getDate($current_entry->promoted_expiry)->toUnix();
                $e_exp = JFactory::getDate($current_entry->expiry_time)->toUnix();
                $promoted_comparison_date =  $e_exp < $p_exp ? $e_exp : $p_exp;
            } else if ($current_entry->promoted_expiry != '0000-00-00 00:00:00' && $current_entry->expiry_time == '0000-00-00 00:00:00') {
                $promoted_comparison_date =  JFactory::getDate($current_entry->promoted_expiry)->toUnix();
            }

        }

            
        // Go through the form fields to process the PUBLISHING PERIOD, PROMOTED entry option.
        // We will process the other fields further down.
        foreach ($form_fields as $form_field)
        {
            if ( empty($data[$form_field->form_field_name]) && $form_field->field_type != 'promoted' )
                continue;
                
            // Get the field properties.
            $properties = empty($form_field->properties) ? new RSDirectoryFormFieldProperties : $form_field->properties;
                
            if ($form_field->field_type == 'publishing_period' || $form_field->field_type == 'promoted')
            {
                @$items = $properties->get('items') ? unserialize( $properties->get('items') ) : array();
                $items = $items ? $items : array();

                if ($form_field->field_type == 'publishing_period')
                {
                    $period = isset($items[$data[$form_field->form_field_name] - 1]) ? $items[$data[$form_field->form_field_name] - 1]->period : 0;
                    $period_credits = isset($items[$data[$form_field->form_field_name] - 1]) ? $items[$data[$form_field->form_field_name] - 1]->credits : 0;
                }
                else
                {
                    if (isset($data[$form_field->form_field_name]) && ($data[$form_field->form_field_name] == 'custom_high' || $data[$form_field->form_field_name] == 'custom_low') && (int)$properties->get('daily_cost'))
                    {
                        // Get the published id
                        $db = JFactory::getDbo();
                        $query = $db->getQuery(true)
                            ->select( $db->qn('id'))
                            ->from( $db->qn('#__rsdirectory_fields', 'f') )
                            ->where( $db->qn('f.name') . ' = ' . $db->q('publishing-period') );

                        $db->setQuery($query, 0 , 1);

                        if ($published_field_id = $db->loadResult())
                        {
                            $published_field_data = RSDirectoryHelper::getField($published_field_id);
                            @$items_published = $published_field_data->properties->get('items') ? unserialize( $published_field_data->properties->get('items') ) : array();
                            $items_published = $items_published ? $items_published : array();

                            $current_index = (int) ($data['publishing_period'] - 1);
                            $current_period = isset($items_published[$current_index]) ? $items_published[$current_index]->period : 0;

                            // find the best period and build the credits
                            if (!empty($items))
                            {
                                $is_high = ($data[$form_field->form_field_name] == 'custom_high');
                                $diffs = array();
                                foreach ($items as $index => $item) {
                                    $diffs[$index] = $current_period - $item->period;
                                }

                                if ($is_high)
                                {
                                    $index_found = array_search(min($diffs), $diffs);
                                }
                                else
                                {
                                    $index_found = array_search(max($diffs), $diffs);
                                }

                                if (isset($items[$index_found])) {
                                    $promoted_period = (int) $current_period;
                                    if ($is_high)
                                    {
                                        $nr_days = $promoted_period - (int) $items[$index_found]->period;
                                        $promoted_credits = $items[$index_found]->credits + ($properties->get('daily_cost') * $nr_days);
                                        $promoted_period_custom = 'custom_high';
                                    }
                                    else
                                    {
                                        $promoted_credits = $properties->get('daily_cost') * $promoted_period;
                                        $promoted_period_custom = 'custom_low';
                                    }

                                }
                            }
                        }

                    }
                    else if (isset($data[$form_field->form_field_name]) && isset($items[$data[$form_field->form_field_name] - 1]))
                    {
                        $promoted_period = (int) $items[$data[$form_field->form_field_name] - 1]->period;
						// only users that can modify all entries can use unlimited and it should be free
                        $promoted_credits = $promoted_period > 0 ? $items[$data[$form_field->form_field_name] - 1]->credits : 0;
                    }

                }
            }
                
            if ( in_array( $form_field->field_type, array('big_subtitle', 'small_subtitle', 'description', 'price', 'promoted') ) )
            {
                $skip_promoted = false;
                if (!empty($current_entry) && $form_field->field_type == 'promoted' && !$can_moderate) {
                    // promoted unlimited
                    if ($promoted_period == 0 && $promoted_purchased) {
                        $skip_promoted = true;
                    }
                    // promoted with limit
                    if (!is_bool($promoted_period) && $promoted_comparison_date && $promoted_comparison_date >= JFactory::getDate()->toUnix()) {
                        // get the current values
                        if (empty($data[$form_field->form_field_name])) {
                            $promoted_period = (int)$current_entry->promoted_period;
                            $promoted_credits = (int)$current_entry->promoted_credits;

                            if ($current_entry->promoted_selected == 'custom_high' || $current_entry->promoted_selected == 'custom_low')
                            {
                                $promoted_period_custom = $current_entry->promoted_selected;
                            }
                        }

                        $skip_promoted = true;
                    }
                }

                // if the promoted is not selected when adding/editing skip as well if above conditions have passed
                if (!$skip_promoted && empty($data[$form_field->form_field_name])) {
                    $skip_promoted = true;
                }

                // if the promoted is unlimited only the users that cand modify all users can use it
                if ($promoted_period === 0 && !RSDirectoryHelper::checkUserPermission('can_edit_all_entries')) {
                    $skip_promoted = true;
                    $promoted_period = false;
                }


                if ($skip_promoted) {
                    continue;
                }

                $entry_credits_objects[] = (object)array(
                    'object_type' => 'form_field',
                    'object_id' => $form_field->id,
                    'credits' => $form_field->field_type == 'promoted' ? $promoted_credits : $properties->get('credits'),
                );

            }
        }

        // Get the publishing period.
        $period = empty($period) ? 0 : $period;

        // Get the publishing period credits.
        $period_credits = empty($period_credits) ? 0 : $period_credits;

        // Get the promoted period.
        $promoted_period = ($promoted_period === false || $promoted_period == -1) ? false : $promoted_period;

        // Get the promoted period credits.
        $promoted_credits = empty($promoted_credits) ? 0 : $promoted_credits;
            
        // Process user.
        if ($can_moderate)
        {
            if ($is_site)
            {
                if ( empty($current_entry) )
                {
                    $user_id = JFactory::getUser()->id;
                }
                else
                {
                    $user_id = $current_entry->user_id;
                }
            }
            else
            {
                // Get the selected user id.
                $user_id = $data['user_id'];
            }
        }
        else
        {
            if ( JFactory::getUser()->id )
            {
                // Get the logged in user id.
                $user_id = JFactory::getUser()->id;
            }
            // Proceed with the user registration.
            else
            {
                require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/registration.php';
                    
                $reg = new RSDirectoryRegistration;
                $reg->setData( array(
                    'name' => $data['reg']['name'],
                    'email' => $data['reg']['email'],
                ) );
                    
                if ( $reg->register() )
                {
                    $user_id = $reg->getUserId();
                        
                    // Remember the id of the newly registered user.
                    $app->setUserState('com_rsdirectory.registration.user.id', $user_id);
                }
                else
                {
                    $reg_errors = $reg->getErrors();
                        
                    if ($reg_errors)
                    {
                        foreach ($reg_errors as $reg_error)
                        {
                            $this->setError($reg_error);
                        }
                    }
                    else
                    {
                        $this->setError( JText::_('COM_RSDIRECTORY_ENTRY_SAVE_ERROR') );
                    }
                        
                    return false;
                }
            }
        }

        $overwrite_promoted_expiry = false;
        if ($promoted_period !== false && !empty($data['promoted_expiry'])) {
            $overwrite_promoted_expiry = JFactory::getDate($data['promoted_expiry'])->toSql();
        }

        // Initialize the entry data.
        $entry_data = (object)array(
            'category_id' => $data['category_id'],
            'form_id' => $form->id,
            'user_id' => $user_id,
            'title' => empty($data['title']) ? '' : trim($data['title']),
            'big_subtitle' => empty($data['big_subtitle']) ? '' : $data['big_subtitle'],
            'small_subtitle' => empty($data['small_subtitle']) ? '' : $data['small_subtitle'],
            'description' => empty($data['description']) ? '' : $data['description'],
            'price' => empty($data['price']) ? 0 : $data['price'],
            'period' => $period,
            'period_credits' => $period_credits,
            'renew' => empty($data['renew']) ? 0 : 1,
            'promoted' => ($promoted_period === false) ? 0 : 1,
            'promoted_selected' => ($promoted_period === false) ? -1 : (empty($promoted_period_custom) ? $promoted_period : $promoted_period_custom),
            'promoted_period' => ($promoted_period === false) ? 0 : $promoted_period,
            'promoted_credits' => ($promoted_period === false) ? 0 : $promoted_credits,
        );

        $modified_after_expiration = false;
		$isPaid = true;
		
		// remember the title for further logging
        $entry_title_log =  $entry_data->title;  
		
        // Inserting a new entry.
        if ( empty($current_entry) )
        {
            $operation = 'insert';
                
            $entry_data->ip = RSDirectoryHelper::getIp(true);
            $entry_data->created_time = JFactory::getDate()->toSql();
                
            $entry_credits_objects[] = (object)array(
                'object_type' => 'publishing_period',
                'object_id' => 0,
                'credits' => $period_credits,
            );

            // Avoid default text fields values not being set
            $entry_data->metadata_title = '';
            $entry_data->metadata_keywords = '';
            $entry_data->metadata_description = '';

            // Insert the entry.
            $db->insertObject('#__rsdirectory_entries', $entry_data, 'id');

            // Get the id of the inserted entry.
            $entry_id = $entry_data->id;

            // Log the action
            $log_params = array(
                'type'          => 'entry_add',
                'entry_id'      => $entry_id,
                'entry_name'    => $entry_data->title
            );
            RSDirectoryHelper::logAction($log_params);
                
            $emails[] = 'submit_entry';
        }
        // Saving an existing entry.
        else
        {
            $operation = 'save';

            $isPaid = $current_entry->paid ? true : false;
            $modified_after_expiration = $can_moderate && $expired && $current_entry->modified_time != '0000-00-00 00:00:00' && JFactory::getDate($current_entry->modified_time)->toUnix() > JFactory::getDate($current_entry->expiry_time)->toUnix();
            // Get the entry id.
            $entry_id = $current_entry->id;
                
            $entry_data->id = $current_entry->id;
            $entry_data->modified_time = JFactory::getDate()->toSql();
            $entry_data->modified_user_id = JFactory::getUser()->id;
			
            // Update the entry.
            $db->updateObject('#__rsdirectory_entries', $entry_data, 'id');
                
            // Check if the publishing period is unpaid.
            $query = $db->getQuery(true)
                   ->select( $db->qn('id') )
                   ->from( $db->qn('#__rsdirectory_entries_credits') )
                   ->where( $db->qn('entry_id') . ' = ' . $db->q($entry_id) )
                   ->where( $db->qn('object_type') . ' = ' . $db->q('publishing_period') )
                   ->where( $db->qn('paid') . ' = ' . $db->q(0) );
                   
            $db->setQuery($query);
            $unpaid_publishing_period = $db->loadResult();
            
            if ($period != $current_entry->period || ($expired && !$modified_after_expiration) || $unpaid_publishing_period || ($modified_after_expiration && $period != $current_entry->period))
            {
                $entry_credits_objects[] = (object)array(
                    'object_type' => 'publishing_period',
                    'object_id' => 0,
                    'credits' => (($expired && $renew_types == 2 && $period == $current_entry->period && !$unpaid_publishing_period) ? 0 : $period_credits),
                );
            }
                
            // If the entry author has changed...
            if ($current_entry->user_id != $user_id)
            {
                // Assign the entries credits to the new user.
                $query = $db->getQuery(true)
                       ->update( $db->qn('#__rsdirectory_entries_credits') )
                       ->set( $db->qn('user_id') . ' = ' . $db->q($user_id) )
                       ->where( $db->qn('entry_id') . ' = ' . $db->q($current_entry->id) );
                        
                $db->setQuery($query);
                $db->execute();
                    
                // Assign the entries files to the new user.
                $query = $db->getQuery(true)
                       ->update( $db->qn('#__rsdirectory_uploaded_files', 'f') )
                       ->innerJoin( $db->qn('#__rsdirectory_uploaded_files_fields_relations', 'r') . ' ON ' . $db->qn('f.id') . ' = ' . $db->qn('r.file_id') )
                       ->set( $db->qn('f.user_id') . ' = ' . $db->q($user_id) )
                       ->where( $db->qn('r.entry_id') . ' = ' . $db->q($current_entry->id) );
                       
                $db->setQuery($query);
                $db->execute();

                // Log the action
                $log_params = array(
                    'type'          => 'entry_update_user',
                    'entry_id'      => $entry_id,
                    'entry_name'    => $entry_data->title,
                    'new_user'      => $user_id,
                    'old_user'      => $current_entry->user_id,
                );
                RSDirectoryHelper::logAction($log_params);
            } else {
                // Log the action
                $log_params = array(
                    'type'          => 'entry_update',
                    'entry_id'      => $entry_id,
                    'entry_name'    => $entry_data->title
                );
                RSDirectoryHelper::logAction($log_params);
            }
        }
            
            
        // Copy the files from the old entry to the new entry, if the task is save2copy.
        if ( !empty($entry_copy) )
        {
            // Get the files uploaded to the entry.
            $old_files = RSDirectoryHelper::getFilesObjectList(0, $entry_copy->id);
                
            if ($old_files)
            {
                // Set the src dir.
                $src_dir = JPATH_ROOT  . "/components/com_rsdirectory/files/entries/$entry_copy->id";
                    
                // Create the destination folder and get its path.
                $dst_dir = RSDirectoryHelper::createEntryDir($entry_id);
                    
                // Proceed if the destination folder was successfully created.
                if ($dst_dir)
                {
                    // Initialize the files array (to be inserted into the #__rsdirectory_uploaded_files_fields_relations table).
                    $new_files = array();
                        
                    foreach ($old_files as $old_file)
                    {
                        // Get the extension.
                        $ext = strtolower( JFile::getExt($old_file->original_file_name) );
                            
                        // Get a hash.
                        $hash = RSDirectoryHelper::getHash();
                            
                        // Generate a file name.
                        $file_name = $hash . ($ext ? ".$ext" : '');
                            
                        if ( JFile::copy("$src_dir/$old_file->file_name", "$dst_dir/$file_name") )
                        {
                            // Insert the file.
                            $new_file = (object)array(
                                'user_id' => $user_id,
                                'hash' => $hash,
                                'file_name' => $file_name,
                                'original_file_name' => $old_file->original_file_name,
                            );
                                
                            $db->insertObject('#__rsdirectory_uploaded_files', $new_file, 'id');
                                
                            $new_files[] = array(
                                $db->q($new_file->id),
                                $db->q($entry_id),
                                $db->q($old_file->field_id),
                                $db->q($old_file->ordering),
                            );
                                
                            // Get the form field.
                            $form_field = RSDirectoryHelper::findFormField( array('id' => $old_file->field_id), $form_fields );
                                
                            $entry_credits_objects[] = (object)array(
                                'object_type' => 'uploaded_file',
                                'object_id' => $form_field->id,
                                'credits' => $form_field->properties->get('credits_per_file'),
                            );
                                
                            $entry_credits_objects[] = (object)array(
                                'object_type' => 'form_field',
                                'object_id' => $form_field->id,
                                'credits' => $form_field->properties->get('credits'),
                            );
                        }   
                    }
                        
                    if ( !empty($new_files) )
                    {    
                        $columns = array(
                            'file_id',
                            'entry_id',
                            'field_id',
                            'ordering',
                        );
                            
                        $query = $db->getQuery(true)
                               ->insert( $db->qn('#__rsdirectory_uploaded_files_fields_relations') )
                               ->columns( $db->qn($columns) );
                                
                        foreach ($new_files as $file)
                        {
                            $query->values( implode(',', $file) );
                        }
                            
                        $db->setQuery($query);
                        $db->execute();
                    }
                }
            }
        }
            
        // Remember the item id.
        $this->setState( $this->getName() . '.id', $entry_id );
            
            
        // Initialize the entry custom fields data object.
        $entry_custom_data = (object)array(
            'entry_id' => $entry_id,
        );
		
		// need to store the $form_field->id so that we can process the uploaded and existing files after the old ones are deleted (if they are deleted)
		$entry_files_data = array();
            
        $old_entry_files_data = array();
        // Process the IMAGES field and the non-core fields.
        // We are processing the IMAGES field here, because we need the entry id.

        // store the curent loaded files per field
        $files_loaded = array();
        foreach ($form_fields as $form_field)
        {
            if ( $form_field->create_column || in_array( $form_field->field_type, array('images', 'image_upload', 'fileupload') ) )
            {
                // Get the field properties.
                $properties = empty($form_field->properties) ? null : $form_field->properties;
                    
                if ( in_array( $form_field->field_type, array('images', 'image_upload', 'fileupload') ) )
                {
					
                    $entry_dir = RSDirectoryHelper::createEntryDir($entry_id);
                        
                    if ( $entry_dir && is_writable($entry_dir) )
                    {
                        // Initialize the files array (to be inserted into the #__rsdirectory_uploaded_files_fields_relations table).
                        $files = array();
                            
                        // Get the ordering.
                        $query = $db->getQuery(true)
                               ->select( 'MAX(' . $db->qn('ordering') . ')' )
                               ->from( $db->qn('#__rsdirectory_uploaded_files_fields_relations') )
                               ->where( $db->qn('field_id') . ' = ' . $db->q($form_field->id) );
                                
                        $db->setQuery($query);
                        $ordering = (int)$db->loadResult();
						
						// need to get all previous files to check for new ones when saving and the entry is not expired
						if (!empty($current_entry)) {
							$old_entry_files_data[$form_field->id] = RSDirectoryHelper::getFilesObjectList($form_field->id, $current_entry->id);
						}
						
                        if ( !empty($form_files[$form_field->form_field_name]) )
                        {
                            foreach ($form_files[$form_field->form_field_name] as $file)
                            {
                                if (!$file['name'])
                                    continue;
                                    
                                // Increment the ordering value.
                                $ordering++;
                                    
                                // Get the extension.
                                $ext = strtolower( JFile::getExt($file['name']) );
                                    
                                // Get a hash.
                                $hash = RSDirectoryHelper::getHash();
                                    
                                // Generate a file name.
                                $file_name = $hash . ($ext ? ".$ext" : '');
                                    
                                // Move the uploaded file from tmp to its new directory.
                                JFile::upload($file['tmp_name'], "$entry_dir/$file_name");
                                    
                                 
								if (empty($current_entry)) {
									$entry_credits_objects[] = (object)array(
										'object_type' => 'uploaded_file',
										'object_id' => $form_field->id,
										'credits' => $properties->get('credits_per_file'),
									);
								}
                                    
                                // Insert the file.
                                $file = (object)array(
                                    'user_id' => $user_id,
                                    'hash' => $hash,
                                    'file_name' => $file_name,
                                    'original_file_name' => $file['name'],
                                );
                                    
                                $db->insertObject('#__rsdirectory_uploaded_files', $file, 'id');
                                    
                                $files[] = array(
                                    $db->q($file->id),
                                    $db->q($entry_id),
                                    $db->q($form_field->id),
                                    $db->q($ordering),
                                );
                            }
                        }
                            
                        if ( !empty($files) )
                        {
							if (empty($current_entry)) {
								$entry_credits_objects[] = (object)array(
									'object_type' => 'form_field',
									'object_id' => $form_field->id,
									'credits' => $properties->get('credits'),
								);
							}
                            else
                            {
                                if (!isset( $files_loaded[$current_entry->id]))
                                {
                                    $files_loaded[$current_entry->id] = array();
                                }
                                if (!isset( $files_loaded[$current_entry->id][$form_field->id]))
                                {
                                    $query = $db->getQuery(true)
                                        ->select('COUNT(*)')
                                        ->from($db->qn('#__rsdirectory_uploaded_files_fields_relations'))
                                        ->where($db->qn('field_id') . ' = ' . $db->q($form_field->id))
                                        ->where($db->qn('entry_id') . ' = ' . $db->q($current_entry->id));

                                    $db->setQuery($query);

                                    $files_loaded[$current_entry->id][$form_field->id] = (int)$db->loadResult();
                                }
                            }
                                
                            $columns = array(
                                'file_id',
                                'entry_id',
                                'field_id',
                                'ordering',
                            );
                                
                            $query = $db->getQuery(true)
                                   ->insert( $db->qn('#__rsdirectory_uploaded_files_fields_relations') )
                                   ->columns( $db->qn($columns) );
                                    
                            foreach ($files as $file)
                            {
                                $query->values( implode(',', $file) );
                            }
                                
                            $db->setQuery($query);
                            $db->execute();
                        }
						
						// check for the files
						if (!empty($current_entry)) {
							$entry_files_data[] = (object) array(
												'field_id'=>$form_field->id,
												'credits_per_file' => $form_field->properties->get('credits_per_file'),
												'credits'=>$form_field->properties->get('credits')
												);
						}
                    }
                }
                else
                {
                    $filled_in = false;

                    if ($form_field->field_type == 'dropdown' || $form_field->field_type == 'country')
                    {
                        if ( !isset($data[$form_field->form_field_name]) )
                        {
                            $value = '';
                        }
                        else
                        {
                            if ($properties->get('multiple') )
                            {
                                $value = implode("\r\n", $data[$form_field->form_field_name]);
                            }
                            else
                            {
                                $value = $data[$form_field->form_field_name];
                            }

                            if (strlen($value)) {
                                // get the items
                                $items = RSDirectoryHelper::getOptions( $form_field->properties->get('items') );

                                $grouped_options = RSDirectoryHelper::getGroupedListOptions($items);

                                // The default credits set up for this field
                                $default_credits = $properties->get('credits');

                                // hold the custom credits found
                                $custom_credits_found = array();

                                // hold the normal credits
                                $normal_credits_found = 0;
                                if (isset($grouped_options['credits'])) {
                                    if ($properties->get('multiple') ) {
                                        foreach ($data[$form_field->form_field_name] as $selected_value) {
                                            if (isset($grouped_options['credits'][$selected_value])) {
                                                $custom_credits_found[] = (int)$grouped_options['credits'][$selected_value];
                                            } else {
                                                if ($default_credits && !$normal_credits_found) {
                                                    $normal_credits_found = $default_credits;
                                                }
                                            }
                                        }
                                    } else {
                                        if (isset($grouped_options['credits'][$value])) {
                                            $custom_credits_found[] = (int)$grouped_options['credits'][$value];
                                        } else {
                                            $normal_credits_found = $default_credits;
                                        }
                                    }
                                } else {
                                    $normal_credits_found = $default_credits;
                                }

                                $all_credits = $normal_credits_found;

                                if (!empty($custom_credits_found)) {
                                    $custom_credits_found = array_sum($custom_credits_found);

                                    $all_credits += $custom_credits_found;
                                }


                                if (empty($current_entry)) {
                                    $entry_credits_objects[] = (object)array(
                                        'object_type' => 'form_field',
                                        'object_id' => $form_field->id,
                                        'credits' => $all_credits,
                                    );
                                } else {
                                    // get the paid amount
                                    $entry_credits = RSDirectoryCredits::getEntryCreditsObjectList($current_entry->id);
                                    $paid = RSDirectoryHelper::findElements(array('object_type' => 'form_field', 'object_id' => $form_field->id, 'paid' => 1), $entry_credits);

                                    // take the differences
                                    $diff_total = 0;
                                    $paid_diffs = RSDirectoryHelper::findElements(array('object_type' => 'form_field', 'object_id' => $form_field->id, 'paid' => 1, 'is_diff' => 1), $entry_credits, false);
                                    if ($paid_diffs) {
                                        foreach ($paid_diffs as $pd) {
                                            $diff_total += (int) $pd->credits;
                                        }
                                    }

                                    // the difference between the credits base on the selection and the already payed ones
                                    if (!empty($paid->credits)) {
                                        $diff_credits = $all_credits - ((int)$paid->credits + $diff_total);
                                    } else {
                                        $diff_credits = $all_credits -  $diff_total;
                                    }

                                    // if the entry is expired, then we must charge all the credits, except if a previous modification has been made after expiration
                                    if ($expired && !$modified_after_expiration) {
                                        $diff_credits = $all_credits;
                                    }

                                    if ($diff_credits != 0) {
                                        $entry_credits_objects[] = (object)array(
                                            'object_type' => 'form_field',
                                            'object_id' => $form_field->id,
                                            'credits' => $diff_credits,
                                            'is_diff' => ($all_credits != $diff_credits ? 1 : 0)
                                        );
                                    }
                                }
                            }
                        }
                    }
                    else if ($form_field->field_type == 'checkboxgroup')
                    {
                        if ( !isset($data[$form_field->form_field_name]) )
                        {
                            $value = '';
                        }
                        else
                        {
                            $value = implode("\r\n", $data[$form_field->form_field_name]);

                            if (strlen($value)) {
                                // Initiate the credits array
                                $credits_all = array();

                                // get the items
                                $items = RSDirectoryHelper::getOptions( $form_field->properties->get('items') );

                                // hold the cleared items, without flags
                                $default_items = array();

                                foreach ($items as $item) {
                                    $item_value = $item->value;
                                    $text = $item->text;

                                    if (stripos($item_value, '[d]') !== false || stripos($text, '[d]') !== false) {
                                        $item_value = str_replace('[d]', '', $item_value);
                                        $text = str_replace('[d]', '', $text);
                                    }

                                    // check for credits in the value
                                    if (preg_match('/\[c-\d+]/', $item_value, $credits) != false) {
                                        $item_value = str_replace($credits[0], '', $item_value);

                                        // get the credits value
                                        $credit_value = str_replace(array('[', ']', 'c-'), '', $credits[0]);
                                        $credits_all[$item_value] = (int)$credit_value;
                                    }

                                    // check for credits in the text
                                    if (preg_match('/\[c-\d+]/', $text, $credits)) {
                                        $text = str_replace($credits[0], '', $text);

                                        // get the credits value
                                        $credit_value = str_replace(array('[', ']', 'c-'), '', $credits[0]);
                                        // we need to save as reference the actual value
                                        if (!isset($credits_all[$item_value])) {
                                            $credits_all[$item_value] = (int)$credit_value;
                                        }
                                    }

                                    $default_items[$item_value] = $text;
                                }

                                // The default credits set up for this field
                                $default_credits = $properties->get('credits');

                                // hold the custom credits found
                                $custom_credits_found = array();

                                // hold the normal credits
                                $normal_credits_found = 0;
                                if (!empty($credits_all)) {
                                    foreach ($data[$form_field->form_field_name] as $selected_value) {
                                        if (isset($credits_all[$selected_value])) {
                                            $custom_credits_found[] = (int)$credits_all[$selected_value];
                                        } else {
                                            if ($default_credits && !$normal_credits_found) {
                                                $normal_credits_found = $default_credits;
                                            }
                                        }
                                    }
                                } else {
                                    $normal_credits_found = $default_credits;
                                }

                                $all_credits = $normal_credits_found;

                                if (!empty($custom_credits_found)) {
                                    $custom_credits_found = array_sum($custom_credits_found);

                                    $all_credits += $custom_credits_found;
                                }


                                if (empty($current_entry)) {
                                    $entry_credits_objects[] = (object)array(
                                        'object_type' => 'form_field',
                                        'object_id' => $form_field->id,
                                        'credits' => $all_credits,
                                    );
                                } else {
                                    // get the paid amount
                                    $entry_credits = RSDirectoryCredits::getEntryCreditsObjectList($current_entry->id);
                                    $paid = RSDirectoryHelper::findElements(array('object_type' => 'form_field', 'object_id' => $form_field->id, 'paid' => 1), $entry_credits);

                                    // take the differences
                                    $diff_total = 0;
                                    $paid_diffs = RSDirectoryHelper::findElements(array('object_type' => 'form_field', 'object_id' => $form_field->id, 'paid' => 1, 'is_diff' => 1), $entry_credits, false);
                                    if ($paid_diffs) {
                                        foreach ($paid_diffs as $pd) {
                                            $diff_total += (int) $pd->credits;
                                        }
                                    }

                                    // the difference between the credits base on the selection and the already payed ones
                                    $diff_credits = $all_credits - ((int)$paid->credits + $diff_total);

                                    // if the entry is expired, then we must charge all the credits, except if a previous modification has been made after expiration
                                    if ($expired && !$modified_after_expiration) {
                                        $diff_credits = $all_credits;
                                    }

                                    if ($diff_credits != 0) {
                                        $entry_credits_objects[] = (object)array(
                                            'object_type' => 'form_field',
                                            'object_id' => $form_field->id,
                                            'credits' => $diff_credits,
                                            'is_diff' => ($all_credits != $diff_credits ? 1 : 0)
                                        );
                                    }
                                }
                            }
                        }
                    }
                    else if ($form_field->field_type == 'radiogroup')
                    {
                        if ( !isset($data[$form_field->form_field_name]) )
                        {
                            $value = '';
                        }
                        else
                        {
                            $value = $data[$form_field->form_field_name];

                            if (strlen($value)) {
                                // Initiate the credits array
                                $credits_all = array();

                                // get the items
                                $items = RSDirectoryHelper::getOptions( $form_field->properties->get('items') );

                                // hold the cleared items, without flags
                                $default_items = array();

                                foreach ($items as $item) {
                                    $item_value = $item->value;
                                    $text = $item->text;

                                    if (stripos($item_value, '[d]') !== false || stripos($text, '[d]') !== false) {
                                        $item_value = str_replace('[d]', '', $item_value);
                                        $text = str_replace('[d]', '', $text);
                                    }

                                    // check for credits in the value
                                    if (preg_match('/\[c-\d+]/', $item_value, $credits) != false) {
                                        $item_value = str_replace($credits[0], '', $item_value);

                                        // get the credits value
                                        $credit_value = str_replace(array('[', ']', 'c-'), '', $credits[0]);
                                        $credits_all[$item_value] = (int)$credit_value;
                                    }

                                    // check for credits in the text
                                    if (preg_match('/\[c-\d+]/', $text, $credits)) {
                                        $text = str_replace($credits[0], '', $text);

                                        // get the credits value
                                        $credit_value = str_replace(array('[', ']', 'c-'), '', $credits[0]);
                                        // we need to save as reference the actual value
                                        if (!isset($credits_all[$item_value])) {
                                            $credits_all[$item_value] = (int)$credit_value;
                                        }
                                    }

                                    $default_items[$item_value] = $text;
                                }

                                // The default credits set up for this field
                                $default_credits = $properties->get('credits');

                                // hold the custom credits found
                                $custom_credits_found = array();

                                // hold the normal credits
                                $normal_credits_found = 0;

                                if (!empty($credits_all)) {
                                    if (isset($credits_all[$value])) {
                                        $custom_credits_found[] = (int)$credits_all[$value];
                                    } else {
                                        $normal_credits_found = $default_credits;
                                    }
                                } else {
                                    $normal_credits_found = $default_credits;
                                }

                                $all_credits = $normal_credits_found;

                                if (!empty($custom_credits_found)) {
                                    $custom_credits_found = array_sum($custom_credits_found);

                                    $all_credits += $custom_credits_found;
                                }


                                if (empty($current_entry)) {
                                    $entry_credits_objects[] = (object)array(
                                        'object_type' => 'form_field',
                                        'object_id' => $form_field->id,
                                        'credits' => $all_credits,
                                    );
                                } else {
                                    // get the paid amount
                                    $entry_credits = RSDirectoryCredits::getEntryCreditsObjectList($current_entry->id);

                                    // first paid credits
                                    $paid = RSDirectoryHelper::findElements(array('object_type' => 'form_field', 'object_id' => $form_field->id, 'paid' => 1), $entry_credits);

                                    // take the differences
                                    $diff_total = 0;
                                    $paid_diffs = RSDirectoryHelper::findElements(array('object_type' => 'form_field', 'object_id' => $form_field->id, 'paid' => 1, 'is_diff' => 1), $entry_credits, false);
                                    if ($paid_diffs) {
                                        foreach ($paid_diffs as $pd) {
                                            $diff_total += (int) $pd->credits;
                                        }
                                    }

                                    // the difference between the credits base on the selection and the already payed ones
                                    if (!empty($paid->credits)) {
                                        $diff_credits = $all_credits - ((int)$paid->credits + $diff_total);
                                    } else {
                                        $diff_credits = $all_credits - $diff_total;
                                    }


                                    // if the entry is expired, then we must charge all the credits, except if a previous modification has been made after expiration
                                    if ($expired && !$modified_after_expiration) {
                                        $diff_credits = $all_credits;
                                    }

                                    if ($diff_credits != 0) {
                                        $entry_credits_objects[] = (object)array(
                                            'object_type' => 'form_field',
                                            'object_id' => $form_field->id,
                                            'credits' => $diff_credits,
                                            'is_diff' => ($all_credits != $diff_credits ? 1 : 0)
                                        );
                                    }
                                }
                            }
                        }
                    }
                    else if ($form_field->field_type == 'calendar')
                    {
                        if ( !empty($data[$form_field->form_field_name]) )
                        {
                            $date = explode('/', $data[$form_field->form_field_name]);    
                        }  else  {
                            $date = array();
                        }
                            
                        list($month, $day, $year) = isset($date[2]) ? $date : array('00', '00', '0000');
                            
                        if ($month != '00' && $day != '00' && $year != '0000')
                        {
							
							if (!empty($current_entry) && !$expired && (strlen($current_entry->{'f_'.$form_field->id}) && $isPaid) && $current_entry->{'f_'.$form_field->id} != '0000-00-00 00:00:00') {
								$filled_in = false;
							} else {
                                if (!empty($current_entry) && $modified_after_expiration && $isPaid && $current_entry->{'f_'.$form_field->id} != '0000-00-00 00:00:00') {
                                    $filled_in = false;
                                } else {
                                    $filled_in = true;
                                }
							}
                        }

                        $value = "$year-$month-$day 00:00:00";
                    }
                    else if ($form_field->field_type == 'dropdown_date_picker')
                    {

                        $year = empty($data[$form_field->form_field_name]['year']) ? '0' : $data[$form_field->form_field_name]['year'];
                        $month = empty($data[$form_field->form_field_name]['month']) ? '0' : $data[$form_field->form_field_name]['month'];
                        $day = empty($data[$form_field->form_field_name]['day']) ? '0' : $data[$form_field->form_field_name]['day'];
                            
                        $show_day_dropdown = $properties->get('show_day_dropdown');
                        $show_month_dropdown = $properties->get('show_month_dropdown');
                        $show_year_dropdown = $properties->get('show_year_dropdown');
                            
                        // The form field was filled in.
                        if (
                            ($show_year_dropdown && $show_month_dropdown && $show_day_dropdown && $year && $month && $day) ||
                            ($show_year_dropdown && $show_month_dropdown && $year && $month) ||
                            ($show_year_dropdown && $show_day_dropdown && $year && $day) ||
                            ($show_month_dropdown && $show_day_dropdown && $month && $day)
                        )
                        {
							if (!empty($current_entry) && !$expired && (strlen($current_entry->{'f_'.$form_field->id}) && $isPaid) && $current_entry->{'f_'.$form_field->id} != '0000-00-00 00:00:00') {
								$filled_in = false;
							} else {
                                if (!empty($current_entry) && $modified_after_expiration && $isPaid && $current_entry->{'f_'.$form_field->id} != '0000-00-00 00:00:00') {
                                    $filled_in = false;
                                } else {
                                    $filled_in = true;
                                }
							}
							
                        }
						
						// if the year/month/day are not filled in then we need to make them in the correct format
						if (!$year) {
							$year = str_pad($year, 4, '0');
						}
						
						if (!$month) {
							$month = $year == '0000' ? str_pad($month, 2, '0') : '01'; // only if the year is "0000" can the month be "00"
						}
						
						if (!$day) {
							$day = $year == '0000' ? str_pad($day, 2, '0') : '01'; // only if the year is "0000" can the day be "00"
						}
                            
                        $value = "$year-$month-$day 00:00:00";
                    }
                    else if ($form_field->field_type == 'map')
                    {
                       $has_entered_address = false;
                        if ( isset($data[$form_field->form_field_name]['address']) )
                        {
                            $address_column_name = "{$form_field->column_name}_address";
                                
                            $entry_custom_data->$address_column_name = $data[$form_field->form_field_name]['address'];
							if (strlen($data[$form_field->form_field_name]['address'])) {
								$has_entered_address = true;
							}
                        }
                            
                        if ( isset($data[$form_field->form_field_name]['lat'], $data[$form_field->form_field_name]['lng']) )
                        {
                            $lat_column_name = "{$form_field->column_name}_lat";
                            $lng_column_name = "{$form_field->column_name}_lng";
                                
                            $entry_custom_data->$lat_column_name = (float)$data[$form_field->form_field_name]['lat'];
                            $entry_custom_data->$lng_column_name = (float)$data[$form_field->form_field_name]['lng'];
                             
							if($has_entered_address || ((float)$data[$form_field->form_field_name]['lat'] != 0 && (float)$data[$form_field->form_field_name]['lng'] != 0)) {   
								if (!$expired && !empty($current_entry) && ((strlen($current_entry->{'f_'.$form_field->id.'_address'}) || (strlen($current_entry->{'f_'.$form_field->id.'_lat'}) && strlen($current_entry->{'f_'.$form_field->id.'_lng'}))) && $isPaid)) {
									$filled_in = false;
								} else {
                                    if (!empty($current_entry) && $modified_after_expiration && $isPaid && (strlen($current_entry->{'f_'.$form_field->id.'_address'}) || (strlen($current_entry->{'f_'.$form_field->id.'_lat'}) && strlen($current_entry->{'f_'.$form_field->id.'_lng'})))) {
                                        $filled_in = false;
                                    } else {
                                        $filled_in = true;
                                    }
								}
							}
                        }
						
						if ($filled_in)
						{
							$entry_credits_objects[] = (object)array(
								'object_type' => 'form_field',
								'object_id' => $form_field->id,
								'credits' => $properties->get('credits'),
							);    
						}
                            
                        continue;
                    }
                    else
                    {
                        if ( !isset($data[$form_field->form_field_name]) )
                        {
                            $value = '';
                        }
                        else
                        {
                            if ($properties->get('multiple') )
                            {
                                $value = implode("\r\n", $data[$form_field->form_field_name]);
                            }
                            else
                            {
                                $value = $data[$form_field->form_field_name];
                            }
                            if (strlen($value)) {
								// need to check if the field has been filled before so that we will not charge it
								if (!empty($current_entry) && !$expired && (strlen($current_entry->{'f_'.$form_field->id}) && $isPaid)) {
									$filled_in = false;
								} else {
                                    if (!empty($current_entry) && $modified_after_expiration && $isPaid && strlen($current_entry->{'f_'.$form_field->id})) {
                                        $filled_in = false;
                                    } else {
                                        $filled_in = true;
                                    }
								}
							}
                        }
                    }   
                    if ($filled_in)
                    {
                        $entry_credits_objects[] = (object)array(
                            'object_type' => 'form_field',
                            'object_id' => $form_field->id,
                            'credits' => $properties->get('credits'),
                        );    
                    }
                        
                    $entry_custom_data->{$form_field->column_name} = $value;
                }
            }
        }
		
		// Delete leftover files
		if (!empty($this->to_delete)) {
			RSDirectoryHelper::deleteFiles($this->to_delete);
		}
		
		// check for the files
		if (!empty($current_entry) && !empty($entry_files_data)) {
		
			foreach ($entry_files_data as $field) {
				$all_files = RSDirectoryHelper::getFilesObjectList($field->field_id, $current_entry->id);
				
				if ($all_files)
				{
                    $current_files = count($all_files);
                    // determine if there are more files set than previous
                    $more_files = isset($files_loaded[$current_entry->id][$field->field_id]) ? $current_files - $files_loaded[$current_entry->id][$field->field_id] : 0;
                    $more_files = $more_files > 0 ? $more_files : 0;

					foreach ($all_files as $i => $file)
					{
                        if (
                            (
                                ($expired && !$modified_after_expiration) ||
                                !$isPaid ||
                                (!$expired && empty($old_entry_files_data[$field->field_id]))
                            ) ||
                            ($i >= ($current_files - $more_files))
                        ) {
                            $entry_credits_objects[] = (object)array(
                                'object_type' => 'uploaded_file',
                                'object_id' => $field->field_id,
                                'credits' => $field->credits_per_file,
                            );
                        }
							
						if (($expired && !$modified_after_expiration) || !$isPaid || (!$expired && empty($old_entry_files_data[$field->field_id]))) {
							$entry_credits_objects[] = (object)array(
								'object_type' => 'form_field',
								'object_id' => $field->field_id,
								'credits' => $field->credits,
							);
						}
					}
				}
			}
		}
            
        // Handle custom fields values.
        $entries_custom = JTable::getInstance('Entriescustom', 'RSDirectoryTable');
        if (!empty($current_entry))
        {
            $entries_custom->load(array('entry_id'=>$entry_custom_data->entry_id));
        }
        $entries_custom->save($entry_custom_data);

        // Process credits.
        if ( !empty($current_entry))
        {
            // Remove existing unpaid credits objects.
            $query = $db->getQuery(true)
                   ->delete( $db->qn('#__rsdirectory_entries_credits') )
                   ->where( $db->qn('entry_id') . ' = ' . $db->q($current_entry->id) . ' AND ' . $db->qn('paid') . ' = ' . $db->q(0) );
               
            $db->setQuery($query);
            $db->execute();

            // Get the difference between the existing credits objects and the new credits objects.
            $entry_credits_objects = RSDirectoryCredits::getEntryCreditsDiff($entry_credits_objects, $current_entry->id);
        }

        // Clean the entry credits objects array.
        $entry_credits_objects = RSDirectoryCredits::cleanEntryCredits($entry_credits_objects);
        
        // Calculate the total credits count.
        $total = RSDirectoryCredits::calculateCredits($entry_credits_objects);

        // Check if the user has sufficient credits.
        $sufficient_credits = RSDirectoryCredits::checkUserCredits($total,$user_id);
			
		// Verify if we can mark the entry as paid.
		$paid = ($can_moderate && !$charge_credits) ||
				($can_moderate && $charge_credits && $sufficient_credits) ||
				( empty($current_entry) && $sufficient_credits ) ||
				( $sufficient_credits && !empty($data['finalize']) ) ||
				( !empty($current_entry->paid) && $sufficient_credits );
				
		
        // Insert the entries credits objects into the database.
        if ( !empty($entry_credits_objects) )
        {
            RSDirectoryCredits::addEntryCreditsObjects(
                $entry_id,
                $entry_credits_objects,
                $user_id,
                ($can_moderate && !$charge_credits) ? 1 : 0, // Grant the author free access to the filled in fields.
                $paid ? 1 : 0 // Mark the fields as paid.
            );
        }
            
        // Set publishing data.
        $entry_data = (object)array(
            'id' => $entry_id,
            'paid' => $paid ? 1 : 0,
        );
        
        // Set the publishing status.
        if ($can_moderate)
        {
            $published = (($data['published'] && $paid) ? 1 : 0);
			
			if (!$published && $data['published']) {
				if (isset($data['charge'])) {
					$app->enqueueMessage( JText::_('COM_RSDIRECTORY_INSUFFICIENT_USER_CREDITS_FOR_PUBLISH_WITH_CHARGE'), 'info' );
				} 
				else
				{
					$app->enqueueMessage( JText::_('COM_RSDIRECTORY_INSUFFICIENT_USER_CREDITS_FOR_PUBLISH'), 'info' );
				}
			}
        }
        else
        {
            $published = $paid && RSDirectoryHelper::checkUserPermission('auto_publish_entries') ? 1 : 0;
        }

			
        // Reset the published_time and expiry_time if the entry is UNPUBLISHED.
        if (!$published)
        {
            $entry_data->published_time = '0000-00-00 00:00:00';
        }
            
        if ($operation == 'insert')
        {
            $entry_data->new = (int) !$published;
                
            if ($published)
            {
                // log the published state
                $log_params = array(
                    'type' => 'entry_published',
                    'entry_id' => $entry_data->id,
                    'entry_name' => $entry_title_log
                );
                RSDirectoryHelper::logAction($log_params);

                if ( !$can_moderate || !trim($data['published_time']) || $data['published_time'] == '0000-00-00 00:00:00' )
                {
                    $entry_data->published_time = JFactory::getDate()->toSql();
                }
                else
                {
                    $entry_data->published_time = JFactory::getDate($data['published_time'])->toSql();
                }

                // handle the promoted expiry date if promoted_period is not set to unlimited
                if ($promoted_period !== false) {
                    if (!$overwrite_promoted_expiry && !empty($promoted_period)) {
                        $promoted_expiry = JFactory::getDate($entry_data->published_time);
                        $promoted_expiry->modify("+$promoted_period days");
                        $entry_data->promoted_expiry = $promoted_expiry->toSql();
                    } else if($overwrite_promoted_expiry){
                        $entry_data->promoted_expiry = $overwrite_promoted_expiry;
                    }
                }

                    
                // If the expiry time value is specified, calculate the period from that.
                if ( $can_moderate && isset($data['expiry_time']) && $data['expiry_time'] != '0000-00-00 00:00:00' )
                {
                    $period = RSDirectoryHelper::getDateDiff( $entry_data->published_time, JFactory::getDate($data['expiry_time'])->toSql(), 86400 );    
                }
                    
                if ($period)
                {
                    $expiry_time = JFactory::getDate($entry_data->published_time);
                    $expiry_time->modify("+$period days");
                        
                    $entry_data->expiry_time = $expiry_time->toSql();
                }
                else
                {
                    $entry_data->expiry_time = '0000-00-00 00:00:00';
                }
                
				if (!$import) {	
					$emails[] = 'publish_entry';
				}
            } else {
                if ($promoted_period !== false && $overwrite_promoted_expiry) {
                    $entry_data->promoted_expiry = $overwrite_promoted_expiry;
                }
            }

            if ($entry_data->new && !$published && !$can_moderate)
			{
                $entry_data->activation_code = JApplicationHelper::getHash(JUserHelper::genRandomPassword());
            }
        }
        else
        {
            if ($published && !$current_entry->published)
            {
                $emails[] = 'publish_entry';
            }
            else if (!$published && $current_entry->published)
            {
                $emails[] = 'unpublish_entry';
            }

            // if the user does not have permission then we must not change the promoted option if already running
            $update_promoted = true;

            if (!empty($current_entry) && !$can_moderate && $promoted_period && !empty($data[$form_field->form_field_name])) {
                // promoted with limit
                if (!$published || ($promoted_comparison_date && $promoted_comparison_date >= JFactory::getDate()->toUnix())) {
                    $update_promoted = false;
                }
            }


            if ($published)
            {
                $entry_data->new = 0;
					
                if ($can_moderate)
                {
					$change_if_publish_manual = false;
                    if ( !trim($data['published_time']) || $data['published_time'] == '0000-00-00 00:00:00' )
                    {
                        $published_time = JFactory::getDate()->toSql();
                    }
                    else
                    {
                        $published_time = JFactory::getDate($data['published_time'])->toSql();
						$change_if_publish_manual = true;
                    }
                        
                    // Set a new published time if the status changed from unpublished to published, if the period change or if the published time changed.
                    if ( !$current_entry->published || $period != $current_entry->period || $published_time != $current_entry->published_time )
                    {
                        $entry_data->published_time = $published_time;
                        $update_expiry_time = true;
						
						// do not accept changing the expiry date if the publishing date is not changed manually and if the entry is unpublished and not expired and the selected period is the same (the entry is changed and by the owner and requires moderation)
						if (!$current_entry->published && $period == $current_entry->period && !$expired && !$change_if_publish_manual)
                        {
                            $update_expiry_time = false;
                        }
                    }
                }
                // Set a new published time if the entry is auto-published and the user selected a new publishing period.
				// OR If the entry expired and we're republishing it
				// OR if the entry was unpublished and we're publishing it
                else if ($period != $current_entry->period || $expired || !$current_entry->published)
                {
                    $entry_data->published_time = JFactory::getDate()->toSql();
                    $update_expiry_time = true;
                }
            }
                
            // Set a new expiry time if the published time changed.
            if ( !empty($update_expiry_time) )
            {
                // handle the promoted expiry date if promoted_period is not set to unlimited
                if ($promoted_period !== false) {
                    if (!$overwrite_promoted_expiry && !empty($promoted_period)) {
                        $promoted_expiry = JFactory::getDate($entry_data->published_time);
                        $promoted_expiry->modify("+$promoted_period days");
                        $entry_data->promoted_expiry = $promoted_expiry->toSql();
                    } else if($overwrite_promoted_expiry){
                        $entry_data->promoted_expiry = $overwrite_promoted_expiry;
                    }
                }

                // If the expiry time value is specified, calculate the period from that.
                if ( $can_moderate && isset($data['expiry_time']) && $data['expiry_time'] != '0000-00-00 00:00:00' )
                {
                    $period = RSDirectoryHelper::getDateDiff( $entry_data->published_time, JFactory::getDate($data['expiry_time'])->toSql(), 86400 );    
                }
                    
                if ($period)
                {
                    $expiry_time = JFactory::getDate($entry_data->published_time);
                    $expiry_time->modify("+$period days");
                        
                    $entry_data->expiry_time = $expiry_time->toSql();
                }
                else
                {
                    $entry_data->expiry_time = '0000-00-00 00:00:00';
                }
            } else {
                // handle the promoted expiry date if promoted_period is not set to unlimited
                if ($promoted_period !== false && $update_promoted) {
                    if (!$overwrite_promoted_expiry && !empty($promoted_period)) {
                        $start_promoted =  JFactory::getDate()->toSql();
                        if ($current_entry->published_time != '0000-00-00 00:00:00') {
                            $current_diff = RSDirectoryHelper::getDateDiff($start_promoted, $current_entry->published_time);
                            if ($current_diff < 0) {
                                $start_promoted = $current_entry->published_time;
                            }
                        }
                        $promoted_expiry = JFactory::getDate($start_promoted);
                        $promoted_expiry->modify("+$promoted_period days");
                        $entry_data->promoted_expiry = $promoted_expiry->toSql();
                    } else if($overwrite_promoted_expiry) {
                        $entry_data->promoted_expiry = $overwrite_promoted_expiry;
                    }
                }
            }

            if (!$published && $promoted_period !== false && $overwrite_promoted_expiry) {
                $entry_data->promoted_expiry = $overwrite_promoted_expiry;
            }
        }

        // reset the promoted_expiry datetime if promoted not selected
        if ($promoted_period === false) {
            $entry_data->promoted_expiry = '0000-00-00 00:00:00';
        }


        if ($can_moderate)
        {
            // Mark the entry and entry credits as paid if we are saving the entry as published or as paid.
            if ($published || !empty($data['paid']) )
            {
                $entry_data->paid = 1;
            }
            // Mark the entry and entry credits an unpaid if we are saving the entry as unpaid.
            else if ( empty($data['paid']) )
            {
                $entry_data->paid = 0;
            }
                
            $query = $db->getQuery(true)
                   ->update( $db->qn('#__rsdirectory_entries_credits') )
                   ->set( $db->qn('paid') . ' = ' . $db->q($entry_data->paid) )
                   ->where( $db->qn('entry_id') . ' = ' . $db->q($entry_id) );
                 
            $db->setQuery($query);
            $db->execute();
        }
			
		$entry_data->published = $published;

        // Log the action published/upublished
        $log_params = array(
            'type'          => ($published ? 'entry_published' : 'entry_unpublished'),
            'entry_id'      => $entry_id,
            'entry_name'    => $entry_title_log
        );
        RSDirectoryHelper::logAction($log_params);


        $log_paid = $operation == 'insert' ? true : $current_entry->paid != $paid;

        if ($log_paid)
        {
            // Log the paid status for the entry
            $log_params = array(
                'type'          => ($paid ? 'entry_paid' : 'entry_unpaid'),
                'entry_id'      => $entry_id,
                'entry_name'    => $entry_title_log
            );
            RSDirectoryHelper::logAction($log_params);
        }

        // Save publishing data.
        $db->updateObject('#__rsdirectory_entries', $entry_data, 'id');
            
            
        // Autogenerate title/big subtitle/small subtitle and update entry.
        if ($form->use_title_template || $form->use_big_subtitle_template || $form->use_small_subtitle_template || $form->use_description_template || !empty($form->metadata_title) || !empty($form->metadata_keywords) || !empty($form->metadata_description))
        {
            require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/placeholders.php';
                
            // Get a fresh copy of the entry.
            $entry = RSDirectoryHelper::getEntry($entry_id);
                
            $values = (object)array(
                'id' => $entry_id,
            );

            $changed = array();
                
            if ($form->use_title_template)
            {
                $values->title = RSDirectoryPlaceholders::getInstance($form->title_template, $form_fields, $entry, $form)
                               ->setParams( array('title', 'database') )
                               ->process();
                $changed[] = 'title';
            }
                
            if ($form->use_big_subtitle_template)
            {
                $values->big_subtitle = RSDirectoryPlaceholders::getInstance($form->big_subtitle_template, $form_fields, $entry, $form)
                                      ->setParams( array('big-subtitle', 'database') )
                                      ->process();
                $changed[] = 'big_subtitle';
            }
                
            if ($form->use_small_subtitle_template)
            {
                $values->small_subtitle = RSDirectoryPlaceholders::getInstance($form->small_subtitle_template, $form_fields, $entry, $form)
                                        ->setParams( array('small-subtitle', 'database') )
                                        ->process();
                $changed[] = 'small_subtitle';
            }
			
                
            if ($form->use_description_template)
            {
                $values->description = RSDirectoryPlaceholders::getInstance($form->description_template, $form_fields, $entry, $form)
                                     ->setParams( array('description', 'database') )
                                     ->process();
                $changed[] = 'description';
            }

            // change the entry values if something above has changed
            if (!empty($changed))
            {
                foreach ($changed as $col) {
                    $entry->{$col} = $values->{$col};
                }
            }

            if (!empty($form->metadata_title))
            {

                $values->metadata_title = RSDirectoryPlaceholders::getInstance($form->metadata_title, $form_fields, $entry, $form)
                    ->setParams(array('metadata_title', 'database'))
                    ->process();
            }

            if (!empty($form->metadata_keywords))
            {
                $values->metadata_keywords = RSDirectoryPlaceholders::getInstance($form->metadata_keywords, $form_fields, $entry, $form)
                    ->setParams(array('metadata_keywords', 'database'))
                    ->process();
            }

            if (!empty($form->metadata_description))
            {
                $values->metadata_description = RSDirectoryPlaceholders::getInstance($form->metadata_description, $form_fields, $entry, $form)
                    ->setParams(array('metadata_description', 'database'))
                    ->process();
            }
                
            $db->updateObject('#__rsdirectory_entries', $values, 'id');
        }
            
        if ( !empty($published) && ( $operation == 'insert' || !empty($current_entry->new) ) )
        {
            $table = $this->getTable();
            $table->jomSocialActivity($entry_id);
        }
            
        // Send emails.
        if ($emails)
        {
            require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/email.php';
                
            // Get a fresh copy of the entry.
            $entry = RSDirectoryHelper::getEntry($entry_id);
                
            foreach ($emails as $email)
            {
                RSDirectoryEmail::getInstance($email, $form_fields, $entry, $form)->send();
            }
        }
            
        return true;
    }

    /**
     * Method to auto-populate the model state. 
     * 
     * @access protected
     */
    protected function populateState()
    {
        // Get mainframe.
        $app = JFactory::getApplication();
            
        // Get the pk of the record from the request.
        $pk = $app->input->getInt('id');
        $this->setState( $this->getName() . '.id', $pk );
            
        if ( $app->isClient('site') )
        {
            $app = JFactory::getApplication('site');
                
            $params = $app->getParams();
            $this->setState('params', $params);
        }
    }
        
    /**
     * Method to mark an entry as paid/unpaid.
     *
     * @access public
     *
     * @param mixed $cid
     * @param bool $paid
     *
     * @return bool
     */
    public function markAsPaid($cid, $paid)
    {
        if (!$cid)
            return false;
			
		if ( !is_array($cid) )
		{
			$cid = (array)$cid;
		}
			
		// Sanitize input.
		$cid = RSDirectoryHelper::arrayInt($cid);
			
		if (!$cid)
			return false;
		
		// Get Dbo.
        $db = JFactory::getDbo();
			
		$in_values = array();
			
		foreach ($cid as $id)
		{
            $in_values[] = $db->q($id);
		}
			
		$in_values = implode(',', $in_values);;
			
        $query = $db->getQuery(true)
               ->update( $db->qn('#__rsdirectory_entries') )
               ->set( $db->qn('paid') . ' = ' . $db->q($paid ? 1 : 0) )
               ->where( $db->qn('id') . ' IN (' . $in_values . ')' );
               
        $db->setQuery($query);
        $db->execute();
            
        $query = $db->getQuery(true)
               ->update( $db->qn('#__rsdirectory_entries_credits') )
               ->set( $db->qn('paid') . ' = ' . $db->q($paid ? 1 : 0) )
               ->where( $db->qn('entry_id') . ' IN (' . $in_values . ')' );
                 
        $db->setQuery($query);
        $db->execute();

        // Log the paid/unpaid status for the selected entries
        $entries = RSDirectoryHelper::getEntriesObjectListByIds($cid);
        foreach ($entries as $entry)
        {
            if (!empty($entry)) {
                $log_params = array(
                    'type' => ($paid ? 'entry_paid' : 'entry_unpaid'),
                    'entry_id' => $entry->id,
                    'entry_name' => $entry->title
                );
                RSDirectoryHelper::logAction($log_params);
            }
        }

		if (!$paid)
		{
			$this->publish($cid, 0);
		}
			
        return true;
    }
		
	/**
	 * Method to finalize an entry.
	 *
	 * @access public
	 *
	 * @param int $id
	 *
	 * @return bool
	 */
	public function finalize($id)
	{
		// Get Dbo.
		$db = JFactory::getDbo();
			
		// Get user object.
		$user = JFactory::getUser();
			
		// Get persmissions.
		$can_edit_all_entries = RSDirectoryHelper::checkUserPermission('can_edit_all_entries');
			
		if ( !$can_edit_all_entries && RSDirectoryCredits::getUserCredits() != 'unlimited' )
		{
			// Get the number of unpaid entry credits.
			$cost = RSDirectoryCredits::getUnpaidEntryCreditsSum($id);
				
			// Charge user.
			$query = $db->getQuery(true)
			       ->update( $db->qn('#__rsdirectory_users') )
				   ->set( $db->qn('credits') . ' = ' . $db->qn('credits') . ' - ' . $db->q($cost) )
				   ->where( $db->qn('user_id') . ' = ' . $db->q($user->id) );
				   
			$db->setQuery($query);
			$db->execute();
		}
			
		$this->markAsPaid($id, 1);
			
		// Publish the entry if the user has the right permission.
		if ( RSDirectoryHelper::checkUserPermission('auto_publish_entries') )
		{
			$this->getTable()->publish($id, 1);
		}
			
		return true;
	}

    public function activate($entry_id, $code)
	{
        $app 	= JFactory::getApplication();
        $db  	= JFactory::getDbo();
        $entry 	= RSDirectoryHelper::getEntry($entry_id);
		
        // Check if the entry exists
        if (is_null($entry))
		{
            throw new Exception(JText::_('COM_RSDIRECTORY_ACTIVATION_ERROR_ENTRY_NOT_EXISTS'));
        }

        if ($entry->published)
		{
            // remove the activation code if it's still defined
            $query = $db->getQuery(true)
				->update('#__rsdirectory_entries')
                ->set($db->qn('activation_code')." = ''")
                ->where($db->qn('id').'='.$db->q($entry->id));
            $db->setQuery($query)
				->execute();
			
			throw new Exception(JText::_('COM_RSDIRECTORY_ACTIVATION_ERROR_ENTRY_ALREADY_PUBLISHED'));
        }

        // The entry must be new
        if (!$entry->new)
		{
            throw new Exception(JText::_('COM_RSDIRECTORY_ACTIVATION_ERROR_ENTRY_NOT_NEW'));
        }

        if (empty($entry->activation_code))
		{
            throw new Exception(JText::_('COM_RSDIRECTORY_ACTIVATION_ERROR_NO_ACTIVATION_CODE'));
        }

        // Check if the activation code is correct
        if ($entry->activation_code != $code)
		{
            throw new Exception(JText::_('COM_RSDIRECTORY_ACTIVATION_ERROR_ACTIVATION_CODE_NOT_CORRESPONDING'));
        }

        // Check if the entry is paid
        if (!$entry->paid)
		{
            throw new Exception(JText::_('COM_RSDIRECTORY_ACTIVATION_ERROR_ENTRY_NOT_PAID'));
        }

        JTable::addIncludePath(JPATH_ADMINISTRATOR.'/components/com_rsdirectory/tables/');
        $table = JTable::getInstance('Entry', 'RSDirectoryTable');
        if ($table->publish($entry->id, 1))
		{
			$app->enqueueMessage(JText::_('COM_RSDIRECTORY_ACTIVATION_SUCCESS'));
			$app->redirect(RSDirectoryRoute::getEntryURL($entry_id, $entry->title, 'default', 0, false, false));
		}
    }
}