<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * Fields model.
 */
class RSDirectoryModelFields extends JModelList
{
    /**
     * Class constructor.
     *
     * @access public
     * 
     * @param array $config
     */
    public function __construct( $config = array() )
    {
        if ( empty($config['filter_fields']) )
        {
            $config['filter_fields'] = array(
                'ordering',
                'published',
                'form_id',
                'field_type',
                'type',
                'core_field',
                'required',
                'searchable_simple',
                'searchable_advanced',
                'id',
                'name',
            );
        }
            
        parent::__construct($config);
    }
        
    /**
     * Method to get a JDatabaseQuery object for retrieving the data set from a database.
     *
     * @access protected
     *
     * @return object A JDatabaseQuery object to retrieve the data set.
     */
    protected function getListQuery()
    {
        // Get DBO.
        $db = JFactory::getDBO();
            
        // Get filtering states.
        $form_id = $this->getState('filter.form_id');
            
        $select = array(
            $db->qn('f') . '.*',
            $db->qn('ft.core'),
            $db->qn('ft.type'),
            $db->qn('fpc.value', 'credits'),
            $db->qn('fpss.value', 'searchable_simple'),
            $db->qn('fpsa.value', 'searchable_advanced'),
        );
            
        if ($form_id)
        {
            $select[] = $db->qn('ff.ordering');
        }
            
        $query = $db->getQuery(true)
               ->select($select)
               ->from( $db->qn('#__rsdirectory_fields', 'f') )
               ->innerJoin( $db->qn('#__rsdirectory_field_types', 'ft') . ' ON ' . $db->qn('f.field_type_id') . ' = ' . $db->qn('ft.id') )
               ->leftJoin( $db->qn('#__rsdirectory_fields_properties', 'fpc') . ' ON ' . $db->qn('f.id') . ' = ' . $db->qn('fpc.field_id') . ' AND ' . $db->qn('fpc.property_name') . ' = ' . $db->q('credits') )
               ->leftJoin( $db->qn('#__rsdirectory_fields_properties', 'fpss') . ' ON ' . $db->qn('f.id') . ' = ' . $db->qn('fpss.field_id') . ' AND ' . $db->qn('fpss.property_name') . ' = ' . $db->q('searchable_simple') )
               ->leftJoin( $db->qn('#__rsdirectory_fields_properties', 'fpsa') . ' ON ' . $db->qn('f.id') . ' = ' . $db->qn('fpsa.field_id') . ' AND ' . $db->qn('fpsa.property_name') . ' = ' . $db->q('searchable_advanced') );
                
        // Filter by published state.  
        $published = $this->getState('filter.published');
            
        if ($published !== '')
        {
            $query->where( $db->qn('f.published') . ' = ' . $db->q($published) );
        }
            
        // Filter by form.
        if ($form_id)
        {
            $query->innerJoin( $db->qn('#__rsdirectory_forms_fields', 'ff') . ' ON ' . $db->qn('f.id') . ' = ' . $db->qn('ff.field_id') )
                  ->where( $db->qn('ff.form_id') . ' = ' . $db->q($form_id) );
        }
            
        // Filter by field type.  
        if ( $field_type = $this->getState('filter.field_type') )
        {
            $query->where( $db->qn('ft.type') . ' = ' . $db->q($field_type) );
        }
            
        // Filter by core field.
        $core_field = $this->getState('filter.core_field');
            
        if ($core_field !== '')
        {
            $query->where( $db->qn('ft.core') . ' = ' . $db->q($core_field) );
        }
            
        // Is required?
        $required = $this->getState('filter.required');
            
        if ($required !== '')
        {
            $query->where( $db->qn('f.required') . ' = ' . $db->q($required) );
        }
            
        // Filter by searchable simple.
        $searchable_simple = $this->getState('filter.searchable_simple');
            
        if ($searchable_simple !== '')
        {
            if ($searchable_simple == -1)
            {
                $query->where( $db->qn('fpss.value') . ' IS NULL' );    
            }
            else
            {
                $query->where( $db->qn('fpss.value') . ' = ' . $db->q($searchable_simple) );    
            }
        }
            
        // Filter by searchable advanced.
        $searchable_advanced = $this->getState('filter.searchable_advanced');
            
        if ($searchable_advanced !== '')
        {
            if ($searchable_advanced == -1)
            {
                $query->where( $db->qn('fpsa.value') . ' IS NULL' );
            }
            else if ($searchable_advanced == 0)
            {
                $query->where( '(' . $db->qn('fpsa.value') . ' = ' . $db->q(0) . ' OR ' . $db->qn('fpsa.value') . ' = ' . $db->q('') . ')' );
            }
            else
            {
                $query->where( $db->qn('fpsa.value') . ' = ' . $db->q($searchable_advanced) );
            }
        }
            
        // Filter by search.
        $search = $this->getState('filter.search');
            
        if ($search)
        {
            $search = $db->q( '%' . str_replace( ' ', '%', $db->escape($search, true) ) . '%', false );
                
            $query->where( $db->qn('f.name') . " LIKE $search" );
        }
            
        // If displaying the modal view, limit the results to the following field types.
        if ( JFactory::getApplication()->input->get('layout') == 'modal' )
        {
            $modal_field_types = array(
                $db->q('dropdown'),
                $db->q('checkboxgroup'),
                $db->q('radiogroup'),
                $db->q('country'),
            );
                
            $query->where( $db->qn('ft.type') . ' IN (' . implode(',', $modal_field_types) . ')' );
			
			$query->select($db->qn('fpd.value', 'dependency'));
			$query->leftJoin( $db->qn('#__rsdirectory_fields_properties', 'fpd') . ' ON ' . $db->qn('f.id') . ' = ' . $db->qn('fpd.field_id') . ' AND ' . $db->qn('fpd.property_name') . ' = ' . $db->q('dependency') );
        }
            
        $ordering = $this->getState('list.ordering');
        $direction = $this->getState('list.direction', 'asc');

        // in Joomla 4 we cannot prefix this ordering in the xml or view as it will not be properly used
        switch($ordering)
        {
            case 'name':
            case 'published':
            case 'id':
            case 'required':
                $ordering = 'f.'.$ordering;
            break;
            case 'type':
            case 'core':
                $ordering = 'ft.'.$ordering;
            break;

            case 'ordering':
                $ordering = 'ff.'.$ordering;
            break;
        }
            
        $query->order(
            ( $ordering == 'credits' ? 'CAST(' . $db->qn($ordering) . ' AS UNSIGNED)' : $db->qn($ordering) ) . ' ' . $db->escape($direction)
        );
		
        return $query;
    }
        
    /**
     * Method to auto-populate the model state. 
     * 
     * @access protected
     * 
     * @param string $ordering
     * @param string $direction
     */
    protected function populateState($ordering = null, $direction = null)
    {
        // Adjust the context to support modal layouts.
        if ( $layout = JFactory::getApplication()->input->get('layout') )
        {
            $this->context .= ".$layout";
        }

        $search = $this->getUserStateFromRequest("$this->context.filter.search", 'filter_search');
        $this->setState('filter.search', $search);

        $published = $this->getUserStateFromRequest("$this->context.filter.published", 'filter_published', '');
        $this->setState('filter.published', $published);

        $form = $this->getUserStateFromRequest("$this->context.filter.form_id", 'filter_form_id');
        $this->setState('filter.form_id', $form);

        $field_type = $this->getUserStateFromRequest("$this->context.filter.field_type", 'filter_field_type');
        $this->setState('filter.field_type', $field_type);

        $core_field = $this->getUserStateFromRequest("$this->context.filter.core_field", 'filter_core_field', '');
        $this->setState('filter.core_field', $core_field);

        $required = $this->getUserStateFromRequest("$this->context.filter.required", 'filter_required', '');
        $this->setState('filter.required', $required);

        $searchable_simple = $this->getUserStateFromRequest("$this->context.filter.searchable_simple", 'filter_searchable_simple', '');
        $this->setState('filter.searchable_simple', $searchable_simple);

        $searchable_advanced = $this->getUserStateFromRequest("$this->context.filter.searchable_advanced", 'filter_searchable_advanced', '');
        $this->setState('filter.searchable_advanced', $searchable_advanced);

        // List state information.
        parent::populateState('id', 'asc');
    }
}