<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * Category table.
 */
class RSDirectoryTableCategory extends JTableCategory
{
    /**
     * Method to delete a node and, optionally, its child nodes from the table.
     *
     * @access public
     *
     * @param int $pk The primary key of the node to delete.
     * @param bool $children  True to delete child nodes, false to move them up a level.
     *
     * @return boolean True on success.
     */
    public function delete($pk = null, $children = false)
    {
        static $categories;
        $this->removeImage($pk);

        $has_children = false;

        if ($children)
        {
            if (is_null($categories))
            {
                // need to find if the category has children for proper logging
                $options = array(
                    'extension'  => 'com_rsdirectory',
                    'table'      => '#__rsdirectory_entries',
                    'field'      => 'category_id',
                    'statefield' => 'published',
                    'published'  => 0
                );
                $categories = new JCategories($options);
            }

            $selected_category = $categories->get($pk);
            $has_children = $selected_category->hasChildren();
        }

        $return = parent::delete($pk, $children);

        if ($return) {
            $log_params = array(
                'type' => 'category_delete',
                'category_title' => $this->title,
                'has_children' => ($children && $has_children)
            );
            RSDirectoryHelper::logAction($log_params);
        }

        return $return;
    }
    /**
     * Method to store a row in the database from the JTable instance properties.
     *
     * @param boolean $updateNulls True to update fields even if they are null.
     *
     * @return boolean True on success.
     */
    public function store($updateNulls = false)
    {
        // Get the name of the primary key.
        $k = $this->_tbl_key;

        // check to see if there is an id
        $id = $this->$k;

        if ($id)
        {
            $category = JTable::getInstance('Category', 'RSDirectoryTable');
            $category->load($id);

            $old_published_state = $category->published;
            $old_params = json_decode($category->params);
        }

        $return = parent::store($updateNulls);
        if ($return)
        {
            $state_changed = !$id ? true : false;
            $form_assignation_changed = $state_changed;

            // check to see if the published state has been changed
            if ($id && $old_published_state != $this->published)
            {
                $state_changed = true;
            }

            $current_params = json_decode($this->params);
            if ($id)
            {
                if ($current_params->form_id != $old_params->form_id)
                {
                    $form_assignation_changed = true;
                }
            }

            $new_id = $this->$k;
            // log the create/edit for the form
            $log_params = array(
                'type'           => 'category_' . (! $id ? 'create' : 'update'),
                'category_id'    => $new_id,
                'category_title' => $this->title
            );

            RSDirectoryHelper::logAction($log_params);

            // log the published state if its changed
            if ($state_changed)
            {
                // log the create/edit for the form
                $log_params = array(
                    'type'       => 'category_' . (!$this->published ? 'unpublished' : 'published'),
                    'category_id' => $new_id,
                    'category_title' => $this->title
                );

                RSDirectoryHelper::logAction($log_params);
            }

            // log the form assignation if its changed
            if ($form_assignation_changed)
            {
                // convert the current form_id from string to int
                $current_params->form_id = (int) $current_params->form_id;

                $form = JTable::getInstance('Form', 'RSDirectoryTable');
                if (!empty($current_params->form_id))
                {
                    $form->load($current_params->form_id);
                }
                else if (empty($current_params->form_id) && !empty($this->parent_id))
                {
                    // see if there is a for inherited from the parent category
                    $form_id = $this->findInheritedFormStatic($this->parent_id);
                    $form->load($form_id);
                }

                // log the create/edit for the form
                $log_params = array(
                    'type'           => 'category_form_assignation',
                    'category_id'    => $new_id,
                    'category_title' => $this->title,
                    'form_id'        => $current_params->form_id,
                    'form_title'      => ($form ? $form->title : '')
                );

                RSDirectoryHelper::logAction($log_params);
            }
        }

        return $return;
    }

    protected function findInheritedFormStatic($cat_id) {
        static $form_ids = array();
        $cat_id = (int) $cat_id;

        if (!isset($form_ids[$cat_id])) {
            $form_ids[$cat_id] = $this->findInheritedForm($cat_id);
        }

        return $form_ids[$cat_id];
    }

    protected function findInheritedForm($cat_id) {
        static $categories;
        $cat_id = (int) $cat_id;
		
		// skip the root for no parent categories
        if ($cat_id == 1) {
            return 0;
        }

        if (is_null($categories)) {
            $options = array(
               'extension' => 'com_rsdirectory',
               'table' => '#__rsdirectory_entries',
               'field' => 'category_id',
               'statefield' => 'published',
               'published' => 0
            );
            $categories = new JCategories($options);
        }
        $category = $categories->get($cat_id);
        $params = $category->getParams();
        $form_id = (int)$params->get('form_id');
        $parent_id = (int)$category->get('parent_id');

        if (!empty($form_id))
        {
            return $form_id;
        }

        if ($parent_id > 1) {
            return $this->findInheritedForm($parent_id);
        }

        return 0;
    }

    /**
     * Method to set the publishing state for a row or list of rows in the database table.
     *
     * @param mixed $pks
     * @param integer $state
     * @param integer $userId
     *
     * @return boolean
     */
    public function publish($pks = null, $state = 1, $userId = 0)
    {
        $k = $this->_tbl_key;

        // Sanitize input.
        $pks = array_map('intval', $pks);

        // If there are no primary keys set check to see if the instance key is set.
        if ( empty($pks) )
        {
            if (!$this->$k)
                return false;

            $pks = array($this->$k);
        }

        $return = parent::publish($pks, $state, $userId);

        if ($return) {
            foreach  ($pks as $pk) {
                $category = JTable::getInstance('Category', 'RSDirectoryTable');
                $category->load($pk);

                if ($category) {
                    $log_params = array(
                        'type' => 'category_' . (!$state ? 'unpublished' : 'published'),
                        'category_id' => $pk,
                        'category_title' => $category->title
                    );
                    RSDirectoryHelper::logAction($log_params);
                }
            }
        }

        return $return;

    }
        
    /**
     * Remove category image.
     *
     * @access public
     *
     * @param int $pk The category id.
     *
     * @return bool
     */
    public function removeImage($pk = null, $map_icon = false)
    {
        $k = $this->_tbl_key;
        $pk = is_null($pk) ? $this->$k : $pk;
            
        $this->load($pk);
            
        // Get params.
        $params = new JRegistry($this->params);
            
        // Convert the params to an array.
        $params = $params->toArray();
            
        // Get the file id.
        if ($map_icon) {
            $file_id = $params['map_icon'];
        } else {
            $file_id = $params['thumbnail'];
        }

        if ($file_id)
        {
            // Unset the thumbnail value.
            unset($params['thumbnail']);
                
            // Convert the params array to a JRegistry.
            $params = new JRegistry($params);
                
            // Set the new params value for the category.
            $this->params = $params->toString();
                
            // Save the new values.
            $this->store();
                
            // Get the file data.
            if ($map_icon) {
                $file = RSDirectoryHelper::getCategoryMapIconObject($file_id);
            } else {
                $file = RSDirectoryHelper::getCategoryThumbObject($file_id);
            }
                
            // Get DBO.
            $db = JFactory::getDbo();
                
            $query = $db->getQuery(true)
                   ->delete( $db->qn('#__rsdirectory_uploaded_files') )
                   ->where( $db->qn('id') . ' = ' . $db->q($file_id) );
                   
            $db->setQuery($query);
            $db->execute();

            if ($map_icon) {
                $query = $db->getQuery(true)
                    ->delete( $db->qn('#__rsdirectory_uploaded_files_cat_mapicons_rel') )
                    ->where( $db->qn('file_id') . ' = ' . $db->q($file_id) );

                $db->setQuery($query);
                $db->execute();
            } else {
                $query = $db->getQuery(true)
                    ->delete( $db->qn('#__rsdirectory_uploaded_files_categories_relations') )
                    ->where( $db->qn('file_id') . ' = ' . $db->q($file_id) );

                $db->setQuery($query);
                $db->execute();
            }

                
            return JFile::delete(JPATH_ROOT . "/components/com_rsdirectory/files/images/categories/$file->file_name");
        }
            
        return false;
    }
}