<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * Review table.
 */
class RSDirectoryTableReview extends JTable
{
    public $review = '';
    public $owner_reply = '';

    /**
     * The class constructor.
     *
     * @access public
     * 
     * @param object Database connector object
     */
    public function __construct(&$db)
    {
        parent::__construct('#__rsdirectory_reviews', 'id', $db);
    }
        
    /**
     * Method to store a row in the database from the JTable instance properties.
     *
     * @param bool $updateNulls 
     *
     * @return bool True on success.
     */
    public function store($updateNulls = false)
    {
        // need this for logging
        $store_type = !empty($this->id) ? 'update': 'add';

        if ( empty($this->id) )
        {
            $this->ip = RSDirectoryHelper::getIp(true);
            $this->created_time = JFactory::getDate()->toSql();
        }
        else
        {
            // Get the old review.
            $old_review = JTable::getInstance('Review', 'RSDirectoryTable');
            $old_review->load($this->id);    
        }
            
        $return = parent::store($updateNulls);
            
        $is_admin = JFactory::getApplication()->isClient('administrator');
            
        // Update the entry rating if ratings are enabled.
        if ( $is_admin || RSDirectoryConfig::getInstance()->get('enable_ratings') )
        {
            $this->updateEntryRating();
        }
            
        // Update the entry rating if the entry was changed for the current review.
        if ( $is_admin && !empty($old_review) && $old_review->entry_id != $this->entry_id )
        {
            $this->updateEntryRating($old_review->entry_id);
        }


        $entry_id = ($store_type == 'update') ? $old_review->entry_id: $this->entry_id;

        if ($entry = RSDirectoryHelper::getEntry($entry_id)) {
            // log the review add / update
            $log_params = array(
                'type' => 'review_'.$store_type,
                'review_id'=> $this->id,
                'entry_id' => $entry->id,
                'entry_name' => $entry->title
            );
            RSDirectoryHelper::logAction($log_params);

            // log the owner reply
            if ($store_type == 'update' && md5($this->owner_reply) != md5($old_review->owner_reply)) {
                // try and catch the owner reply
                $log_params = array(
                    'type' => 'review_add_owner_reply',
                    'review_id'=> $this->id,
                    'entry_id' => $entry->id,
                    'entry_name' => $entry->title
                );
                RSDirectoryHelper::logAction($log_params);
            }

            $state_changed = $store_type == 'add' ? true : false;

            // check to see if the published state has been changed
            if ($store_type == 'update' && $old_review->published != $this->published)
            {
                $state_changed = true;
            }

            if ($state_changed) {
                // log the review state
                $log_params = array(
                    'type' => 'review_' . (!$this->published ? 'unpublished' : 'published'),
                    'review_id'=> $this->id,
                    'entry_id' => $entry->id,
                    'entry_name' => $entry->title
                );

                RSDirectoryHelper::logAction($log_params);
            }
        }
            
        return $return;
    }
        
    /**
     * Method to set the publishing state for a row or list of rows in the database table.  
     *
     * @param mixed $pks 
     * @param integer $state 
     * @param integer $userId
     *
     * @return boolean
     */
    public function publish($pks = null, $state = 1, $userId = 0)
    {

        if ( $return = parent::publish($pks, $state, $userId) )
        {

			foreach ($pks as $pk) {
				if ($pk != $this->id) {
                    $review = JTable::getInstance('Review', 'RSDirectoryTable');
                    $review->load($pk);
                    $entry_id = $review->entry_id;
				} else {
					$entry_id = null;
				}

                $this->updateEntryRating($entry_id);

                // log the rating state
                $entry_id = is_null($entry_id) ? $this->entry_id : $entry_id;
                if ($entry = RSDirectoryHelper::getEntry($entry_id)) {
                    $log_params = array(
                        'type' => ($state == 1 ? 'review_published' : 'review_unpublished'),
                        'review_id'=> $pk,
                        'entry_id' => $entry->id,
                        'entry_name' => $entry->title
                    );
                    RSDirectoryHelper::logAction($log_params);
                }
			}
        }
            
        return $return;
    }
        
    /**
     * Method to delete a row from the database table by primary key value.
     *
     * @access public
     *
     * @param mixed
     *
     * @return boolean
     */
    public function delete($pk = null)
    {

        $pks = (array) $pk;

        // Sanitize input.
        $pks = array_map('intval', $pks);


        $db = JFactory::getDbo();

        $ratings_ids = array();
        foreach ($pks as $id)
        {
            $ratings_ids[] = $db->q($id);
        }


        $query = $db->getQuery(true)
            ->select($db->qn('entry_id'))
            ->from($db->qn('#__rsdirectory_reviews'))
            ->where( $db->qn('id') . ' IN (' . implode(',', $ratings_ids) . ')' );
        $db->setQuery($query);

        //loadColumn
        if ($entries_ids =  $db->loadColumn()) {
            $entries = RSDirectoryHelper::getEntriesObjectListByIds($entries_ids);
            if ($entries)
            {
                foreach ($entries as $entry)
                {
                    // user log
                    if (!empty($entry)) {
                        $log_params = array(
                            'type' => 'review_delete',
                            'entry_id' => $entry->id,
                            'entry_name' => $entry->title
                        );
                        RSDirectoryHelper::logAction($log_params);
                    }
                }
            }
        }

        if ( $return = parent::delete($pk) )
        {
            $this->updateEntryRating();
        }
            
        return $return;
    }
        
    /**
     * Update entry rating.
     *
     * @access private
     *
     * @param int $entry_id
     *
     * @return bool
     */
    private function updateEntryRating($entry_id = null)
    {
        $entry_id = is_null($entry_id) ? $this->entry_id : $entry_id;
            
        if ( empty($entry_id) )
            return false;
            
        // Get DBO.
        $db = JFactory::getDbo();
            
        // Get the number of ratings and their sum.
        $query = $db->getQuery(true)
               ->select( 'COUNT(*) AS ' . $db->qn('count') . ', SUM(' . $db->qn('score') . ') AS ' . $db->qn('sum') )
               ->from( $db->qn('#__rsdirectory_reviews') )
               ->where( $db->qn('entry_id') . ' = ' . $db->q($entry_id) )
               ->where( $db->qn('score') . ' > ' . $db->q(0) )
               ->where( $db->qn('published') . ' = ' . $db->q(1) );
                    
        $db->setQuery($query);
            
        $data = $db->loadObject();
            
        // Update entry.
        $entry = (object)array(
            'id' => $entry_id,
            'avg_rating' => $data->sum && $data->count ? $data->sum / $data->count : 0,
            'ratings_count' => $data->count,
        );
            
        $db->updateObject('#__rsdirectory_entries', $entry, 'id');
            
        return true;
    }
}
