<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * Transaction table.
 */
class RSDirectoryTableTransaction extends JTable
{
	public $gateway_params = '';
	public $gateway_log = '';
    /**
     * The class constructor.
     *
     * @access public
     * 
     * @param object Database connector object
     */
    public function __construct(&$db)
    {
        parent::__construct('#__rsdirectory_users_transactions', 'id', $db);
    }
        
    /**
	 * Method to set the publishing state for a row or list of rows in the database
	 * table. The method respects checked out rows by other users and will attempt
	 * to checkin rows that it can after adjustments are made.
	 *
	 * @param   mixed    $pks     An optional array of primary key values to update.
	 *                            If not set the instance property value is used.
	 * @param   integer  $state   The publishing state. eg. [0 = unpublished, 1 = published]
	 * @param   integer  $userId  The user id of the user performing the operation.
	 *
	 * @return  boolean  True on success; false if $pks is empty.
	 *
	 * @link    http://docs.joomla.org/JTable/publish
	 * @since   11.1
	 */
	public function publish($pks = null, $state = 1, $userId = 0)
	{
		$k = $this->_tbl_key;
            
		// Sanitize input.
		$pks = array_map('intval', $pks);
		$state = (int)$state;

		// If there are no primary keys set check to see if the instance key is set.
		if ( empty($pks) )
		{
			if ($this->$k)
			{
				$pks = array($this->$k);
			}
			// Nothing to set publishing state on, return false.
			else
			{
				return false;
			}
		}
            
        $db = $this->_db;
			
		$where = $k . ' = ' . implode(' OR ' . $k . ' = ', $pks);
			
		$query = $db->getQuery(true)
		       ->select('*')
			   ->from($this->_tbl)
			   ->where($where);
			   
		$db->setQuery($query);
			
		$transactions = $db->loadObjectList();
			
		// Get an instance of the Entry table.
		$entry_table = JTable::getInstance('Entry', 'RSDirectoryTable');
			
		foreach ($transactions as $transaction)
		{
			$transaction_data = (object)array(
				'id' => $transaction->id,
				'status' => $state ? 'finalized' : 'pending',
				'date_finalized' => $state ? JFactory::getDate()->toSql() : '0000-00-00 00:00:00',
			);
			
            if ($state && !$transaction->finalized)
			{
				RSDirectoryCredits::addUserCredits($transaction->user_id, $transaction->credits);
				$transaction_data->finalized = 1;

				// need to load the email helper
				require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/email.php';

				$email_custom_fields = array(
					'price-credits' 		=> $transaction->price,
					'tax' 					=> $transaction->tax,
					'tax-type' 				=> $transaction->tax_type,
					'tax-value' 			=> $transaction->tax_value,
					'total' 				=> $transaction->total,
					'credit-title' 			=> $transaction->credit_title,
					'currency' 				=> $transaction->currency,
					'credits' 				=> $transaction->credits,
					'payment-method' 		=> $transaction->gateway,
					'date-purchased' 		=> $transaction->date_created,
					'gateway-order-number' 	=> $transaction->gateway_order_number,
					'gateway-order-type' 	=> $transaction->gateway_order_type,
					'date-finalized' 		=> $transaction->date_finalized,
					'status' 				=> JText::_('COM_RSDIRECTORY_TRANSACTION_STATUS_'.strtoupper($transaction_data->status))
				);
					
				if ($transaction->entry_id)
				{
					$entry = RSDirectoryHelper::getEntry($transaction->entry_id);
						
					// Proceed only if the entry is unpaid.
					if (!$entry->paid)
					{
						$unpaid_credits = RSDirectoryCredits::getUnpaidEntryCreditsSum($entry->id);
							
						// Proceed only if the user has enough credits.	
						if ( RSDirectoryCredits::checkUserCredits($unpaid_credits, $entry->user_id) )
						{
							// Mark the entry as paid.
							$query = $db->getQuery(true)
							       ->update( $db->qn('#__rsdirectory_entries') )
								   ->set( $db->qn('paid') . ' = ' . $db->q(1) )
								   ->where( $db->qn('id') . ' = ' . $db->q($entry->id) );
								   
							$db->setQuery($query);
							$db->execute($db);
								
							// Mark entry credit objects as paid.
							$query = $db->getQuery(true)
							       ->update( $db->qn('#__rsdirectory_entries_credits') )
								   ->set( $db->qn('paid') . ' = ' . $db->q(1) )
								   ->where( $db->qn('entry_id') . ' = ' . $db->q($entry->id) );
								   
							$db->setQuery($query);
							$db->execute($db);
								
							// Charge user credits.
							if ( RSDirectoryCredits::getUserCredits($entry->user_id) != 'unlimited' )
							{
								$query = $db->getQuery(true)
								       ->update( $db->qn('#__rsdirectory_users') )
									   ->set( $db->qn('credits') . ' = ' . $db->qn('credits') . ' - ' . $db->q($unpaid_credits) )
									   ->where( $db->qn('user_id') . ' = ' . $db->q($entry->user_id) );
									   
								$db->setQuery($query);
								$db->execute();
							}
								
							// Publish the entry if the user has auto-publish permission.
							if ( RSDirectoryHelper::checkUserPermission('auto_publish_entries', $entry->user_id) )
							{
								$entry_table->publish($entry->id);
							}
						}

						// After the changes for the entry we need to get the entry object again, updated
						$entry = RSDirectoryHelper::getEntry($transaction->entry_id);
					}

					// Send the transaction email
					// Get the form associated to the category.
					$form = RSDirectoryHelper::getCategoryInheritedForm($entry->category_id);

					// Get the published form fields assigned to the form.
					$form_fields = RSDirectoryHelper::getFormFields($form->id, 1, 1);

					RSDirectoryEmail::getInstance('credit_purchase_processed_finalized_with_entry', $form_fields, $entry, $form, $email_custom_fields)->send();
				} else {
					$entry = (object) array('skip_entry'=>true, 'user_id' => $transaction->user_id);
					RSDirectoryEmail::getInstance('credit_purchase_processed_finalized_no_entry', null, $entry, null, $email_custom_fields)->send();
				}
			}
				
			$db->updateObject($this->_tbl, $transaction_data, $k);

			// Log the action
			$log_params = array(
				'type'          	=> $state == 1 ? 'credits_finalized' : 'credits_pending',
				'entry_id'      	=> !empty($transaction->entry_id) ? $transaction->entry_id : '',
				'transaction_id'    => $transaction->id,
				'credit_title'		=> $transaction->credit_title
			);
			RSDirectoryHelper::logAction($log_params);
		}

            
		$this->setError('');
		return true;
	}

	/**
	 * Method to delete a row from the database table by primary key value.
	 *
	 * @access public
	 *
	 * @param mixed
	 *
	 * @return boolean
	 */
	public function delete($pk = null)
	{

		$pks = (array) $pk;

		// Sanitize input.
		$pks = array_map('intval', $pks);


		$db = JFactory::getDbo();

		$transactions_ids = array();
		foreach ($pks as $id)
		{
			$transactions_ids[] = $db->q($id);
		}


		$query = $db->getQuery(true)
			->select($db->qn('entry_id'))
			->from($db->qn('#__rsdirectory_users_transactions'))
			->where( $db->qn('id') . ' IN (' . implode(',', $transactions_ids) . ')' );
		$db->setQuery($query);

		//loadColumn
		if ($entries_ids =  $db->loadColumn())
		{
			foreach ($entries_ids as $entry_id)
			{
				if (empty($entry_id))
				{
					// user log without entry
					$log_params = array(
						'type' => 'credits_delete',
					);
					RSDirectoryHelper::logAction($log_params);

					continue;
				}

				// user log with entry
				$log_params = array(
					'type' => 'credits_delete',
					'entry_id' => $entry_id,
				);
				RSDirectoryHelper::logAction($log_params);

			}
		}

		return parent::delete($pk);
	}
}